% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %     Flags																	% %
% %																				% %
% %     The translator is constrolled using the following flags:				% %
% %																				% %
% %			1) output_language_flag(F)											% %
% %																				% %
% %				The output language												% %
% %																				% %
% %					F = tstp (default)											% %
% %					F = tptp													% %
% %					F = kif														% %
% %																				% %
% %			2) check_predicate_constant_consistency_flag(B)						% %
% %																				% %
% %             Check if predicate and constant definitions are consistency     % %
% %						regarding arities										% %
% %																				% %
% %					F = off (default)											% %
% %																				% %
% %			3) solver_flag														% %
% %																				% %
% %				Adapt output to the characteristics of one solver               % %
% %																				% %
% %					F = vampire (default)										% %
% %					F = eprover													% %
% %					F = vampire7												% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

:- dynamic	output_language_flag/1, check_predicate_constant_consistency_flag/1,
		solver_flag/1. 

:- retractall((output_language_flag(_))).
:- retractall((check_decidability_flag(_))).
:- retractall((check_predicate_function_consistency_flag(_))).
:- retractall((eliminate_function_symbols_flag(_))).
:- retractall((solver_flag(_))).

:- assertz((output_language_flag(tstp))).
:- assertz((check_decidability_flag(off))).
:- assertz((check_predicate_function_consistency_flag(off))).
:- assertz((eliminate_function_symbols_flag(on))).
:- assertz((solver_flag(vampire))).


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %     Index of auxiliary variables:											% %
% %																				% %
% %             We use the function "index_of_auxiliary_variables/1"            % %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

:- dynamic index_of_auxiliary_variables/1. 


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %     This file is divided into several parts:								% %
% %																				% %
% %             Part I:         Interaction with the user						% %
% %																				% %
% %             Part II:        Input											% %
% %																				% %
% %             Part III:       Ontology navigation								% %
% %																				% %
% %             Part IV:        Axiom transformation							% %
% %																				% %
% %             Part V:         Output											% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

:- reconsult('readAxioms.pl').
:- reconsult('readGoals.pl').
:- reconsult('compactAxioms.pl').
:- reconsult('verifyAxioms.pl').
:- reconsult('discardAxioms.pl').
:- reconsult('simplifyAxioms.pl').
:- reconsult('closeAxioms.pl').
:- reconsult('closeGoals.pl').
:- reconsult('expandAxioms.pl').
:- reconsult('typeAxioms.pl').
:- reconsult('typeGoals.pl').
:- reconsult('addTypeAxioms.pl').
:- reconsult('createGoals.pl').
:- reconsult('writeAxioms.pl').
:- reconsult('writeGoals.pl').
:- reconsult('listOperations.pl').
:- reconsult('ontologyInformation.pl').
:- reconsult('formatAxioms.pl').

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %     Part I:         Interaction with the user								% %
% %																				% %
% %             All the predicates to call and configure the translation        % %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %


%%% main
%%%             Use default ontologies

main:-
		deletePreviousInformation,
		
		print_to_screen('Reading input axiom files ... '),
		print_to_screen('\n'),
		readAxioms,																% Read input files
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Compacting axioms (I) ...'),
		compactAxioms,															% Compact axioms (first time)
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Closing axioms ...'),									% ONLY FOR SUMO
		closeAxioms,															% Close axioms
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('Verifying axioms ...'),
		verifyAxioms(NW),														% Verify axioms
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),
		print_to_screen(NW),
		print_to_screen(' error(s) occurred.'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Expanding axioms ...'),
		expandAxioms,															% Expand axioms
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Creating automatic axioms and goals ...'),
		createGoals,															% Create white-box tests
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Typing axioms ...'),
		typeAxioms,																% Type axioms
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Adding type axioms ...'),
		addTypeAxioms,															% Add type axioms
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Compacting axioms (II) ...'),
		compactAxioms,															% Compact axioms (second time)
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Simplify axioms ...'),
		simplifyAxioms,															% Simplify axioms
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Writing axioms ...'),
		print_to_screen('\n'),
		writeAxioms,															% Write output files of axioms
		print_to_screen('Done!'),
		
		print_to_screen('\n\n\n'),
		print_to_screen('Reading input goal files ... '),
		print_to_screen('\n'),
		readGoals,																% Read input files
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Closing goals ...'),
		closeGoals,																% Close axioms
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Typing goals ...'),
		typeGoals,																% Type goals
		print_to_screen('\n'),
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Writing goals ...'),
		print_to_screen('\n'),
		writeGoals,																% Write output files of goals
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Saving information ...'),
		print_to_screen('\n'),
		saveAdimenSUMOAxioms,													% Save information
		saveAdimenSUMOTests,
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Setting alternative ATP ...'),
		set_solver_flag(eprover),												% Setting E
		print_to_screen('Done!'),
		print_to_screen('\n'),

		print_to_screen('\n'),
		print_to_screen('Writing execution scripts ...'),
		print_to_screen('\n'),
		writeExecutionScripts,													% Write execution scripts
		print_to_screen('Done!'),
		print_to_screen('\n').


%%% set_output_language_flag(in F)
%%%             F: flag -> tstp (default), tptp, kif or mace4

set_output_language_flag(F):-
		(
		 (
		  F = tstp
		 );
		 (
		  F = tptp
		 );
		 (
		  F = kif
		 );
		 (
		  F = mace4
		 )
		),
		!,
		output_language_flag(OF),
		retractall((output_language_flag(_))),
		assertz((output_language_flag(F))),
		print_to_screen('output_language_flag updated.'),
		print_to_screen('\n'),
		print_to_screen('Old value: '),
		print_to_screen(OF),
		print_to_screen('\n'),
		print_to_screen('New value: '),
		print_to_screen(F),
		print_to_screen('\n').
set_output_language_flag(F):-
		print_to_screen('Non-valid option for output_language_flag [tstp (default), tptp, kif, mace4] --> '),
		print_to_screen(F),
		print_to_screen('\n'),
		output_language_flag(OF),
		print_to_screen('Current value: '),
		print_to_screen(OF),
		print_to_screen('\n').

%%% set_predicate_constant_consistency_flag(in F)
%%%             F: flag -> off (default) or on

set_predicate_constant_consistency_flag(F):-
		(
		 (
		  F = off
		 );
		 (
		  F = on
		 )
		),
		!,
		check_predicate_constant_consistency_flag(OF),
		retractall((check_predicate_constant_consistency_flag(_))),
		assertz((check_predicate_constant_consistency_flag(F))),
		print_to_screen('check_predicate_constant_consistency_flag updated.'),
		print_to_screen('\n'),
		print_to_screen('Old value: '),
		print_to_screen(OF),
		print_to_screen('\n'),
		print_to_screen('New value: '),
		print_to_screen(F),
		print_to_screen('\n').
set_check_predicate_constant_consistency_flag(F):-
		print_to_screen('Non-valid option for check_predicate_constant_consistency_flag [off (default), on] --> '),
		print_to_screen(F),
		print_to_screen('\n'),
		check_predicate_constant_consistency_flag(OF),
		print_to_screen('Current value: '),
		print_to_screen(OF),
		print_to_screen('\n').

%%% set_eliminate_function_symbols_flag(in F)
%%%             F: flag -> on (default) or off

set_eliminate_function_symbols_flag(F):-
		(
		 (
		  F = off
		 );
		 (
		  F = on
		 )
		),
		!,
		eliminate_function_symbols_flag(OF),
		retractall((eliminate_function_symbols_flag(_))),
		assertz((eliminate_function_symbols_flag(F))),
		print_to_screen('eliminate_function_symbols_flag updated.'),
		print_to_screen('\n'),
		print_to_screen('Old value: '),
		print_to_screen(OF),
		print_to_screen('\n'),
		print_to_screen('New value: '),
		print_to_screen(F),
		print_to_screen('\n').
set_eliminate_function_symbols_flag(F):-
		print_to_screen('Non-valid option for eliminate_function_symbols_flag [on (default), off] --> '),
		print_to_screen(F),
		print_to_screen('\n'),
		eliminate_function_symbols_flag(OF),
		print_to_screen('Current value: '),
		print_to_screen(OF),
		print_to_screen('\n').

%%% set_solver_flag(in F)
%%%             F: flag -> vampire (default), eprover or vampire7

set_solver_flag(F):-
		(
		 (
		  F = vampire
		 );
		 (
		  F = eprover
		 );
		 (
		  F = vanhelsing
		 );
		 (
		  F = vampire7
		 );
		 (
		  F = tptpSUMO
		 );
		 (
		  F = mace4
		 )
		),
		!,
		solver_flag(OF),
		retractall((solver_flag(_))),
		assertz((solver_flag(F))),
		print_to_screen('solver_flag updated.'),
		print_to_screen('\n'),
		print_to_screen('Old value: '),
		print_to_screen(OF),
		print_to_screen('\n'),
		print_to_screen('New value: '),
		print_to_screen(F),
		print_to_screen('\n').
set_solver_flag(F):-
		print_to_screen('Non-valid option for solver_flag [vampire (default), eprover, vanhelsing, vampire7, mace4] --> '),
		print_to_screen(F),
		print_to_screen('\n'),
		solver_flag(OF),
		print_to_screen('Current value: '),
		print_to_screen(OF),
		print_to_screen('\n').
