% %
% % Add type axioms
% %
% %     Type information of atoms is restricted by singular axioms

addTypeAxioms:-
		getAllPredicates(LP),
		addTypeAxiomsListOfPredicates(LP).

%%% addTypeAxiomsListOfPredicates(in LP)
%%%             LP: list of pairs (S,A) where
%%%			S is the symbol of a predicate
%%%			A is the arity of a predicate
%%%             Action: type axioms are added

addTypeAxiomsListOfPredicates(LP):-
		addTypeListOfAxioms(LP,'type',0).

%%% addTypeListOfAxioms(in LP, in K, in I)
%%%             LP: list of pairs (S,A) where
%%%			S is the symbol of a predicate
%%%			A is the arity of a predicate
%%%             K: the key prefix of the new axioms
%%%             I: last index used for keys
%%%             Action: type axioms are added

addTypeListOfAxioms([],K,I):-
		setNumberOfLines(K,I),
		setNumberOfAxioms(K,I).
addTypeListOfAxioms([(P,N)|LP],K,I):-
		getListOfSingleVariables(N,LV),
		F =..[P|LV],
		typeFormula(F,F1,LVT),
		extractTypeInformation(LV,LVT,NLVT,_),
		variablesTypeFormula(NLVT,F2),
		(
		 (
		  F2 \= [],
		  !,
		  NI is I + 1,
		  getAxiomKey(K,NI,NK),
		  saveAxiom(0,K,NI,NI,NK,'forall*'(LV,implies(F1,and(F2))))
		 );
		 (
		  NI = I
		 )
		),
		addTypeListOfAxioms(LP,K,NI).

%%%	getListOfSingleVariables(in I, out LV)
%%%		I: integer
%%%		LV: a list of single variables of lenght I

getListOfSingleVariables(0,[]).
getListOfSingleVariables(I,NLV):-
		number_codes(I,LC),
		atom_codes(N,LC),
		atom_concat('X',N,NV),
		NI is I-1,
		getListOfSingleVariables(NI,LV),
		append(LV,['#var'(NV)],NLV).
