% %
% %	Close axioms
% %
% %		Universal quantifiers are added

closeAxioms:-
		findall((F,C),getNumberOfAxioms(F,C),LFC),
		closeAxiomsListOfFiles(LFC).

%%% closeAxiomsListOfFiles(in LFC)
%%		LFC: list of pairs (F,C) where
%%		F is the name of the file
%%		C is the number of axioms in F
%%		Action: axioms are closed

closeAxiomsListOfFiles([]).
closeAxiomsListOfFiles([(F,C)|LFC]):-
		closeAxiomsFile(F,0,C),
		closeAxiomsListOfFiles(LFC).

%%% closeAxiomsFile(in F, in I, in N)
%%		F: file name
%%		I: index of the last treated axiom
%%		N: number of axioms
%%		Action: axioms are closed

closeAxiomsFile(_,N,N):-
		!.
closeAxiomsFile(F,I,N):-
		NI is I + 1,
		getAxiomKey(F,NI,K),
		!,
		(
		 (
		  1 is NI mod 100,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t'),
		  print_to_screen('Current axiom --> '),
		  print_to_screen(K),
		  print_to_screen(' 0')
		 );
		 (
		  1 is NI mod 10,
		  !,
		  NI1 is NI // 10,
		  NI2 is NI1 mod 10,
		  print_to_screen(' '),
		  print_to_screen(NI2)
		 );
		 (
		  true
		 )
		),
		getAxiom(K,C,A),
		(
		 (
		  C \= 0,
		  C \= 5,
		  !
		 );
		 (
		  closeAxiom(A,NA),
		  setAxiom(K,C,NA)
		 )
		),
		closeAxiomsFile(F,NI,N).

%%% closeAxiom(in A, out NA)
%%		A: an axiom
%%		NA: the closed axiom

closeAxiom(A,NA):-
		closeFormula(A,LV),
		(
		 (
		  LV = [],
		  !,
		  NA = A
		 );
		 (
		  NA = forall(LV,A)
		 )
		).

%%% closeFormula(in F, out LV)
%%		F: a formula
%%		LV: the list of variables that occur in F

closeFormula(exists(LV,F),NLV):-
		!,
		closeFormula(F,LVF),
		delete_list(LVF,LV,NLV).
closeFormula(forall(LV,F),NLV):-
		!,
		closeFormula(F,LVF),
		delete_list(LVF,LV,NLV).
closeFormula('exists*'(LV,F),NLV):-
		!,
		closeFormula(F,LVF),
		delete_list(LVF,LV,NLV).
closeFormula('forall*'(LV,F),NLV):-
		!,
		closeFormula(F,LVF),
		delete_list(LVF,LV,NLV).
closeFormula(and(LF),LV):-
		!,
		closeListOfFormulas(LF,LV).
closeFormula(or(LF),LV):-
		!,
		closeListOfFormulas(LF,LV).
closeFormula(implies(F1,F2),NLV):-
		!,
		closeFormula(F1,LV1),
		closeFormula(F2,LV2),
		union(LV1,LV2,NLV).
closeFormula(equiv(F1,F2),NLV):-
		!,
		closeFormula(F1,LV1),
		closeFormula(F2,LV2),
		union(LV1,LV2,NLV).
closeFormula(not(F),LV):-
		!,
		closeFormula(F,LV).
closeFormula(equal(T1,T2),NLV):-
		!,
		closeTerm(T1,LV1),
		closeTerm(T2,LV2),
		union(LV1,LV2,NLV).
closeFormula(A,LV):-
		A =..[_|ST],
		closeListOfTerms(ST,LV).

%%% closeListOfFormulas(in LF, out LV)
%%		LF: a list of formulas
%%		LV: the list of variables that occur in LF

closeListOfFormulas([],[]).
closeListOfFormulas([F|LF],NLV):-
		closeFormula(F,LV1),
		closeListOfFormulas(LF,LV2),
		union(LV1,LV2,NLV).

%%% closeListOfTerms(in LT, out LV)
%%		LT: a list of terms
%%		LV: the list of variables that occur in LT

closeListOfTerms([],[]).
closeListOfTerms([T|LT],NLV):-
		closeTerm(T,LV1),
		closeListOfTerms(LT,LV2),
		union(LV1,LV2,NLV).

%%% closeTerm(in T, out LV)
%%		T: a term
%%		LV: the list of variables that occur in T

closeTerm(T,[T]):-
		isSingleVariable(T),
		!.
closeTerm(T,[T]):-
		isRowVariable(T),
		!.
closeTerm(T,[]):-
		atom(T),
		!.
closeTerm(T,LV):-
		T =..[_|ST],
		closeListOfTerms(ST,LV).
