% %
% % Transfomation 1: compact Axioms
% %
% %		"and/1" and "or/1" lists are compacted when possible

compactAxioms:-
		findall((F,C),getNumberOfAxioms(F,C),LFC),
		compactAxiomsListOfFiles(LFC).

%%% compactGoal
%%%		Use default file

compactGoal:-
		compactAxiomsFile('goal',0,1).

%%% compactAxiomsListOfFiles(in LFC)
%%%		LFC: list of pairs (F,C) where
%%%			F is the name of the file
%%%			C is the number of axioms in F
%%%		Action: compact each axiom in the list of files

compactAxiomsListOfFiles([]).
compactAxiomsListOfFiles([(F,C)|LFC]):-
		compactAxiomsFile(F,0,C),
		compactAxiomsListOfFiles(LFC).

%%% compactAxiomsFile(in F, in I, in N)
%%%		F: file name
%%%		I: index of the last treated axiom
%%%		N: number of axioms
%%%		Action: compact each axiom in F

compactAxiomsFile(_,N,N):-
		!.
compactAxiomsFile(F,I,N):-
		NI is I + 1,
		getAxiomKey(F,NI,K),
		!,
		(
		 (
		  1 is NI mod 100,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t'),
		  print_to_screen('Current axiom --> '),
		  print_to_screen(K),
		  print_to_screen(' 0')
		 );
		 (
		  1 is NI mod 10,
		  !,
		  NI1 is NI // 10,
		  NI2 is NI1 mod 10,
		  print_to_screen(' '),
		  print_to_screen(NI2)
		 );
		 (
		  true
		 )
		),
		getAxiom(K,C,A),
		(
		 (
		  C >= 0,
		  C =< 2,
		  !,
		  compactAxiom(A,NA),
		  setAxiom(K,C,NA)
		 );
		 (
		  true
		 )
		),
		compactAxiomsFile(F,NI,N).

%%% compactAxiom(in A, out NA)
%%%		A: an axiom
%%%		NA: the axiom where the lists of "and/1" and "or/1" have been compacted in a single list when possible

compactAxiom(exists(LV,F),NF):-
		!,
		compactAxiom(F,F1),
		compactAxiomExists(LV,F1,NF).
compactAxiom(forall(LV,F),NF):-
		!,
		compactAxiom(F,F1),
		compactAxiomForall(LV,F1,NF).
compactAxiom('exists*'(LV,F),NF):-
		!,
		compactAxiom(F,F1),
		compactAxiomExistsStar(LV,F1,NF).
compactAxiom('forall*'(LV,F),NF):-
		!,
		compactAxiom(F,F1),
		compactAxiomForallStar(LV,F1,NF).
compactAxiom(and(LF),and(NLF)):-
		!,
		compactAxiomAnd(LF,LF1),
		compactListOfAxioms(LF1,NLF).
compactAxiom(or(LF),or(NLF)):-
		!,
		compactAxiomOr(LF,LF1),
		compactListOfAxioms(LF1,NLF).
compactAxiom('@and'(RV,H,S,F),'@and'(RV,H,S,NF)):-
		!,
		compactAxiom(F,NF).
compactAxiom('@or'(RV,H,S,F),'@or'(RV,H,S,NF)):-
		!,
		compactAxiom(F,NF).
compactAxiom(implies(F1,implies(F21,F22)),NF):-
		!,
		compactAxiom(implies(and([F1,F21]),F22),NF).
compactAxiom(implies(F1,forall(LV,F2)),NF):-
		!,
		compactAxiom(forall(LV,implies(F1,F2)),NF).
compactAxiom(implies(F1,'forall*'(LV,F2)),NF):-
		!,
		compactAxiom('forall*'(LV,implies(F1,F2)),NF).
compactAxiom(implies(F1,F2),implies(NF1,NF2)):-
		!,
		compactAxiom(F1,NF1),
		compactAxiom(F2,NF2).
compactAxiom(equiv(F1,F2),equiv(NF1,NF2)):-
		!,
		compactAxiom(F1,NF1),
		compactAxiom(F2,NF2).
compactAxiom(not(F),not(NF)):-
		!,
		compactAxiom(F,NF).
compactAxiom(F,F).

%%% compactAxiomExists(in LV, in F, out NF)
%%%		LV: a list of variables
%%%		F: a formula
%%%		NF: the formula that is obtained from compacting exists LV: F

compactAxiomExists(LV1,exists(LV2,F),NF):-
		!,
		append(LV1,LV2,NLV),
		compactAxiomExists(NLV,F,NF).
compactAxiomExists(LV,F,exists(LV,F)).

%%% compactAxiomForall(in LV, in F, out NF)
%%%		LV: a list of variables
%%%		F: a formula
%%%		NF: the formula that is obtained from compacting forall LV: F

compactAxiomForall(LV1,forall(LV2,F),NF):-
		!,
		append(LV1,LV2,NLV),
		compactAxiomForall(NLV,F,NF).
compactAxiomForall(LV,F,forall(LV,F)).

%%% compactAxiomExistsStar(in LV, in F, out NF)
%%%		LV: a list of variables
%%%		F: a formula
%%%		NF: the formula that is obtained from compacting exists* LV: F

compactAxiomExistsStar(LV1,'exists*'(LV2,F),NF):-
		!,
		append(LV1,LV2,NLV),
		compactAxiomExistsStar(NLV,F,NF).
compactAxiomExistsStar(LV,F,'exists*'(LV,F)).

%%% compactAxiomForallStar(in LV, in F, out NF)
%%%		LV: a list of variables
%%%		F: a formula
%%%		NF: the formula that is obtained from compacting forall* LV: F

compactAxiomForallStar(LV1,'forall*'(LV2,F),NF):-
		!,
		append(LV1,LV2,NLV),
		compactAxiomForallStar(NLV,F,NF).
compactAxiomForallStar(LV,F,'forall*'(LV,F)).

%%% compactAxiomAnd(in LF, out NLF)
%%%		LF: a list of formulas
%%%		NLF: the list that is obtained from compacting conjunction

compactAxiomAnd([],[]).
compactAxiomAnd([and(F)|LF],NLF):-
		!,
		compactAxiomAnd(F,F1),
		compactAxiomAnd(LF,LF1),
		append(F1,LF1,NLF).
compactAxiomAnd([F|LF],[F|NLF]):-
		compactAxiomAnd(LF,NLF).

%%% compactAxiomOr(in LF, out NLF)
%%%		LF: a list of formulas
%%%		NLF: the list that is obtained from compacting disjunction

compactAxiomOr([],[]).
compactAxiomOr([or(F)|LF],NLF):-
		!,
		compactAxiomOr(F,F1),
		compactAxiomOr(LF,LF1),
		append(F1,LF1,NLF).
compactAxiomOr([F|LF],[F|NLF]):-
		compactAxiomOr(LF,NLF).

%%% compactListOfAxioms(in LF, out NLF)
%%%		LF: list of formulas
%%%		NLF: the list of compacted formulas

compactListOfAxioms([],[]).
compactListOfAxioms([F|LF],[NF|NLF]):-
		compactAxiom(F,NF),
		compactListOfAxioms(LF,NLF).
