% %
% %	createGoals
% %
% %		Goals are created automatically

createGoals:-
		findall((F,C),(getOntologyList(L),getNumberOfAxioms(F,C),F \= 'predefinitions',member(F,L)),LFC),
		open('whiteBox.txt',write,S),
%leash([]),trace,
		createGoalsListOfFiles(S,0,0,LFC),
		close(S).

% %
% %	createOrGoals
% %
% %		Or goals are created automatically

createOrGoals:-
		findall((F,C),(getOntologyList(L),getNumberOfAxioms(F,C),F \= 'predefinitions',member(F,L)),LFC),
		open('orWhiteBox.txt',write,S),
%leash([]),trace,
		createOrGoalsListOfFiles(S,0,0,LFC),
		close(S).

%%% createGoalsListOfFiles(in S, in I, in T, in LFC)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		LFC: list of pairs (F,C) where
%%%			F is the name of the file
%%%			C is the number of axioms in F
%%%		Action: goals are created automatically

createGoalsListOfFiles(_,I,T,[]):-
		setNumberOfGoals('whiteBoxFalsityTest',I),
		setNumberOfGoals('whiteBoxTruthTest',I),
		setNumberOfLines('whiteBox',I),
		setNumberOfAxioms('whiteBox',I),
		setNumberOfTests(T).
createGoalsListOfFiles(S,I,T,[(F,C)|LFC]):-
		createGoalsFile(S,I,T,F,0,C,NI,NT),
		createGoalsListOfFiles(S,NI,NT,LFC).

%%% createOrGoalsListOfFiles(in S, in I, in T, in LFC)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		LFC: list of pairs (F,C) where
%%%			F is the name of the file
%%%			C is the number of axioms in F
%%%		Action: or goals are created automatically

createOrGoalsListOfFiles(_,I,T,[]):-
		setNumberOfGoals('orWhiteBoxFalsityTest',I),
		setNumberOfGoals('orWhiteBoxTruthTest',I),
		setNumberOfLines('orWhiteBox',I),
		setNumberOfAxioms('orWhiteBox',I),
		setNumberOfTests(T).
createOrGoalsListOfFiles(S,I,T,[(F,C)|LFC]):-
		createOrGoalsFile(S,I,T,F,0,C,NI,NT),
		createOrGoalsListOfFiles(S,NI,NT,LFC).

%%% createGoalsFile(in S, in G, in T, in F, in I, in N, out NG, out NT)
%%%		S: output stream
%%%		G: index of goals
%%%		T: index of tests
%%%		F: file name
%%%		I: index of the last treated axiom
%%%		N: number of axioms
%%%		NG: new index of goals
%%%		NT: new index of tests
%%%		Action: goals are created automatically

createGoalsFile(_,G,T,_,N,N,G,T):-
		!.
createGoalsFile(S,G,T,F,I,N,NG,NT):-
		NI is I + 1,
		getAxiomKey(F,NI,K),
%print_to_screen(K),
%print_to_screen('\n'),
		!,
		(
		 (
		  1 is NI mod 100,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t'),
		  print_to_screen('Current axiom --> '),
		  print_to_screen(K),
		  print_to_screen(' 0')
		 );
		 (
		  1 is NI mod 10,
		  !,
		  NI1 is NI // 10,
		  NI2 is NI1 mod 10,
		  print_to_screen(' '),
		  print_to_screen(NI2)
		 );
		 (
		  true
		 )
		),
/*
(
 (
  K = 'milo1133A1188',
  !,
  leash([]), trace
 );
 (
  notrace
 )
),
*/
		getAxiom(K,C,A),
		(
		 (
		  C = 0,
		  !,
		  distributeQuantifiers(A,QF),
		  typeAxiom(QF,TF),
		  simplifyFormula(TF,1,SF),
		  createAxiomGoals(S,G,T,K,SF,G1,T1)
		 );
		 (
		  G1 = G,
		  T1 = T
		 )
		),
		createGoalsFile(S,G1,T1,F,NI,N,NG,NT).

%%% createOrGoalsFile(in S, in G, in T, in F, in I, in N, out NG, out NT)
%%%		S: output stream
%%%		G: index of goals
%%%		T: index of tests
%%%		F: file name
%%%		I: index of the last treated axiom
%%%		N: number of axioms
%%%		NG: new index of goals
%%%		NT: new index of tests
%%%		Action: or goals are created automatically

createOrGoalsFile(_,G,T,_,N,N,G,T):-
		!.
createOrGoalsFile(S,G,T,F,I,N,NG,NT):-
		NI is I + 1,
		getAxiomKey(F,NI,K),
%print_to_screen(K),
		!,
		(
		 (
		  1 is NI mod 100,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t'),
		  print_to_screen('Current axiom --> '),
		  print_to_screen(K),
		  print_to_screen(' 0')
		 );
		 (
		  1 is NI mod 10,
		  !,
		  NI1 is NI // 10,
		  NI2 is NI1 mod 10,
		  print_to_screen(' '),
		  print_to_screen(NI2)
		 );
		 (
		  true
		 )
		),
/*
(
 (
  K = 'milo1133A1188',
  !,
  leash([]), trace
 );
 (
  notrace
 )
),
*/
		getAxiom(K,C,A),
		(
		 (
		  C = 0,
		  !,
		  distributeQuantifiers(A,QF),
%		  typeAxiom(QF,TF),
		  simplifyFormula(QF,1,SF),
		  createAxiomOrGoals(S,G,T,K,SF,G1,T1)
		 );
		 (
		  G1 = G,
		  T1 = T
		 )
		),
		createOrGoalsFile(S,G1,T1,F,NI,N,NG,NT).

%%% simplifyFormula (in F, in M, out NF)
%%%		F: a formula
%%%		M: mode
%%%			mode 0 -> negation is distributed (default)
%%%			mode 1 -> negation is NOT distributed
%%%		NF: the formula that is obtained from simplifying and compacting F

% Default

simplifyFormula(F,NF):-
		compactAxiom(F,F1),
		simplifyAxiom(F1,F2),
		compactAxiom(F2,NF).

simplifyFormula(F,M,NF):-
		compactAxiom(F,F1),
		simplifyAxiom(F1,M,F2),
		compactAxiom(F2,NF).

%%% createAxiomGoals(in S, in I, in T, in K, in A, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tets
%%%		K: an axiom key
%%%		A: an axiom
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from A

createAxiomGoals(S,I,T,K,A,NI,NT):-
		createFormulaGoals(S,I,T,K,[],A,NI,NT).

%%% createAxiomOrGoals(in S, in I, in T, in K, in A, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tets
%%%		K: an axiom key
%%%		A: an axiom
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from A

createAxiomOrGoals(S,I,T,K,A,NI,NT):-
		createFormulaOrGoals(S,I,T,K,[],A,NI,NT).

%%% createFormulaGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from F

createFormulaGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedFormulaGoals(S,I,T,K,NLVP,F,NI,NT).
createFormulaGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedFormulaGoals(S,I,T,K,NLVP,F,NI,NT).
createFormulaGoals(S,I,T,K,LV,and([F]),NI,NT):-
		!,
		createFormulaGoals(S,I,T,K,LV,F,NI,NT).
createFormulaGoals(S,I,T,K,LV,and([F|LF]),NI,NT):-
		!,
		createFormulaGoals(S,I,T,K,LV,F,I1,T1),
		createFormulaGoals(S,I1,T1,K,LV,and(LF),NI,NT).
createFormulaGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createFormulaGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createFormulaGoals(S,I,T,K,LVP,not(F),NI,NT):-
		!,
		createFormulaGoals(S,I,T,K,LVP,F,NI,NT).
createFormulaGoals(S,I,T,K,LV,F,NI,NT):-
		createSubformulaGoals(S,I,T,K,LV,F,NI,NT).

%%% createNonClosedFormulaGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from F

createNonClosedFormulaGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedFormulaGoals(S,I,T,K,NLVP,F,NI,NT).
createNonClosedFormulaGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedFormulaGoals(S,I,T,K,NLVP,F,NI,NT).
createNonClosedFormulaGoals(S,I,T,K,LV,and([F]),NI,NT):-
		!,
		createNonClosedFormulaGoals(S,I,T,K,LV,F,NI,NT).
createNonClosedFormulaGoals(S,I,T,K,LV,and([F|LF]),NI,NT):-
		!,
		createNonClosedFormulaGoals(S,I,T,K,LV,F,I1,T1),
		createNonClosedFormulaGoals(S,I1,T1,K,LV,and(LF),NI,NT).
createNonClosedFormulaGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createNonClosedFormulaGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createNonClosedFormulaGoals(S,I,T,K,LVP,not(F),NI,NT):-
		!,
		createNonClosedFormulaGoals(S,I,T,K,LVP,F,NI,NT).
createNonClosedFormulaGoals(S,I,T,K,LV,F,NI,NT):-
		createNonClosedSubformulaGoals(S,I,T,K,LV,F,NI,NT).

%%% createFormulaOrGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from F

createFormulaOrGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedFormulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createFormulaOrGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedFormulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createFormulaOrGoals(S,I,T,K,LV,and([F]),NI,NT):-
		!,
		createFormulaOrGoals(S,I,T,K,LV,F,NI,NT).
createFormulaOrGoals(S,I,T,K,LV,and([F|LF]),NI,NT):-
		!,
		createFormulaOrGoals(S,I,T,K,LV,F,I1,T1),
		createFormulaOrGoals(S,I1,T1,K,LV,and(LF),NI,NT).
createFormulaOrGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createFormulaOrGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createFormulaOrGoals(S,I,T,K,LVP,not(F),NI,NT):-
		!,
		createFormulaOrGoals(S,I,T,K,LVP,F,NI,NT).
createFormulaOrGoals(S,I,T,K,LV,F,NI,NT):-
		createSubformulaOrGoals(S,I,T,K,LV,F,NI,NT).

%%% createNonClosedFormulaOrGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from F

createNonClosedFormulaOrGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedFormulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createNonClosedFormulaOrGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedFormulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createNonClosedFormulaOrGoals(S,I,T,K,LV,and([F]),NI,NT):-
		!,
		createNonClosedFormulaOrGoals(S,I,T,K,LV,F,NI,NT).
createNonClosedFormulaOrGoals(S,I,T,K,LV,and([F|LF]),NI,NT):-
		!,
		createNonClosedFormulaOrGoals(S,I,T,K,LV,F,I1,T1),

		createNonClosedFormulaOrGoals(S,I1,T1,K,LV,and(LF),NI,NT).
createNonClosedFormulaOrGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createNonClosedFormulaOrGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createNonClosedFormulaOrGoals(S,I,T,K,LVP,not(F),NI,NT):-
		!,
		createNonClosedFormulaOrGoals(S,I,T,K,LVP,F,NI,NT).
createNonClosedFormulaOrGoals(S,I,T,K,LV,F,NI,NT):-
		createNonClosedSubformulaOrGoals(S,I,T,K,LV,F,NI,NT).

%%% createSubformulaGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from F

createSubformulaGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		saveTest(S,I,K,forallTest,'forall*'(LVP,'forall*'(LV,F)),I1),
		saveTest(S,I1,K,negatedForallTest,'forall*'(LVP,not('forall*'(LV,F))),I2),
		append(LVP,LV,NLVP),
		createNonClosedSubformulaGoals(S,I2,T,K,NLVP,F,NI,NT).
createSubformulaGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		saveTest(S,I,K,existsTest,'forall*'(LVP,'exists*'(LV,F)),I1),
		saveTest(S,I1,K,negatedExistsTest,'forall*'(LVP,not('exists*'(LV,F))),I2),
		append(LVP,LV,NLVP),
		createNonClosedSubformulaGoals(S,I2,T,K,NLVP,F,NI,NT).
createSubformulaGoals(S,I,T,K,LVP,and(LF),NI,NT):-
		!,
		createAndSubformulasGoals(S,I,T,K,LVP,LF,NI,NT).
createSubformulaGoals(S,I,T,K,LVP,or(LF),NI,NT):-
		!,
		createOrSubformulasGoals(S,I,T,K,LVP,LF,NI,NT).
createSubformulaGoals(S,I,T,K,LVP,implies(F1,F2),NI,NT):-
		!,
		saveTest(S,I,K,antecedentImpliesTest,'forall*'(LVP,not(F1)),I1),
		saveTest(S,I1,K,negatedAntecedentImpliesTest,'forall*'(LVP,F1),I2),
		saveTest(S,I2,K,consequentImpliesTest,'forall*'(LVP,F2),I3),
		saveTest(S,I3,K,negatedConsequentImpliesTest,'forall*'(LVP,not(F2)),I4),
		createSubformulaGoals(S,I4,T,K,LVP,not(F1),I5,T1),
		createSubformulaGoals(S,I5,T1,K,LVP,F2,NI,NT).
createSubformulaGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createSubformulaGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createSubformulaGoals(S,I,T,K,LVP,not(F),NI,NT):-
		createSubformulaGoals(S,I,T,K,LVP,F,NI,NT).
createSubformulaGoals(_,I,T,_,_,_,I,T).

%%% createNonClosedSubformulaGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from F

createNonClosedSubformulaGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		saveNonClosedTest(S,I,K,forallTest,'forall*'(LVP,'forall*'(LV,F)),I1,TKG1),
		saveNonClosedTest(S,I1,K,negatedForallTest,'forall*'(LVP,not('forall*'(LV,F))),I2,TKG2),
		append(LVP,LV,NLVP),
		T1 is T+1,
		saveTestInformation(K,forallTest,T1,TKG1,TKG2),
		createNonClosedSubformulaGoals(S,I2,T1,K,NLVP,F,NI,NT).
createNonClosedSubformulaGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		saveNonClosedTest(S,I,K,existsTest,'forall*'(LVP,'exists*'(LV,F)),I1,TKG1),
		saveNonClosedTest(S,I1,K,negatedExistsTest,'forall*'(LVP,not('exists*'(LV,F))),I2,TKG2),
		append(LVP,LV,NLVP),
		T1 is T+1,
		saveTestInformation(K,existsTest,T1,TKG1,TKG2),
		createNonClosedSubformulaGoals(S,I2,T1,K,NLVP,F,NI,NT).
createNonClosedSubformulaGoals(S,I,T,K,LVP,and(LF),NI,NT):-
		!,
		createNonClosedAndSubformulasGoals(S,I,T,K,LVP,LF,NI,NT).
createNonClosedSubformulaGoals(S,I,T,K,LVP,or(LF),NI,NT):-
		!,
		createNonClosedOrSubformulasGoals(S,I,T,K,LVP,LF,NI,NT).
createNonClosedSubformulaGoals(S,I,T,K,LVP,implies(F1,F2),NI,NT):-
		!,
		saveNonClosedTest(S,I,K,antecedentImpliesTest,'forall*'(LVP,not(F1)),I1,TKG1),
		saveNonClosedTest(S,I1,K,negatedAntecedentImpliesTest,'forall*'(LVP,F1),I2,TKG2),
		saveNonClosedTest(S,I2,K,consequentImpliesTest,'forall*'(LVP,F2),I3,TKG3),
		saveNonClosedTest(S,I3,K,negatedConsequentImpliesTest,'forall*'(LVP,not(F2)),I4,TKG4),
		T1 is T+1,
		saveTestInformation(K,antecedentImpliesTest,T1,TKG1,TKG2),
		T2 is T1+1,
		saveTestInformation(K,consequentImpliesTest,T2,TKG3,TKG4),
		createNonClosedSubformulaGoals(S,I4,T2,K,LVP,not(F1),I5,T3),
		createNonClosedSubformulaGoals(S,I5,T3,K,LVP,F2,NI,NT).
createNonClosedSubformulaGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createNonClosedSubformulaGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createNonClosedSubformulaGoals(S,I,T,K,LVP,not(F),NI,NT):-
		createNonClosedSubformulaGoals(S,I,T,K,LVP,F,NI,NT).
createNonClosedSubformulaGoals(_,I,T,_,_,_,I,T).

%%% createSubformulaOrGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from F

createSubformulaOrGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedSubformulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createSubformulaOrGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedSubformulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createSubformulaOrGoals(S,I,T,K,LVP,and(LF),NI,NT):-
		!,
		createAndSubformulasOrGoals(S,I,T,K,LVP,LF,NI,NT).
createSubformulaOrGoals(S,I,T,K,LVP,or(LF),NI,NT):-
		!,
		createOrSubformulasOrGoals(S,I,T,K,LVP,LF,NI,NT).
createSubformulaOrGoals(S,I,T,K,LVP,implies(F1,F2),NI,NT):-
		!,
		saveOrTest(S,I,K,antecedentImpliesTest,'forall*'(LVP,not(F1)),I1),
		saveOrTest(S,I1,K,negatedAntecedentImpliesTest,'forall*'(LVP,F1),I2),
		saveOrTest(S,I2,K,consequentImpliesTest,'forall*'(LVP,F2),I3),
		saveOrTest(S,I3,K,negatedConsequentImpliesTest,'forall*'(LVP,not(F2)),I4),
		createSubformulaOrGoals(S,I4,T,K,LVP,not(F1),I5,T1),
		createSubformulaOrGoals(S,I5,T1,K,LVP,F2,NI,NT).
createSubformulaOrGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createSubformulaOrGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createSubformulaOrGoals(S,I,T,K,LVP,not(F),NI,NT):-
		createSubformulaOrGoals(S,I,T,K,LVP,F,NI,NT).
createSubformulaOrGoals(_,I,T,_,_,_,I,T).

%%% createNonClosedSubformulaOrGoals(in S, in I, in T, in K, in LV, in F, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		F: a formula
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from F

createNonClosedSubformulaOrGoals(S,I,T,K,LVP,'forall*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedSubformulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createNonClosedSubformulaOrGoals(S,I,T,K,LVP,'exists*'(LV,F),NI,NT):-
		!,
		append(LVP,LV,NLVP),
		createNonClosedSubformulaOrGoals(S,I,T,K,NLVP,F,NI,NT).
createNonClosedSubformulaOrGoals(S,I,T,K,LVP,and(LF),NI,NT):-
		!,
		createNonClosedAndSubformulasOrGoals(S,I,T,K,LVP,LF,NI,NT).
createNonClosedSubformulaOrGoals(S,I,T,K,LVP,or(LF),NI,NT):-
		!,
		createNonClosedOrSubformulasOrGoals(S,I,T,K,LVP,LF,NI,NT).
createNonClosedSubformulaOrGoals(S,I,T,K,LVP,implies(F1,F2),NI,NT):-
		!,
		saveNonClosedOrTest(S,I,K,antecedentImpliesTest,'forall*'(LVP,not(F1)),I1,TKG1),
		saveNonClosedOrTest(S,I1,K,negatedAntecedentImpliesTest,'forall*'(LVP,F1),I2,TKG2),
		saveNonClosedOrTest(S,I2,K,consequentImpliesTest,'forall*'(LVP,F2),I3,TKG3),
		saveNonClosedOrTest(S,I3,K,negatedConsequentImpliesTest,'forall*'(LVP,not(F2)),I4,TKG4),
		T1 is T+1,
		saveTestInformation(K,antecedentImpliesTest,T1,TKG1,TKG2),
		T2 is T1+1,
		saveTestInformation(K,consequentImpliesTest,T2,TKG3,TKG4),
		createNonClosedSubformulaOrGoals(S,I4,T2,K,LVP,not(F1),I5,T3),
		createNonClosedSubformulaOrGoals(S,I5,T3,K,LVP,F2,NI,NT).
createNonClosedSubformulaOrGoals(S,I,T,K,LVP,equiv(F1,F2),NI,NT):-
		!,
		createNonClosedSubformulaOrGoals(S,I,T,K,LVP,and([implies(F1,F2),implies(F2,F1)]),NI,NT).
createNonClosedSubformulaOrGoals(S,I,T,K,LVP,not(F),NI,NT):-
		createNonClosedSubformulaOrGoals(S,I,T,K,LVP,F,NI,NT).
createNonClosedSubformulaOrGoals(_,I,T,_,_,_,I,T).

%%% createAndSubformulaGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in conjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from and(LF)

createAndSubformulasGoals(_,I,T,_,_,[],I,T).
createAndSubformulasGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveTest(S,I,K,andTest,'forall*'(LVP,F),I1),
		saveTest(S,I1,K,negatedAndTest,'forall*'(LVP,not(F)),I2),
		createSubformulaGoals(S,I2,T,K,LVP,F,I3,T1),
		createAndSubformulasGoals(S,I3,T1,K,LVP,LF,NI,NT).

%%% createNonClosedAndSubformulaGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in conjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from and(LF)

createNonClosedAndSubformulasGoals(_,I,T,_,_,[],I,T).
createNonClosedAndSubformulasGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveNonClosedTest(S,I,K,andTest,'forall*'(LVP,F),I1,TKG1),
		saveNonClosedTest(S,I1,K,negatedAndTest,'forall*'(LVP,not(F)),I2,TKG2),
		T1 is T+1,
		saveTestInformation(K,andTest,T1,TKG1,TKG2),
		createNonClosedSubformulaGoals(S,I2,T1,K,LVP,F,I3,T2),
		createNonClosedAndSubformulasGoals(S,I3,T2,K,LVP,LF,NI,NT).

%%% createAndSubformulaOrGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in conjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from and(LF)

createAndSubformulasOrGoals(_,I,T,_,_,[],I,T).
createAndSubformulasOrGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		createSubformulaOrGoals(S,I,T,K,LVP,F,I1,T1),
		createAndSubformulasOrGoals(S,I1,T1,K,LVP,LF,NI,NT).

%%% createNonClosedAndSubformulaOrGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in conjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from and(LF)

createNonClosedAndSubformulasOrGoals(_,I,T,_,_,[],I,T).
createNonClosedAndSubformulasOrGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		createNonClosedSubformulaOrGoals(S,I,T,K,LVP,F,I1,T1),
		createNonClosedAndSubformulasOrGoals(S,I1,T1,K,LVP,LF,NI,NT).

%%% createOrSubformulaGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in disjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from or(LF)

createOrSubformulasGoals(_,I,T,_,_,[],I,T).
createOrSubformulasGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveTest(S,I,K,orTest,'forall*'(LVP,F),I1),
		saveTest(S,I1,K,negatedOrTest,'forall*'(LVP,not(F)),I2),
		createSubformulaGoals(S,I2,T,K,LVP,F,I3,T1),
		createOrSubformulasGoals(S,I3,T1,K,LVP,LF,NI,NT).

%%% createNonClosedOrSubformulaGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in disjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: white-box automatic tests are generated from or(LF)

createNonClosedOrSubformulasGoals(_,I,T,_,_,[],I,T).
createNonClosedOrSubformulasGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveNonClosedTest(S,I,K,orTest,'forall*'(LVP,F),I1,TKG1),
		saveNonClosedTest(S,I1,K,negatedOrTest,'forall*'(LVP,not(F)),I2,TKG2),
		T1 is T+1,
		saveTestInformation(K,andTest,T1,TKG1,TKG2),
		createNonClosedSubformulaGoals(S,I2,T1,K,LVP,F,I3,T2),
		createNonClosedOrSubformulasGoals(S,I3,T2,K,LVP,LF,NI,NT).

%%% createOrSubformulaOrGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in disjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from or(LF)

createOrSubformulasOrGoals(_,I,T,_,_,[],I,T).
createOrSubformulasOrGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveOrTest(S,I,K,orTest,'forall*'(LVP,F),I1),
		saveOrTest(S,I1,K,negatedOrTest,'forall*'(LVP,not(F)),I2),
		createSubformulaOrGoals(S,I2,T,K,LVP,F,I3,T1),
		createOrSubformulasOrGoals(S,I3,T1,K,LVP,LF,NI,NT).

%%% createNonClosedOrSubformulaOrGoals(in S, in I, in T, in K, in LV, in LF, out NI, out NT)
%%%		S: output stream
%%%		I: index of goals
%%%		T: index of tests
%%%		K: an axiom key
%%%		LV: a list of variables
%%%		LF: a list of formulas in disjunction
%%%		NI: new index of goals
%%%		NT: new index of tests
%%%		Action: or white-box automatic tests are generated from or(LF)

createNonClosedOrSubformulasOrGoals(_,I,T,_,_,[],I,T).
createNonClosedOrSubformulasOrGoals(S,I,T,K,LVP,[F|LF],NI,NT):-
		saveNonClosedOrTest(S,I,K,orTest,'forall*'(LVP,F),I1,TKG1),
		saveNonClosedOrTest(S,I1,K,negatedOrTest,'forall*'(LVP,not(F)),I2,TKG2),
		T1 is T+1,
		saveTestInformation(K,andTest,T1,TKG1,TKG2),
		createNonClosedSubformulaOrGoals(S,I2,T1,K,LVP,F,I3,T2),
		createNonClosedOrSubformulasOrGoals(S,I3,T2,K,LVP,LF,NI,NT).

%%% saveTest(in S, in I, in K, in M, in T, out NI, out TKG)
%%%		S: output stream
%%%		I: index of goals
%%%		K: an axiom key
%%%		M: a test mode
%%%		T: a test
%%%		NI: new index of goals
%%%		TKG: key of the new truth-test
%%%		Action: the new test is stored

saveTest(S,I,K,M,T,I):-
		simplifyFormula(T,FST),
		simplifyFormula(not(FST),TST),
		toPrologVariables(FST,FLV,NFST),
		getGoalFile(FKG,NFST,'whiteBoxFalsityTest'),
		remove_variables(FLV,NFLV),
		no_doubles(NFLV),
		toPrologVariables(TST,TLV,NTST),
		getGoalFile(TKG,NTST,'whiteBoxTruthTest'),
		remove_variables(TLV,NTLV),
		no_doubles(NTLV),
		!,
		saveTestMode(FKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n'),
		saveTestMode(TKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n').
saveTest(S,I,K,M,T,NI):-
		simplifyFormula(T,FST),
		simplifyFormula(not(FST),TST),
		NI is I+1,
		getGoalKey('whiteBoxFalsityTest',NI,FKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getGoalKey('whiteBoxTruthTest',NI,TKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveGoal('whiteBoxFalsityTest',NI,NI,FKG,FST),
		saveTestMode(FKG,K,M),
		saveGoal('whiteBoxTruthTest',NI,NI,TKG,TST),
		saveTestMode(TKG,K,M),
		getAxiomKey('whiteBox',NI,KA),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,KA),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveAxiom(0,'whiteBox',NI,NI,KA,TST).

%%% saveNonClosedTest(in S, in I, in K, in M, in T, out NI, out TKG)
%%%		S: output stream
%%%		I: index of goals
%%%		K: an axiom key
%%%		M: a test mode
%%%		T: a test
%%%		NI: new index of goals
%%%		TKG: key of the new truth-test
%%%		Action: the new test is stored

saveNonClosedTest(S,I,K,M,T,I,TKG):-
		simplifyFormula(T,FST),
		simplifyFormula(not(FST),TST),
		toPrologVariables(FST,FLV,NFST),
		getGoalFile(FKG,NFST,'whiteBoxFalsityTest'),
		remove_variables(FLV,NFLV),
		no_doubles(NFLV),
		toPrologVariables(TST,TLV,NTST),
		getGoalFile(TKG,NTST,'whiteBoxTruthTest'),
		remove_variables(TLV,NTLV),
		no_doubles(NTLV),
		!,
		saveTestMode(FKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n'),
		saveTestMode(TKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),

		print_to_file(S,'\n').
saveNonClosedTest(S,I,K,M,T,NI,TKG):-
		simplifyFormula(T,FST),
		simplifyFormula(not(FST),TST),
		NI is I+1,
		getGoalKey('whiteBoxFalsityTest',NI,FKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getGoalKey('whiteBoxTruthTest',NI,TKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveGoal('whiteBoxFalsityTest',NI,NI,FKG,FST),
		saveTestMode(FKG,K,M),
		saveGoal('whiteBoxTruthTest',NI,NI,TKG,TST),
		saveTestMode(TKG,K,M),
		getAxiomKey('whiteBox',NI,KA),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,KA),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveAxiom(0,'whiteBox',NI,NI,KA,TST).

%%% saveOrTest(in S, in I, in K, in M, in T, out NI, out TKG)
%%%		S: output stream
%%%		I: index of goals
%%%		K: an axiom key
%%%		M: a test mode
%%%		T: an or test
%%%		NI: new index of goals
%%%		TKG: key of the new or truth-test
%%%		Action: the new or test is stored

saveOrTest(S,I,K,M,T,I):-
		simplifyFormula(T,FST),
		simplifyFormula(not(FST),TST),
		toPrologVariables(FST,FLV,NFST),
		getGoalFile(FKG,NFST,'orWhiteBoxFalsityTest'),
		remove_variables(FLV,NFLV),
		no_doubles(NFLV),
		toPrologVariables(TST,TLV,NTST),
		getGoalFile(TKG,NTST,'orWhiteBoxTruthTest'),
		remove_variables(TLV,NTLV),
		no_doubles(NTLV),
		!,
		saveTestMode(FKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n'),
		saveTestMode(TKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n').
saveOrTest(S,I,K,M,T,NI):-
		simplifyFormula(T,FST),
		simplifyFormula(not(FST),TST),
		NI is I+1,
		getGoalKey('orWhiteBoxFalsityTest',NI,FKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getGoalKey('orWhiteBoxTruthTest',NI,TKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveGoal('orWhiteBoxFalsityTest',NI,NI,FKG,FST),
		saveTestMode(FKG,K,M),
		saveGoal('orWhiteBoxTruthTest',NI,NI,TKG,TST),
		saveTestMode(TKG,K,M),
		getAxiomKey('orWhiteBox',NI,KA),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,KA),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveAxiom(0,'orWhiteBox',NI,NI,KA,TST).

%%% saveNonClosedOrTest(in S, in I, in K, in M, in T, out NI, out TKG)
%%%		S: output stream
%%%		I: index of goals
%%%		K: an axiom key
%%%		M: a test mode
%%%		T: an or test
%%%		NI: new index of goals
%%%		TKG: key of the new or truth-test
%%%		Action: the new or test is stored

saveNonClosedOrTest(S,I,K,M,T,I,TKG):-
		simplifyFormula(T,FST),
		simplifyFormula(not(FST),TST),
		toPrologVariables(FST,FLV,NFST),
		getGoalFile(FKG,NFST,'orWhiteBoxFalsityTest'),
		remove_variables(FLV,NFLV),
		no_doubles(NFLV),
		toPrologVariables(TST,TLV,NTST),
		getGoalFile(TKG,NTST,'orWhiteBoxTruthTest'),
		remove_variables(TLV,NTLV),
		no_doubles(NTLV),
		!,
		saveTestMode(FKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n'),
		saveTestMode(TKG,K,repeated(M)),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,repeated(M)),
		print_to_file(S,'\n').
saveNonClosedOrTest(S,I,K,M,T,NI,TKG):-
		simplifyFormula(T,FST),
		simplifyFormula(not(FST),TST),
		NI is I+1,
		getGoalKey('orWhiteBoxFalsityTest',NI,FKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,FKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		getGoalKey('orWhiteBoxTruthTest',NI,TKG),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,TKG),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveGoal('orWhiteBoxFalsityTest',NI,NI,FKG,FST),
		saveTestMode(FKG,K,M),
		saveGoal('orWhiteBoxTruthTest',NI,NI,TKG,TST),
		saveTestMode(TKG,K,M),
		getAxiomKey('orWhiteBox',NI,KA),
		print_to_file(S,K),
		print_to_file(S,' - '),
		print_to_file(S,KA),
		print_to_file(S,' - '),
		print_to_file(S,M),
		print_to_file(S,'\n'),
		saveAxiom(0,'orWhiteBox',NI,NI,KA,TST).

%%% saveTestMode(in KG, in KA, in M)
%%%		KG: a goal Key
%%%		KA: an axiom key
%%%		M: a test mode
%%%		Action: the test mode is stored

saveTestMode(KG,KA,M):-
		assertz((goalInformation(KG,KA,M))).

%%% saveTestInformation(in K, in M, in I, in TKG1, in TKG2))
%%%		K: an axiom key
%%%		M: a test mode
%%%		I: index of test
%%%		TKG1,TKG2: truth-test keys
%%%		Action: the test information is stored

saveTestInformation(K,M,T,TKG1,TKG2):-
		assertz((testInformation(K,M,T,TKG1,TKG2))).

%%% formulaVariablesToPrologVariables(in F, out LPV, out NF)
%%%		F: a formula
%%%		LPV: a list of Prolog variables
%%%		NF: the formula that is obtained by replacing formula variables with
%%%				Prolog variables in F

formulaVariablesToPrologVariables(F,NLPV,NF):-
		toPrologVariables(F,LPV1,F1),
		findall(V,(sub_term(V,F1),isSingleVariable(V),ground(V)),LV),
		replaceListOfFormulaVariables(LV,F1,LPV2,NF),
		append(LPV1,LPV2,NLPV).

%%% toPrologVariables(in F, out LPV, out NF)
%%%		F: a formula
%%%		LPV: a list of Prolog variables
%%%		NF: the formula that is obtained by replacing formula variables with
%%%				Prolog variables in F

toPrologVariables('exists*'(LV,F),NLPV,'exists*'(_,F2)):-		%LPV1
		!,
		replaceListOfFormulaVariables(LV,F,LPV1,F1),
		toPrologVariables(F1,LPV2,F2),
		append(LPV1,LPV2,NLPV).
toPrologVariables('forall*'(LV,F),NLPV,'forall*'(_,F2)):-		%LPV1
		!,
		replaceListOfFormulaVariables(LV,F,LPV1,F1),
		toPrologVariables(F1,LPV2,F2),
		append(LPV1,LPV2,NLPV).
toPrologVariables('exists*'(LV,F),NLPV,'exists*'(_,F2)):-		%LPV1
		!,
		replaceListOfFormulaVariables(LV,F,LPV1,F1),
		toPrologVariables(F1,LPV2,F2),
		append(LPV1,LPV2,NLPV).
toPrologVariables('forall*'(LV,F),NLPV,'forall*'(_,F2)):-		%LPV1
		!,
		replaceListOfFormulaVariables(LV,F,LPV1,F1),
		toPrologVariables(F1,LPV2,F2),
		append(LPV1,LPV2,NLPV).
toPrologVariables(and(LF),LPV,and(NLF)):-
		!,
		toPrologVariablesListOfFormulas(LF,LPV,NLF).
toPrologVariables(or(LF),LPV,or(NLF)):-
		!,
		toPrologVariablesListOfFormulas(LF,LPV,NLF).
toPrologVariables(implies(F1,F2),NLPV,implies(NF1,NF2)):-
		!,
		toPrologVariables(F1,LPV1,NF1),
		toPrologVariables(F2,LPV2,NF2),
		append(LPV1,LPV2,NLPV).
toPrologVariables(equiv(F1,F2),NLPV,equiv(NF1,NF2)):-
		!,
		toPrologVariables(F1,LPV1,NF1),
		toPrologVariables(F2,LPV2,NF2),
		append(LPV1,LPV2,NLPV).
toPrologVariables(not(F),LPV,not(NF)):-
		!,
		toPrologVariables(F,LPV,NF).
toPrologVariables(F,[],F).

%%% toPrologVariablesListOfFormulas(in LF, out LPV, out NLF)
%%%		LF: a list of formulas
%%%		LPV: a list of Prolog variables
%%%		NLF: the list of formulas that is obtained by replacing formula
%%%				variables with Prolog variables in LF

toPrologVariablesListOfFormulas([],[],[]).
toPrologVariablesListOfFormulas([F|LF],NLPV,[NF|NLF]):-
		toPrologVariables(F,LPV1,NF),
		toPrologVariablesListOfFormulas(LF,LPV2,NLF),
		append(LPV1,LPV2,NLPV).

%%% replaceListOfFormulaVariables(in LV, in F, out LPV, out NF)
%%%		LV: list of variables
%%%		F: a formula
%%%		LPV: a list of Prolog variables
%%%		NF: the formula that is obtained by replacing LV by LPV in F

replaceListOfFormulaVariables([],F,[],F).
replaceListOfFormulaVariables([V|LV],F,[PV|LPV],NF):-
		replaceFormulaVariable(V,PV,F,F1),
		replaceListOfFormulaVariables(LV,F1,LPV,NF).

%%% replaceFormulaVariable(in V, in PV, in F, out NF)
%%%		V: a variable
%%%		PV: a Prolog variable
%%%		F: a formula
%%%		NF: the formula that is obtained by replacing V by PV in F

replaceFormulaVariable(V,PV,'exists*'(LV,F),'exists*'(LV,NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,'forall*'(LV,F),'forall*'(LV,NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,'exists*'(LV,F),'exists*'(LV,NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,'forall*'(LV,F),'forall*'(LV,NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,and(LF),and(NLF)):-
		!,
		replaceFormulaVariableListOfFormulas(V,PV,LF,NLF).
replaceFormulaVariable(V,PV,or(LF),or(NLF)):-
		!,
		replaceFormulaVariableListOfFormulas(V,PV,LF,NLF).
replaceFormulaVariable(V,PV,implies(F1,F2),implies(NF1,NF2)):-
		!,
		replaceFormulaVariable(V,PV,F1,NF1),
		replaceFormulaVariable(V,PV,F2,NF2).
replaceFormulaVariable(V,PV,equiv(F1,F2),equiv(NF1,NF2)):-
		!,
		replaceFormulaVariable(V,PV,F1,NF1),
		replaceFormulaVariable(V,PV,F2,NF2).
replaceFormulaVariable(V,PV,not(F),not(NF)):-
		!,
		replaceFormulaVariable(V,PV,F,NF).
replaceFormulaVariable(V,PV,A,NA):-
		replaceFormulaVariableAtom(V,PV,A,NA).

%%% replaceFormulaVariable(in V, in PV, in F, out NF)
%%%		V: a variable
%%%		PV: a Prolog variable
%%%		F: a formula
%%%		NF: the formula that is obtained by replacing V by PV in F

replaceFormulaVariableListOfFormulas(_,_,[],[]).
replaceFormulaVariableListOfFormulas(V,PV,[F|LF],[NF|NLF]):-
		replaceFormulaVariable(V,PV,F,NF),
		replaceFormulaVariableListOfFormulas(V,PV,LF,NLF).

%%% replaceFormulaVariableAtom(in V, in PV, in A, out NA)
%%%		V: a variable
%%%		PV: a Prolog variable
%%%		A: an atom
%%%		NA: the atom that is obtained by replacing V by PV in A

replaceFormulaVariableAtom(_,_,A,A):-
		var(A),
		!.
replaceFormulaVariableAtom(V,PV,V,PV):-
		!.
replaceFormulaVariableAtom(V,PV,A,NA):-
		A =..[P|ST],
		replaceFormulaVariableSubterms(V,PV,ST,NST),
		NA =..[P|NST].

%%% replaceFormulaVariableSubterms(in V, in PV, in LST, out NLST)
%%%		V: a variable
%%%		PV: a Prolog variable
%%%		LST: a list of subterms
%%%		NLST: the list of subterms that is obtained by replacing V by PV in ST

replaceFormulaVariableSubterms(_,_,[],[]).
replaceFormulaVariableSubterms(V,PV,[ST|LST],[NST|NLST]):-
		replaceFormulaVariableAtom(V,PV,ST,NST),
		replaceFormulaVariableSubterms(V,PV,LST,NLST).
