% %
% % Discard axioms
% %
% %     Axioms with conflictive functions or non-FO axiom are definitively discarded

discardAxioms:-
                findall((F,C),getNumberOfAxioms(F,C),LFC),
                discardAxiomsListOfFiles(LFC).

%%% discardGoal
%%%             Use default file

discardGoal:-
                discardAxiomsFile('goal',0,1).

%%% discardAxiomsListOfFiles(in LFC)
%%%             LFC: list of pairs (F,C) where
%%%                     F is the name of the file
%%%                     C is the number of axioms in F
%%%             Action: axioms containing conflictive functions are discarded

discardAxiomsListOfFiles([]).
discardAxiomsListOfFiles([(F,C)|LFC]):-
                discardAxiomsFile(F,0,C),
                discardAxiomsListOfFiles(LFC).

%%% discardAxiomsFile(in F, in I, in N)
%%%             F: file name
%%%             I: index of the last treated axiom
%%%             N: number of axioms
%%%             Action: axioms containing conflictive functions are discarded

discardAxiomsFile(_,N,N):-
                !.
discardAxiomsFile(F,I,N):-
                NI is I + 1,
                getAxiomKey(F,NI,K),
                !,
                (
                 (
                  1 is NI mod 100,
                  !,
                  print_to_screen('\n'),
                  print_to_screen('\t'),
                  print_to_screen('Current axiom --> '),
                  print_to_screen(K),
                  print_to_screen(' 0')
                 );
                 (
                  1 is NI mod 10,
                  !,
                  NI1 is NI // 10,
                  NI2 is NI1 mod 10,
                  print_to_screen(' '),
                  print_to_screen(NI2)
                 );
                 (
                  true
                 )
                ),
                getAxiom(K,C,A),
                (
                 (
                  C \= 5,
                  !
                 );
                 (
                  C = 5,
                  discardAxiom(A,NC),
                  setAxiom(K,NC,A)
                 )
                ),
                discardAxiomsFile(F,NI,N).
discardAxiomsFile(F,I,_):-
		!,
                NI is I + 1,
                getAxiomKey(F,NI,K),
                print_to_screen('\n'),
                print_to_screen('Error at axiom --> '),
                print_to_screen(K),
                fail.

%%% discardAxiom(in A, out NC)
%%%             A: an axiom
%%%             NC: the new code of the axiom
%%%                 0            if the axiom does not contain conflictive functions and is FO
%%%                 5            otherwise

discardAxiom(A,NC):-
                discardFormula(A,C),
                (
                 (
                  C = 0,
                  !,
                  NC = 0
                 );
                 (
                  NC = 5
                 )
                ).

%%% discardFormula(in F, out C)
%%%             F: a formula
%%%             C: the number of conflictive functions or non-FOL constructs in F

discardFormula(exists(_,F),C):-
                !,
                discardFormula(F,C).
discardFormula(forall(_,F),C):-
                !,
                discardFormula(F,C).
discardFormula('exists*'(_,F),C):-
                !,
                discardFormula(F,C).
discardFormula('forall*'(_,F),C):-
                !,
                discardFormula(F,C).
discardFormula(and(LF),C):-
                !,
                discardListOfFormulas(LF,C).
discardFormula(or(LF),C):-
                !,
                discardListOfFormulas(LF,C).
discardFormula(implies(F1,F2),NC):-
                !,
                discardFormula(F1,C1),
                discardFormula(F2,C2),
                NC is C1 + C2.
discardFormula(equiv(F1,F2),NC):-
                !,
                discardFormula(F1,C1),
                discardFormula(F2,C2),
                NC is C1 + C2.
discardFormula(not(F),C):-
                !,
                discardFormula(F,C).
discardFormula(equal(T1,T2),NC):-
                !,
                discardTerm(T1,C1),
                discardTerm(T2,C2),
                NC is C1 + C2.
discardFormula(A,C):-
                A =..[_|ST],
                discardListOfTerms(ST,C).

%%% discardListOfFormulas(in LF, out C)
%%%             LF: a list of formulas
%%%             C: the number of conflictive functions or non-FOL constructs in LF

discardListOfFormulas([],0).
discardListOfFormulas([F|LF],NC):-
                discardFormula(F,C1),
                discardListOfFormulas(LF,C2),
                NC is C1 + C2.

%%% discardListOfTerms(in LT, out C)
%%%             LT: a list of terms
%%%             C: the number of conflictive functions or non-FOL constructs in LT

discardListOfTerms([],0).
discardListOfTerms([T|LT],NC):-
                discardTerm(T,C1),
                discardListOfTerms(LT,C2),
                NC is C1 + C2.

%%% discardTerm(in T, out C)
%%%             T: a term
%%%             C: the number of conflictive functions or non-FOL constructs in T

discardTerm(T,0):-
                isSingleVariable(T),
                !.
discardTerm(T,0):-
                isRowVariable(T),
                !.
discardTerm(T,0):-
                 atom(T),
                 !.
discardTerm(T,1):-
                 T =..['ListOrderFn'|_],
                 !.
discardTerm(T,1):-
                 T =..['ListFn'|_],
                 !.
discardTerm(T,C):-
                  T =..[F|ST],
                  length(ST,A),
                  isDefinedFunction(F,A),
                  !,
                  discardListOfTerms(ST,C).
discardTerm(_,1).

%%% isDefinedFunction(in F, in A)
%%%              F: a function symbol
%%%              A: a function arity
%%%              Action
%%%                            success        if "F/A" is a defined function
%%%                            failure        if "F/A" is NOT a defined function

isDefinedFunction(F,1):-
                 !,
                 isInstanceOf(F,'UnaryFunction').
isDefinedFunction(F,2):-
                 !,
                 isInstanceOf(F,'BinaryFunction').
isDefinedFunction(F,3):-
                 !,
                 isInstanceOf(F,'TernaryFunction').
isDefinedFunction(F,4):-
                 !,
                 isInstanceOf(F,'QuaternaryFunction').
