% %
% % Expand axioms
% %
% %     1. Row variables and row operators are eliminated
% %     2. Schemas are used to obtain new axioms, relating predicates and constants.

expandAxioms:-
                findall((F,C),numberOfAxioms(F,C),LFC),
                expandAxiomsListOfFiles(LFC).

%%% expandAxiomsListOfFiles(in LFC)
%%%             LFC: list of pairs (F,C) where
%%%                     F is the name of the file
%%%                     C is the number of axioms in F
%%%             Action: row variables and row operators are eliminated
%%%                     schemas are used to obtain new axioms

expandAxiomsListOfFiles([]).
expandAxiomsListOfFiles([(F,C)|LFC]):-
                expandAxiomsFile(F,0,C),
                expandAxiomsListOfFiles(LFC).

%%% expandAxiomsFile(in F, in I, in N)
%%%             F: file name
%%%             I: index of the last treated axiom
%%%             N: number of axioms
%%%             Action: row variables and row operators are eliminated
%%%                     schemas are used to obtain new axioms

expandAxiomsFile(_,N,N):-
                !.
expandAxiomsFile(F,I,N):-
                NI is I + 1,
                getAxiomKey(F,NI,K),
                !,
                (
                 (
                  1 is NI mod 100,
                  !,
                  print_to_screen('\n'),
                  print_to_screen('\t'),
                  print_to_screen('Current axiom --> '),
                  print_to_screen(K),
                  print_to_screen(' 0')
                 );
                 (
                  1 is NI mod 10,
                  !,
                  NI1 is NI // 10,
                  NI2 is NI1 mod 10,
                  print_to_screen(' '),
                  print_to_screen(NI2)
                 );
                 (
                  true
                 )
                ),
                getAxiom(K,C,A),
                (
                 (
                  C = 1,
                  !,
                  NC = 0,
                  expandRowVariablesAxiom(A,A1),
                  removeRowListsAxiom(A1,NA)
                 );
                 (
                  C = 2,
                  !,
                  NC = 0,
                  expandRowVariablesAxiom(A,A1),
                  expandRowOperatorsAxiom(A1,A2),
                  removeRowListsAxiom(A2,NA)
                 );
                 (
                  C = 4,
                  !,
                  NC = C,
                  addSchemaAxiom(A),
                  NA = A
                 );
                  NC = C,
                  NA = A
                ),
                setAxiom(K,NC,NA),
                expandAxiomsFile(F,NI,N).

%%% expandRowVariablesAxiom(in A, out NA)
%%%             A: an axiom
%%%             NA: the axiom after removing row variables

expandRowVariablesAxiom(F,NF):-
                searchRowVariablesFormula(F,F1,LRV),
                instantiateRowVariables(F1,LRV,NF).

%%% searchRowVariablesFormula(in F, out NF, out LRV)
%%%             F: a formula
%%%             NF: the formula that is obtained if we remove row variables and multi-length lists from F.
%%%                     Note that NF contains some Prolog variables that will be instantiated.
%%%             LRV: a list of 3-tuples (RV,PV,LPI) where
%%%                             RV is a row variable
%%%                             PV is the Prolog variable that has replaced RV
%%%                             LPI is a list of (P,I) where P is a predicate and I the argument of P where RV occurs

searchRowVariablesFormula(exists(LV,F),exists(NLV,NF),NLRV):-
                !,
                searchRowVariablesQuantifier(LV,NLV,LRV1),
                searchRowVariablesFormula(F,NF,LRV2),
                joinRowVariables(LRV1,LRV2,NLRV).
searchRowVariablesFormula(forall(LV,F),forall(NLV,NF),NLRV):-
                !,
                searchRowVariablesQuantifier(LV,NLV,LRV1),
                searchRowVariablesFormula(F,NF,LRV2),
                joinRowVariables(LRV1,LRV2,NLRV).
searchRowVariablesFormula('exists*'(LV,F),'exists*'(NLV,NF),NLRV):-
                !,
                searchRowVariablesQuantifier(LV,NLV,LRV1),
                searchRowVariablesFormula(F,NF,LRV2),
                joinRowVariables(LRV1,LRV2,NLRV).
searchRowVariablesFormula('forall*'(LV,F),'forall*'(NLV,NF),NLRV):-
                !,
                searchRowVariablesQuantifier(LV,NLV,LRV1),
                searchRowVariablesFormula(F,NF,LRV2),
                joinRowVariables(LRV1,LRV2,NLRV).
searchRowVariablesFormula(and(LF),and(NLF),NLRV):-
                !,
                searchRowVariablesListOfFormulas(LF,NLF,NLRV).
searchRowVariablesFormula(or(LF),or(NLF),NLRV):-
                !,
                searchRowVariablesListOfFormulas(LF,NLF,NLRV).
searchRowVariablesFormula('@and'(RV,H,S,F),'@and'(PV,H,S,NF),NLRV):-
                isRowVariable(RV),
                !,
                searchRowVariablesFormula(F,NF,LRV2),
                preserveTailRowVariable(S,LRV2,NLRV2),
                joinRowVariables([(RV,PV,[])],NLRV2,NLRV).
searchRowVariablesFormula('@and'(RV,H,S,F),'@and'(RV,H,S,NF),NLRV):-
                !,
                searchRowVariablesFormula(F,NF,LRV),
                preserveTailRowVariable(S,LRV,NLRV).
searchRowVariablesFormula('@or'(RV,H,S,F),'@or'(PV,H,S,NF),NLRV):-
                isRowVariable(RV),
                !,
                searchRowVariablesFormula(F,NF,LRV2),
                preserveTailRowVariable(S,LRV2,NLRV2),
                joinRowVariables([(RV,PV,[])],NLRV2,NLRV).
searchRowVariablesFormula('@or'(RV,H,S,F),'@or'(RV,H,S,NF),NLRV):-
                !,
                searchRowVariablesFormula(F,NF,LRV),
                preserveTailRowVariable(S,LRV,NLRV).
searchRowVariablesFormula(implies(F1,F2),implies(NF1,NF2),NLRV):-
                !,
                searchRowVariablesFormula(F1,NF1,LRV1),
                searchRowVariablesFormula(F2,NF2,LRV2),
                joinRowVariables(LRV1,LRV2,NLRV).
searchRowVariablesFormula(equiv(F1,F2),equiv(NF1,NF2),NLRV):-
                !,
                searchRowVariablesFormula(F1,NF1,LRV1),
                searchRowVariablesFormula(F2,NF2,LRV2),
                joinRowVariables(LRV1,LRV2,NLRV).
searchRowVariablesFormula(not(F),not(NF),NLRV):-
                !,
                searchRowVariablesFormula(F,NF,NLRV).
searchRowVariablesFormula(equal(T1,T2),equal(T1,T2),[]):-
                !.
searchRowVariablesFormula(A,NA,NLRV):-
                A =..[P|LST],
                length(LST,N),
                searchRowVariablesSubterms(P,LST,0,N,NLST,NLRV),
                NA =..[P|NLST].

%%% searchRowVariablesQuantifier(in LV, out NLV, out LRV)
%%%             LV: a list of variables
%%%             NLV: the list of variables that is obtained if we remove row variables and multi-length lists from LV.
%%%                     Note that LV contains some Prolog variables that will be instantiated.
%%%             LRV: a list of 3-tuples (RV,PV,LPI) where
%%%                             RV is a row variable
%%%                             PV is the Prolog variable that has replaced RV
%%%                             LPI is a list of (P,I) where P is a predicate and I the argument of P where RV occurs
%%%                     LPI is the empty list for every 3-tuple in LRV (only in this predicate)

searchRowVariablesQuantifier([],[],[]).
searchRowVariablesQuantifier([V|LV],[NV|NLV],[(V,NV,[])|LRV]):-
                isRowVariable(V),
                !,
                searchRowVariablesQuantifier(LV,NLV,LRV).
searchRowVariablesQuantifier([V|LV],[V|NLV],LRV):-
                isSingleVariable(V),
                searchRowVariablesQuantifier(LV,NLV,LRV).

%%% searchRowVariablesListOfFormulas(in LF, out NLF, out LRV)
%%%             LF: a list of formulas
%%%             NLF: the list of formulas that is obtained if we remove row variables and multi-length lists from LF.
%%%                     Note that NF contains some Prolog variables that will be instantiated.
%%%             LRV: a list of 3-tuples (RV,PV,LPI) where
%%%                             RV is a row variable
%%%                             PV is the Prolog variable that has replaced RV
%%%                             LPI is a list of (P,I) where P is a predicate and I the argument of P where RV occurs

searchRowVariablesListOfFormulas([],[],[]).
searchRowVariablesListOfFormulas([F|LF],[NF|NLF],NLRV):-
                searchRowVariablesFormula(F,NF,LRV1),
                searchRowVariablesListOfFormulas(LF,NLF,LRV2),
                joinRowVariables(LRV1,LRV2,NLRV).

%%% searchRowVariablesSubterms(in P, in LST, in I, in N, out NLST, out LRV)
%%%             P: a predicate
%%%             LST: a list of subterms
%%%             I: index of the last argument that has been treated
%%%             N: number of arguments of P
%%%             NLST: the list that is obtained if we remove row variables and multi-length lists from LST
%%%             LRV: a list of 3-tuples (RV,PV,LPI) where
%%%                             RV is a row variable
%%%                             PV is the Prolog variable that has replaced RV
%%%                             LPI is a list of (P,I) where P is a predicate and I the argument of P where RV occurs

searchRowVariablesSubterms(_,[],N,N,[],[]).
searchRowVariablesSubterms(P,[ST|LST],I,N,[PV|NLST],[(ST,PV,[(P,NI)])|LRV]):-
                isRowVariable(ST),
                !,
                NI is I + 1,
                searchRowVariablesSubterms(P,LST,NI,N,NLST,LRV).
searchRowVariablesSubterms(P,[ST|LST],I,N,[NST|NLST],LRV):-
                isMultiLengthList(ST,NST),
                !,
                NI is I + 1,
                searchRowVariablesSubterms(P,LST,NI,N,NLST,LRV).
searchRowVariablesSubterms(P,[ST|LST],I,N,[ST|NLST],LRV):-
                NI is I + 1,
                searchRowVariablesSubterms(P,LST,NI,N,NLST,LRV).

%%% joinRowVariables(in LRV1, in LRV2, out LRV)
%%%             LRV1, LRV2: list of 3-tuples (RV,PV,LPI) where
%%%                             RV is a row variable
%%%                             PV is the Prolog variable that has replaced RV
%%%                             LPI is a list of (P,I) where P is a predicate and I the argument of P where RV occurs
%%%             NLRV: the list that is obtained from joining LRV1 and LRV2
%%%                     (if RV1 and RV2 are equal, then PV1 and PV2 are unified and LPI1 and LPI2 are appended)

joinRowVariables([],LRV,LRV).
joinRowVariables([(RV,PV,LPI)|LRV1],LRV2,[(RV,PV,NLPI)|NLRV]):-
                joinRowVariable(RV,PV,LRV1,LPI1,NLRV1),
                joinRowVariable(RV,PV,LRV2,LPI2,NLRV2),
                flatten([LPI,LPI1,LPI2],NLPI),
                joinRowVariables(NLRV1,NLRV2,NLRV).

%%% joinRowVariable(in RV, in PV, in LRV, out LPI, out NLRV)
%%%             RV: a row variable
%%%             PV: the Prolog variable that replaces RV
%%%             LRV: a list of 3-tuples (RV,PV,LPI) where
%%%                             RV is a row variable
%%%                             PV is the Prolog variable that has replaced RV
%%%                             LPI is a list of (P,I) where P is a predicate and I the argument of P where RV occurs
%%%             LPI: the list of (P,I) that has been obtained from the 3-tuples we have eliminated in LRV
%%%             NLRV: the 3-tuples that have not been eliminated from LRV

joinRowVariable(_,_,[],[],[]).
joinRowVariable(RV,PV,[(RV,PV,LPI)|LRV],[LPI|NLPI],NLRV):-
                !,
                joinRowVariable(RV,PV,LRV,NLPI,NLRV).
joinRowVariable(RV,PV,[H|LRV],NLPI,[H|NLRV]):-
                joinRowVariable(RV,PV,LRV,NLPI,NLRV).

%%% preserveTailRowVariable(in RV, in LRV, out NLRV)
%%%             RV: a row variable
%%%             LRV: a list of 3-tuples (RV,PV,LPI) where
%%%                             RV' is a row variable
%%%                             PV is the Prolog variable that has replaced RV
%%%                             LPI is a list of (P,I) where P is a predicate and I the argument of P where RV occurs
%%%             NLRV: the list that is obtained from LRV by removing RV
%%%             Action: RV' is unified with PV when RV = RV'

preserveTailRowVariable(_,[],[]).
preserveTailRowVariable(RV,[(RV,PV,_)|LRV],LRV):-
                !,
                RV = PV.
preserveTailRowVariable(RV,[H|LRV],[H|NLRV]):-
                preserveTailRowVariable(RV,LRV,NLRV).

%%% instantiateRowVariables(in F, in LRV, out NF)
%%%             F: a formula
%%%             LRV: a list of 3-tuples (RV,PV,LPI) where
%%%                             RV is a row variable
%%%                             PV is the Prolog variable that has replaced RV
%%%                             LPI is a list of (P,I) where P is a predicate and I the argument of P where RV occurs
%%%             NF: the formula that results from F after instantiating the row variables

instantiateRowVariables(F,[],F).
instantiateRowVariables(F,[(RV,PV,LPI)|LRV],and(LF)):-
                searchAritiesRowVariable(LPI,LA),
                LA \= [],
                !,
                instantiateRowVariableListOfArities(F,RV,PV,LRV,LA,LF).
instantiateRowVariables(F,[(RV,PV,_)|LRV],and(LF)):-                    %%% Clausula para pruebas: este caso no se puede dar!!!
print_to_screen('Warning!!!: Incorrect instantiation of row variables.\n'),
                instantiateRowVariableListOfArities(F,RV,PV,LRV,[2,3,4],LF).

%%% instantiateRowVariableListOfArities(in F, in RV, in PV, in LRV, in LA, out LF)
%%%             F: a formula
%%%             RV: a row variable
%%%             PV: the Prolog variable that replaces RV in F
%%%             LRV: a list of 3-tuples (RV,PV,LPI) where
%%%                             RV is a row variable
%%%                             PV is the Prolog variable that has replaced RV
%%%                             LPI is a list of (P,I) where P is a predicate and I the argument of P where RV occurs
%%%             LA: the list of arities for the instantiation of RV
%%%             NF: the formula that results from F after instantiating the row variables

instantiateRowVariableListOfArities(F,RV,PV,LRV,[A],[NF]):-
                !,
                instantiateRowVariableArity(RV,0,A,PV),
                instantiateRowVariables(F,LRV,NF).
instantiateRowVariableListOfArities(F,RV,PV,LRV,[A|LA],[NF|NLF]):-
                copy_term((F,PV,LRV),(CF,CPV,CLRV)),
                instantiateRowVariableArity(RV,0,A,CPV),
                instantiateRowVariables(CF,CLRV,NF),
                instantiateRowVariableListOfArities(F,RV,PV,LRV,LA,NLF).

%%% instantiateRowVariableArity(in RV, in I, in A, in LV)
%%%             RV: a row variable
%%%             I: the last index that have been treated
%%%             A: arity of the row variable
%%%             LV: the list of variables that instantiates RV

instantiateRowVariableArity(_,A,A,'#row'([])):-
                !.
instantiateRowVariableArity('#rowVar'(V),I,A,'#row'(['#var'(NV)|LV])):-
                NI is I + 1,
                number_codes(NI,LC),
                atom_codes(N,LC),
                atom_concat(V,N,NV),
                instantiateRowVariableArity('#rowVar'(V),NI,A,'#row'(LV)).

%%% searchAritiesRowVariable(in LPI, out LA)
%%%             LPI: a list of pairs (P,I) where
%%%                             P is a predicate
%%%                             I is the index of an argument of P
%%%             LA: list of arities of the I-th argument of P and all the arguments related to it through "relatedArity/2"

searchAritiesRowVariable(LPI,NLA):-
                remove_duplicates(LPI,LPID),
                closureRelatedArity(LPID,NLPID),
                closureRelatedTailArity(NLPID,NLPIDR),
                searchAllAritiesRowVariable(NLPID,LA1),
                searchMaximumArityRowVariable(NLPIDR,A2),
                NA2 is A2 - 1,
                searchAllAritiesFromMaximum(NA2,LA2),           
                union_in_ascending_order(LA1,LA2,NLA).

%%% closureRelatedArity(in LPI, out NLPI)
%%%             LPI: a list of pairs (P,I) where
%%%                             P is a predicate
%%%                             I is the index of an argument of P
%%%             NLPI: the list of pairs (P,I) obtained by the transitive closure of "relatedArity/2" relation on LPI

closureRelatedArity(LPI,NLPI):-
                transitivityRelatedArity(LPI,LPI1),
                delete_list(LPI1,LPI,LPI2),
                LPI2 \= [],
                !,
                append(LPI,LPI2,LPI3),
                closureRelatedArity(LPI3,NLPI).
closureRelatedArity(LPI,LPI).

%%% transitivityRelatedArity(in LPI, out NLPI)
%%%             LPI: a list of pairs (P,I) where
%%%                             P is a predicate
%%%                             I is the index of an argument of P
%%%             NLPI: the list of pairs (P,I) obtained by the transitive (one step) of "relatedArity/2" relation on LPI

transitivityRelatedArity([],[]).
transitivityRelatedArity([(P,I)|LPI],NLPI):-
                findall((NP,NI),relatedArity(P,I,NP,NI),LPI1),
                findall((NP,NI),relatedArity(NP,NI,P,I),LPI2),
                transitivityRelatedArity(LPI,LPI3),
                flatten([LPI1,LPI2,LPI3],LPI4),
                remove_duplicates(LPI4,NLPI).

%%% closureRelatedTailArity(in LPI, out NLPI)
%%%             LPI: a list of pairs (P,I) where
%%%                             P is a predicate
%%%                             I is the index of an argument of P
%%%             NLPI: the list of pairs (P,I) obtained by the transitive closure of "relatedTailArity/2" relation on LPI

closureRelatedTailArity(LPI,NLPI):-
                searchRelatedTailArity(LPI,LPIR),
                closureRelatedArity(LPIR,NLPI).

%%% searchRelatedTailArity(in LPI, out NLPI)
%%%             LPI: a list of pairs (P,I) where
%%%                             P is a predicate
%%%                             I is the index of an argument of P
%%%             NLPI: the list of pairs (P,I) obtained using the relation "relatedTailArity/2" on LPI

searchRelatedTailArity([],[]).
searchRelatedTailArity([(P,I)|LPI],NLPI):-
                findall((NP,NI),relatedTailArity(NP,NI,P,I),LPIR1),
                searchRelatedTailArity(LPI,LPIR2),
                append(LPIR1,LPIR2,LPIR3),
                remove_duplicates(LPIR3,NLPI).

%%% searchAllAritiesRowVariable(in LPI, out LA)
%%%             LPI: a list of pairs (P,I) where
%%%                             P is a predicate
%%%                             I is the index of an argument of P
%%%             LA: list of arities of the I-th argument of P and all the arguments related to it through "relatedArity/2"

searchAllAritiesRowVariable([],[]).
searchAllAritiesRowVariable([(P,I)|LPI],NLA):-
                findall(A,rowArity(P,I,A),LA1),
                sort(LA1,NLA1),
                searchAllAritiesRowVariable(LPI,LA2),
                union_in_ascending_order(LA2,NLA1,NLA).

%%% searchMaximumArityRowVariable(in LPI, out A)
%%%             LPI: a list of pairs (P,I) where
%%%                             P is a predicate
%%%                             I is the index of an argument of P
%%%             A: maximum arity of the I-th argument of P and all the arguments related to it through "relatedArity/2"

searchMaximumArityRowVariable([],0).
searchMaximumArityRowVariable([(P,I)|LPI],M):-
                findall(A,rowArity(P,I,A),LA1),
                max_list(LA1,M1),
                searchMaximumArityRowVariable(LPI,LA2),
                max_list(LA2,M2),
                max_list([M1,M2],M).

%%% searchAllAritiesFromMaximum(in A, out LA)
%%%             A: an arity
%%%             LA: the list of arities from one to A

searchAllAritiesFromMaximum(A,[]):-
                A =< 0,
                !.
searchAllAritiesFromMaximum(A,LA):-
                searchAllAritiesFromMaximum(1,A,LA).

%%% searchAllAritiesFromMaximum(in I, in A, out LA)
%%%             I: index of the current arity
%%%             A: an arity
%%%             LA: the list of arities from one to A

searchAllAritiesFromMaximum(A,A,[A]):-
                !.
searchAllAritiesFromMaximum(I,A,[I|LA]):-
                NI is I + 1,
                searchAllAritiesFromMaximum(NI,A,LA).

%%% removeRowListsAxiom(in F, out NF)
%%%             F: a formula
%%%             NF: the formula that is obtained if we remove multi-length list from F

removeRowListsAxiom(exists(LV,F),exists(NLV,NF)):-
                !,
                removeRowListsFromListOfVariables(LV,NLV),
                removeRowListsAxiom(F,NF).
removeRowListsAxiom(forall(LV,F),forall(NLV,NF)):-
                !,
                removeRowListsFromListOfVariables(LV,NLV),
                removeRowListsAxiom(F,NF).
removeRowListsAxiom('exists*'(LV,F),'exists*'(NLV,NF)):-
                !,
                removeRowListsFromListOfVariables(LV,NLV),
                removeRowListsAxiom(F,NF).
removeRowListsAxiom('forall*'(LV,F),'forall*'(NLV,NF)):-
                !,
                removeRowListsFromListOfVariables(LV,NLV),
                removeRowListsAxiom(F,NF).
removeRowListsAxiom(and(LF),and(NLF)):-
                !,
                removeRowListsFromListOfFormulas(LF,NLF).
removeRowListsAxiom(or(LF),or(NLF)):-
                !,
                removeRowListsFromListOfFormulas(LF,NLF).
removeRowListsAxiom(implies(F1,F2),implies(NF1,NF2)):-
                !,
                removeRowListsAxiom(F1,NF1),
                removeRowListsAxiom(F2,NF2).
removeRowListsAxiom(equiv(F1,F2),equiv(NF1,NF2)):-
                !,
                removeRowListsAxiom(F1,NF1),
                removeRowListsAxiom(F2,NF2).
removeRowListsAxiom(not(F),not(NF)):-
                !,
                removeRowListsAxiom(F,NF).
removeRowListsAxiom(A,NA):-
                removeRowListsAtom(A,NA).

%%% removeRowListsFromListOfVariables(in LV, out NLV)
%%%             LV: a list of (multi-length) lists of variables
%%%             NLV: the list that is obtained from flattening LV

removeRowListsFromListOfVariables([],[]).
removeRowListsFromListOfVariables([H|RLV],NLV):-
                isMultiLengthList(H,LV),
                !,
                removeRowListsFromListOfVariables(RLV,NRLV),
                append(LV,NRLV,NLV).
removeRowListsFromListOfVariables([H|RLV],[H|NRLV]):-
                removeRowListsFromListOfVariables(RLV,NRLV).

%%% removeRowListsFromListOfFormulas(in LF, out NLF)
%%%             LF: a list of formulas
%%%             NLF: the list of formulas that is obtained if we remove multi-length lists from LV

removeRowListsFromListOfFormulas([],[]).
removeRowListsFromListOfFormulas([F|LF],[NF|NLF]):-
                removeRowListsAxiom(F,NF),
                removeRowListsFromListOfFormulas(LF,NLF).

%%% removeRowListsAtom(in A, out NA)
%%%             A: an atom
%%%             NA: the atom that is obtained if we remove multi-length lists from A

removeRowListsAtom(A,NA):-
                A =..[P|ST],
                removeRowListsFromSubterms(ST,NST),
                length(ST,N1),
                flatten(NST,NSTF),
                length(NSTF,N2),
                N1 < N2,
                !,
                number_codes(N2,LC),
                atom_codes(NN,LC),
                atom_concat(P,NN,NP),
                NA =..[NP|NSTF],
                addDomainAxioms(P,NP,N2).
removeRowListsAtom(A,A).

%%% removeRowListsFromSubterms(in ST, out NST)
%%%             ST: a list of subterms
%%%             NST: the list of subterms that is obtained if we remove multi-length lists from ST

removeRowListsFromSubterms([],[]).
removeRowListsFromSubterms([H|ST],[LV|NST]):-
                isMultiLengthList(H,LV),
                !,
                removeRowListsFromSubterms(ST,NST).
removeRowListsFromSubterms([H|ST],[H|NST]):-
                removeRowListsFromSubterms(ST,NST).

%%% expandRowOperatorsAxiom(in F, out NF)
%%%             F: a formula
%%%             NF: the formula that is obtained if we remove row operators from F.

expandRowOperatorsAxiom(exists(LV,F),exists(LV,NF)):-
                !,
                expandRowOperatorsAxiom(F,NF).
expandRowOperatorsAxiom(forall(LV,F),forall(LV,NF)):-
                !,
                expandRowOperatorsAxiom(F,NF).
expandRowOperatorsAxiom('exists*'(LV,F),'exists*'(LV,NF)):-
                !,
                expandRowOperatorsAxiom(F,NF).
expandRowOperatorsAxiom('forall*'(LV,F),'forall*'(LV,NF)):-
                !,
                expandRowOperatorsAxiom(F,NF).
expandRowOperatorsAxiom(and(LF),and(NLF)):-
                !,
                expandRowOperatorsListOfFormulas(LF,NLF).
expandRowOperatorsAxiom(or(LF),or(NLF)):-
                !,
                expandRowOperatorsListOfFormulas(LF,NLF).
expandRowOperatorsAxiom('@and'('#row'(RV),H,S,F),NF):-
                !,
                replaceVariablesWithInFormula(F,H,S,PVH,PVS,F1),
                instantiateRowOperator(RV,F1,PVH,PVS,NLF),
                expandRowOperatorsAxiom(and(NLF),NF).
expandRowOperatorsAxiom('@or'('#row'(RV),H,S,F),NF):-
                !,
                replaceVariablesWithInFormula(F,H,S,PVH,PVS,F1),
                instantiateRowOperator(RV,F1,PVH,PVS,NLF),
                expandRowOperatorsAxiom(or(NLF),NF).
expandRowOperatorsAxiom(implies(F1,F2),implies(NF1,NF2)):-
                !,
                expandRowOperatorsAxiom(F1,NF1),
                expandRowOperatorsAxiom(F2,NF2).
expandRowOperatorsAxiom(equiv(F1,F2),equiv(NF1,NF2)):-
                !,
                expandRowOperatorsAxiom(F1,NF1),
                expandRowOperatorsAxiom(F2,NF2).
expandRowOperatorsAxiom(not(F),not(NF)):-
                !,
                expandRowOperatorsAxiom(F,NF).
expandRowOperatorsAxiom(F,F).

%%% expandRowOperatorsListOfAxioms(in LF, out NLF)
%%%             LF: a list of formulas
%%%             NLF: the list of formulas that is obtained if we remove row operators from LF.

expandRowOperatorsListOfFormulas([],[]).
expandRowOperatorsListOfFormulas([F|LF],[NF|NLF]):-
                expandRowOperatorsAxiom(F,NF),
                expandRowOperatorsListOfFormulas(LF,NLF).

%%% replaceVariablesWithInFormula(in F, in V, in RV, in PV, in PRV, out NF)
%%%             F: a formula
%%%             V: a variable
%%%             RV: a row variable
%%%             PV: a Prolog variable that corresponds with V
%%%             PRV: a Prolog variable that corresponds with RV
%%%             NF: the formula that results from replacing V with PV in F

replaceVariablesWithInFormula(exists(LV,F),V,RV,PV,PRV,exists(NLV,NF)):-
                !,
                replaceVariablesWithInFormula(F,V,RV,PV,PRV,NF),
                replaceVariablesWithInList(LV,V,RV,PV,PRV,NLV).
replaceVariablesWithInFormula(forall(LV,F),V,RV,PV,PRV,forall(NLV,NF)):-
                !,
                replaceVariablesWithInFormula(F,V,RV,PV,PRV,NF),
                replaceVariablesWithInList(LV,V,RV,PV,PRV,NLV).
replaceVariablesWithInFormula('exists*'(LV,F),V,RV,PV,PRV,'exists*'(NLV,NF)):-
                !,
                replaceVariablesWithInFormula(F,V,RV,PV,PRV,NF),
                replaceVariablesWithInList(LV,V,RV,PV,PRV,NLV).
replaceVariablesWithInFormula('forall*'(LV,F),V,RV,PV,PRV,'forall*'(NLV,NF)):-
                !,
                replaceVariablesWithInFormula(F,V,RV,PV,PRV,NF),
                replaceVariablesWithInList(LV,V,RV,PV,PRV,NLV).
replaceVariablesWithInFormula(and(LF),V,RV,PV,PRV,and(NLF)):-
                !,
                replaceVariablesWithInListOfFormulas(LF,V,RV,PV,PRV,NLF).
replaceVariablesWithInFormula(or(LF),V,RV,PV,PRV,or(NLF)):-
                !,
                replaceVariablesWithInListOfFormulas(LF,V,RV,PV,PRV,NLF).
replaceVariablesWithInFormula('@and'(RVA,H,S,F),V,RV,PV,PRV,'@and'(NRV,H,S,NF)):-
                !,
                replaceVariablesWithInTerm(RVA,V,RV,PV,PRV,NRV),
                replaceVariablesWithInFormula(F,V,RV,PV,PRV,NF).
replaceVariablesWithInFormula('@or'(RVA,H,S,F),V,RV,PV,PRV,'@or'(NRV,H,S,NF)):-
                !,
                replaceVariablesWithInTerm(RVA,V,RV,PV,PRV,NRV),
                replaceVariablesWithInFormula(F,V,RV,PV,PRV,NF).
replaceVariablesWithInFormula(implies(F1,F2),V,RV,PV,PRV,implies(NF1,NF2)):-
                !,
                replaceVariablesWithInFormula(F1,V,RV,PV,PRV,NF1),
                replaceVariablesWithInFormula(F2,V,RV,PV,PRV,NF2).
replaceVariablesWithInFormula(equiv(F1,F2),V,RV,PV,PRV,equiv(NF1,NF2)):-
                !,
                replaceVariablesWithInFormula(F1,V,RV,PV,PRV,NF1),
                replaceVariablesWithInFormula(F2,V,RV,PV,PRV,NF2).
replaceVariablesWithInFormula(not(F),V,RV,PV,PRV,not(NF)):-
                !,
                replaceVariablesWithInFormula(F,V,RV,PV,PRV,NF).
replaceVariablesWithInFormula(F,V,RV,PV,PRV,NF):-
                F =..[P|ST],
                replaceVariablesWithInListOfTerms(ST,V,RV,PV,PRV,NST),
                NF =..[P|NST].

%%% replaceVariablesWithInList(in L, in V, in RV, in PV, in PRV, out NL)
%%%             L: a list (of variables)
%%%             V: a variable
%%%             RV: a row variable
%%%             PV: a Prolog variable that corresponds with V
%%%             PRV: a Prolog variable that corresponds with RV
%%%             NL: the list that results from replacing V with PV in L

replaceVariablesWithInList([],_,_,_,_,[]).
replaceVariablesWithInList([V|L],V,RV,PV,PRV,[PV|NL]):-
                !,
                replaceVariablesWithInList(L,V,RV,PV,PRV,NL).
replaceVariablesWithInList([RV|L],V,RV,PV,PRV,[PRV|NL]):-
                !,
                replaceVariablesWithInList(L,V,RV,PV,PRV,NL).
replaceVariablesWithInList([H|L],V,RV,PV,PRV,[H|NL]):-
                replaceVariablesWithInList(L,V,RV,PV,PRV,NL).

%%% replaceVariablesWithInListOfFormulas(in LF, in V, in RV, in PV, in PRV, out NLF)
%%%             F: a list of formulas
%%%             V: a variable
%%%             RV: a row variable
%%%             PV: a Prolog variable that corresponds with V
%%%             PRV: a Prolog variable that corresponds with RV
%%%             NLF: the list of formulas that results from replacing V with PV in LF

replaceVariablesWithInListOfFormulas([],_,_,_,_,[]).
replaceVariablesWithInListOfFormulas([F|LF],V,RV,PV,PRV,[NF|NLF]):-
                replaceVariablesWithInFormula(F,V,RV,PV,PRV,NF),
                replaceVariablesWithInListOfFormulas(LF,V,RV,PV,PRV,NLF).

%%% replaceVariablesWithInTerm(in T, in V, in RV, in PV, in PRV, out NT)
%%%             T: a term
%%%             V: a variable
%%%             RV: a row variable
%%%             PV: a Prolog variable that corresponds with V
%%%             PRV: a Prolog variable that corresponds with RV
%%%             NT: the term that results from replacing V with PV in T

replaceVariablesWithInTerm(V,V,_,PV,_,PV):-
                !.
replaceVariablesWithInTerm(RV,_,RV,_,PRV,PRV):-
                !.
replaceVariablesWithInTerm(T,V,RV,PV,PRV,NT):-
                isMultiLengthList(T,ST),
                !,
                replaceVariablesWithInListOfTerms(ST,V,RV,PV,PRV,NST),
                toMultiLengthList(NST,NT).
replaceVariablesWithInTerm(T,_,_,_,_,T).

%%% replaceVariablesWithInListOfTerms(in LT, in V, in RV, in PV, in PRV, out NLT)
%%%             T: a list of terms
%%%             V: a variable
%%%             RV: a row variable
%%%             PV: a Prolog variable that corresponds with V
%%%             PRV: a Prolog variable that corresponds with RV
%%%             NLT: the list of terms that results from replacing V with PV in LT

replaceVariablesWithInListOfTerms([],_,_,_,_,[]).
replaceVariablesWithInListOfTerms([T|LT],V,RV,PV,PRV,[NT|NLT]):-
                replaceVariablesWithInTerm(T,V,RV,PV,PRV,NT),
                replaceVariablesWithInListOfTerms(LT,V,RV,PV,PRV,NLT).

%%% instantiateRowOperator(in L, in F, in H, in S, out NF)
%%%             L: a list
%%%             F: a formula
%%%             H,S: two Prolog variables occurring in F
%%%             NF: the list of formulas that is obtained from replacing H and S in F with each element in L and eah tail of L

instantiateRowOperator([],_,_,_,[]).
instantiateRowOperator([E],F,E,'#row'([]),[F]):-
                !.
instantiateRowOperator([E|L],F,H,S,[NF|NLF]):-
                copy_term((F,H,S),(NF,NH,NS)),
                NH = E,
                NS = '#row'(L),
                instantiateRowOperator(L,F,H,S,NLF).

%%% addSchemaAxiom(in a)
%%%             A: a schema axiom

addSchemaAxiom('$schema'(C,_,_,VP,F)):-
                replaceVariablePredicateWithInFormula(F,VP,V,NF,LVP),
                getAllInstancesOf(C,LI),
                atom_concat('schema',C,K),
                addNewSchemaAxioms(V,NF,K,0,LVP,LI).

%%% replaceVariablePredicateWithInFormula(in F, in VP, in P, out NF, out LVP)
%%%             F: a formula
%%%             VP: a variable predicate
%%%             V: a Prolog variable for replacing the occurrences of VP in F
%%%             NF: the formula that results from replacing VP in F with V
%%%             LVP: a list of tuples (T,LT) where T is a variable and LT is a list of terms
%%%                     Each T will be unified with =..LT

replaceVariablePredicateWithInFormula(exists(LV,F),VP,V,exists(LV,NF),NLVP):-
                !,
                replaceVariablePredicateWithInFormula(F,VP,V,NF,NLVP).
replaceVariablePredicateWithInFormula(forall(LV,F),VP,V,forall(LV,NF),NLVP):-
                !,
                replaceVariablePredicateWithInFormula(F,VP,V,NF,NLVP).
replaceVariablePredicateWithInFormula('exists*'(LV,F),VP,V,'exists*'(LV,NF),NLVP):-
                !,
                replaceVariablePredicateWithInFormula(F,VP,V,NF,NLVP).
replaceVariablePredicateWithInFormula('forall*'(LV,F),VP,V,'forall*'(LV,NF),NLVP):-
                !,
                replaceVariablePredicateWithInFormula(F,VP,V,NF,NLVP).
replaceVariablePredicateWithInFormula(and(LF),VP,V,and(NLF),NLVP):-
                !,
                replaceVariablePredicateWithInListOfFormulas(LF,VP,V,NLF,NLVP).
replaceVariablePredicateWithInFormula(or(LF),VP,V,or(NLF),NLVP):-
                !,
                replaceVariablePredicateWithInListOfFormulas(LF,VP,V,NLF,NLVP).
replaceVariablePredicateWithInFormula(implies(F1,F2),VP,V,implies(NF1,NF2),NLVP):-
                !,
                replaceVariablePredicateWithInFormula(F1,VP,V,NF1,LVP1),
                replaceVariablePredicateWithInFormula(F2,VP,V,NF2,LVP2),
                append(LVP1,LVP2,NLVP).
replaceVariablePredicateWithInFormula(equiv(F1,F2),VP,V,equiv(NF1,NF2),NLVP):-
                !,
                replaceVariablePredicateWithInFormula(F1,VP,V,NF1,LVP1),
                replaceVariablePredicateWithInFormula(F2,VP,V,NF2,LVP2),
                append(LVP1,LVP2,NLVP).
replaceVariablePredicateWithInFormula(not(F),VP,V,not(NF),NLVP):-
                !,
                replaceVariablePredicateWithInFormula(F,VP,V,NF,NLVP).
replaceVariablePredicateWithInFormula(F,VP,V,NF,[(NF,[V|NST])]):-
                F =..[VP|ST],
                !,
                replaceVariablePredicateWithInListOfTerms(ST,VP,V,NST).
replaceVariablePredicateWithInFormula(F,VP,V,NF,[]):-
                F =..[P|ST],
                replaceVariablePredicateWithInListOfTerms(ST,VP,V,NST),
                NF =..[P|NST].

%%% replaceVariablePredicateWithInListOfFormulas(in LF, in VP, in V, out NLF, out LVP)
%%%             LF: a list of formulas
%%%             VP: a variable predicate
%%%             V: a Prolog variable for replacing the occurrences of VP in LF
%%%             NLF: the list of formulas that results from replacing VP in LF with V
%%%             LVP: a list of tuples (T,LT) where T is a variable and LT is a list of terms
%%%                     Each T will be unified with =..LT

replaceVariablePredicateWithInListOfFormulas([],_,_,[],[]).
replaceVariablePredicateWithInListOfFormulas([F|LF],VP,V,[NF|NLF],NLVP):-
                replaceVariablePredicateWithInFormula(F,VP,V,NF,LVP1),
                replaceVariablePredicateWithInListOfFormulas(LF,VP,V,NLF,LVP2),
                append(LVP1,LVP2,NLVP).

%%% replaceVariablePredicateWithInListOfTerms(in LT, in VP, in V, out NLTF)
%%%             LT: a list of terms
%%%             VP: a variable predicate
%%%             V: a Prolog variable for replacing term occurrences of VP in LT 
%%%             NLT: the formula that results from replacing VP in LT with V

replaceVariablePredicateWithInListOfTerms([],_,_,[]).
replaceVariablePredicateWithInListOfTerms([VP|ST],VP,V,[V|NST]):-
                !,
                replaceVariablePredicateWithInListOfTerms(ST,VP,V,NST).
replaceVariablePredicateWithInListOfTerms([H|ST],VP,V,[H|NST]):-
                replaceVariablePredicateWithInListOfTerms(ST,VP,V,NST).

%%% addNewSchemaAxioms(in V, in F, in K, in I, in LVP, in LO)
%%%             V: a Prolog variable that replaces the variable predicate in NF
%%%             F: a formula where variable predicates have been replaced with V
%%%             K: the key prefix of the new axioms
%%%             I: last index used for keys
%%%             LVP: a list of tuples (T,LT) where T is a variable and LT is a list of terms
%%%                     Each T will be unified with =..LT
%%%             LO: the list of instances of the class
%%%             Action: the axioms that relate predicates and constants are added

addNewSchemaAxioms(_,_,K,I,_,[]):-
                setNumberOfLines(K,I),
                setNumberOfAxioms(K,I).
addNewSchemaAxioms(V,F,K,I,LVP,[O|LO]):-
                copy_term((V,F,LVP),(CV,CF,CLVP)),
                CV = O,
                instantiatePredicateVariables(CLVP),
                NI is I + 1,
                getAxiomKey(K,NI,NK),
                saveAxiom(0,K,NI,NI,NK,CF),
                !,
                addNewSchemaAxioms(V,F,K,NI,LVP,LO).

%%% instantiatePredicateVariables(in LVP)
%%%             LVP: a list of tuples (V,LT) where V is a variable and LT is a list of terms
%%%             Action: each V is unified with =..LT

instantiatePredicateVariables([]).
instantiatePredicateVariables([(V,LT)|LVP]):-
                V =..LT,
                instantiatePredicateVariables(LVP).
