%%% Format axioms

formatAxioms:-
                ontologyList(LF),
                formatFiles(LF).

%%% formatFiles(in LF)
%%%             LF: list of input files (names without extension)

formatFiles([]).
formatFiles([F|LF]):-
                print_to_screen('\t'),
                print_to_screen('Current file --> '),
                print_to_screen(F),
                print_to_screen('\n'),
                atom_concat(F,'.kif',NF),
                atom_concat(F,'.kifx',NFX),
                open(NF, read, S),
                open(NFX, write, SX),
                read_next_line(S,L),
                formatFile(S,SX,1,L),
                close(S),
                close(SX),
                formatFiles(LF).

%%% formatFile(in S, in SX, in N, in L)
%%%             S: stream of the input file
%%%             SX: stream of the output file
%%%             N: index of lines
%%%             L: current line

formatFile(_,_,_,end_of_file):-
                !.
formatFile(S,SX,N,L):-
                % Empty line
                isEmptyLine(L),
                !,
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % Documentation
                L = [CP,CD,CO,CC,CU,CM,CE,CN,CT,CA,CT,CI,CO,CN,CW|RL],
                char_code('(',CP),
                char_code('d',CD),
                char_code('o',CO),
                char_code('c',CC),
                char_code('u',CU),
                char_code('m',CM),
                char_code('e',CE),
                char_code('n',CN),
                char_code('t',CT),
                char_code('a',CA),
                char_code('i',CI),
                char_code(' ',CW),
                !,
                numberOfMarks(RL,I),
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($documentation '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatDocumentation(S,SX,NN,I,NL).
formatFile(S,SX,N,L):-
                % Documentation
                L = [CP,DOLLAR,CD,CO,CC,CU,CM,CE,CN,CT,CA,CT,CI,CO,CN,CW|RL],
                char_code('(',CP),
                char_code('$',DOLLAR),
                char_code('d',CD),
                char_code('o',CO),
                char_code('c',CC),
                char_code('u',CU),
                char_code('m',CM),
                char_code('e',CE),
                char_code('n',CN),
                char_code('t',CT),
                char_code('a',CA),
                char_code('i',CI),
                char_code(' ',CW),
                !,
                numberOfMarks(RL,I),
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($documentation '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatDocumentation(S,SX,NN,I,NL).
formatFile(S,SX,N,L):-
                % Comment
                L = [C|_],
                char_code(';',C),
                !,
                print_to_screen(SX,'\n'),
                atom_codes(LC,L),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatComment(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % subclass
                L = [CPA,CS,CU,CB,CC,CL,CA,CS,CS,CW|RL],
                char_code('(',CPA),
                char_code('s',CS),
                char_code('u',CU),
                char_code('b',CB),
                char_code('c',CC),
                char_code('l',CL),
                char_code('a',CA),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($subclass '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % instance
                L = [CPA,CI,CN,CS,CT,CA,CN,CC,CE,CW|RL],
                char_code('(',CPA),
                char_code('i',CI),
                char_code('n',CN),
                char_code('s',CS),
                char_code('t',CT),
                char_code('a',CA),
                char_code('c',CC),
                char_code('e',CE),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($instance '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % disjoint
                L = [CPA,CD,CI,CS,CJ,CO,CI,CN,CT,CW|RL],
                char_code('(',CPA),
                char_code('d',CD),
                char_code('i',CI),
                char_code('s',CS),
                char_code('j',CJ),
                char_code('o',CO),
                char_code('n',CN),
                char_code('t',CT),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($disjoint '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % subrelation
                L = [CPA,CS,CU,CB,CR,CE,CL,CA,CT,CI,CO,CN,CW|RL],
                char_code('(',CPA),
                char_code('s',CS),
                char_code('u',CU),
                char_code('b',CB),
                char_code('r',CR),
                char_code('e',CE),
                char_code('l',CL),
                char_code('a',CA),
                char_code('t',CT),
                char_code('i',CI),
                char_code('o',CO),
                char_code('n',CN),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($subrelation '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % subAttribute
                L = [CPA,CS,CU,CB,CA,CT,CT,CR,CI,CB,CU,CT,CE,CW|RL],
                char_code('(',CPA),
                char_code('s',CS),
                char_code('u',CU),
                char_code('b',CB),
                char_code('A',CA),
                char_code('t',CT),
                char_code('r',CR),
                char_code('i',CI),
                char_code('t',CT),
                char_code('e',CE),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($subAttribute '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % domain
                L = [CPA,CD,CO,CM,CA,CI,CN,CW|RL],
                char_code('(',CPA),
                char_code('d',CD),
                char_code('o',CO),
                char_code('m',CM),
                char_code('a',CA),
                char_code('i',CI),
                char_code('n',CN),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($domain '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % domainSubclass
                L = [CPA,CD,CO,CM,CA,CI,CN,CSS,CU,CB,CC,CL,CA,CS,CS,CW|RL],
                char_code('(',CPA),
                char_code('d',CD),
                char_code('o',CO),
                char_code('m',CM),
                char_code('a',CA),
                char_code('i',CI),
                char_code('n',CN),
                char_code('S',CSS),
                char_code('u',CU),
                char_code('b',CB),
                char_code('c',CC),
                char_code('l',CL),
                char_code('s',CS),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($domainSubclass '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % range
                L = [CPA,CR,CA,CN,CG,CE,CW|RL],
                char_code('(',CPA),
                char_code('r',CR),
                char_code('a',CA),
                char_code('n',CN),
                char_code('g',CG),
                char_code('e',CE),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($range '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % rangeSubclass
                L = [CPA,CR,CA,CN,CG,CE,CSS,CU,CB,CC,CL,CA,CS,CS,CW|RL],
                char_code('(',CPA),
                char_code('r',CR),
                char_code('a',CA),
                char_code('n',CN),
                char_code('g',CG),
                char_code('e',CE),
                char_code('S',CSS),
                char_code('u',CU),
                char_code('b',CB),
                char_code('c',CC),
                char_code('l',CL),
                char_code('s',CS),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($rangeSubclass '),
                atom_codes(LC,RL),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % partition
                L = [CPA,DOLLAR,CP,CA,CR,CT,CI,CT,CI,CO,CN,CW|RL],
                char_code('(',CPA),
                char_code('$',DOLLAR),
                char_code('p',CP),
                char_code('a',CA),
                char_code('r',CR),
                char_code('t',CT),
                char_code('i',CI),
                char_code('o',CO),
                char_code('n',CN),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($partition '),
                skipWhiteSpaces(RL,NRL1),
                printFormatAtom(SX,NRL1,NRL2),
                print_to_screen(SX,' '),
                skipWhiteSpaces(NRL2,NRL3),
                NRL3 = [CARROBA|_],
                (
                 (
                  char_code('@',CARROBA),
                  !,
                  atom_codes(LC,NRL3),
                  print_to_screen(SX,LC)
                 );
                 (
                  print_to_screen(SX,'@('),
                  printFormatRowList(SX,NRL3),
                  print_to_screen(SX,'))')
                 )
                ),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % partition
                L = [CPA,CP,CA,CR,CT,CI,CT,CI,CO,CN,CW|RL],
                char_code('(',CPA),
                char_code('p',CP),
                char_code('a',CA),
                char_code('r',CR),
                char_code('t',CT),
                char_code('i',CI),
                char_code('o',CO),
                char_code('n',CN),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($partition '),
                skipWhiteSpaces(RL,NRL1),
                printFormatAtom(SX,NRL1,NRL2),
                print_to_screen(SX,' '),
                skipWhiteSpaces(NRL2,NRL3),
                NRL3 = [CARROBA|_],
                (
                 (
                  char_code('@',CARROBA),
                  !,
                  atom_codes(LC,NRL3),
                  print_to_screen(SX,LC)
                 );
                 (
                  print_to_screen(SX,'@('),
                  printFormatRowList(SX,NRL3),
                  print_to_screen(SX,'))')
                 )
                ),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % exhaustiveDecompostion
                L = [CPA,DOLLAR,CE,CX,CH,CA,CU,CS,CT,CI,CV,CE,CD,CE,CC,CO,CM,CP,CO,CS,CI,CT,CI,CO,CN,CW|RL],
                char_code('(',CPA),
                char_code('$',DOLLAR),
                char_code('e',CP),
                char_code('x',CX),
                char_code('h',CH),
                char_code('a',CA),
                char_code('u',CU),
                char_code('s',CS),
                char_code('t',CT),
                char_code('i',CI),
                char_code('v',CV),
                char_code('D',CD),
                char_code('c',CC),
                char_code('o',CO),
                char_code('m',CM),
                char_code('p',CP),
                char_code('n',CN),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($exhaustiveDecomposition '),
                skipWhiteSpaces(RL,NRL1),
                printFormatAtom(SX,NRL1,NRL2),
                print_to_screen(SX,' '),
                skipWhiteSpaces(NRL2,NRL3),
                NRL3 = [CARROBA|_],
                (
                 (
                  char_code('@',CARROBA),
                  !,
                  atom_codes(LC,NRL3),
                  print_to_screen(SX,LC)
                 );
                 (
                  print_to_screen(SX,'@('),
                  printFormatRowList(SX,NRL3),
                  print_to_screen(SX,'))')
                 )
                ),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % exhaustiveDecompostion
                L = [CPA,CE,CX,CH,CA,CU,CS,CT,CI,CV,CE,CD,CE,CC,CO,CM,CP,CO,CS,CI,CT,CI,CO,CN,CW|RL],
                char_code('(',CPA),
                char_code('e',CP),
                char_code('x',CX),
                char_code('h',CH),
                char_code('a',CA),
                char_code('u',CU),
                char_code('s',CS),
                char_code('t',CT),
                char_code('i',CI),
                char_code('v',CV),
                char_code('D',CD),
                char_code('c',CC),
                char_code('o',CO),
                char_code('m',CM),
                char_code('p',CP),
                char_code('n',CN),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($exhaustiveDecomposition '),
                skipWhiteSpaces(RL,NRL1),
                printFormatAtom(SX,NRL1,NRL2),
                print_to_screen(SX,' '),
                skipWhiteSpaces(NRL2,NRL3),
                NRL3 = [CARROBA|_],
                (
                 (
                  char_code('@',CARROBA),
                  !,
                  atom_codes(LC,NRL3),
                  print_to_screen(SX,LC)
                 );
                 (
                  print_to_screen(SX,'@('),
                  printFormatRowList(SX,NRL3),
                  print_to_screen(SX,'))')
                 )
                ),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % disjointDecomposition
                L = [CPA,DOLLAR,CD,CI,CS,CJ,CO,CI,CN,CT,CDD,CE,CC,CO,CM,CP,CO,CS,CI,CT,CI,CO,CN,CW|RL],
                char_code('(',CPA),
                char_code('$',DOLLAR),
                char_code('d',CD),
                char_code('i',CI),
                char_code('s',CS),
                char_code('j',CJ),
                char_code('o',CO),
                char_code('n',CN),
                char_code('t',CT),
                char_code('D',CDD),
                char_code('e',CE),
                char_code('c',CC),
                char_code('m',CM),
                char_code('p',CP),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($disjointDecomposition '),
                skipWhiteSpaces(RL,NRL1),
                printFormatAtom(SX,NRL1,NRL2),
                print_to_screen(SX,' '),
                skipWhiteSpaces(NRL2,NRL3),
                NRL3 = [CARROBA|_],
                (
                 (
                  char_code('@',CARROBA),
                  !,
                  atom_codes(LC,NRL3),
                  print_to_screen(SX,LC)
                 );
                 (
                  print_to_screen(SX,'@('),
                  printFormatRowList(SX,NRL3),
                  print_to_screen(SX,'))')
                 )
                ),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % disjointDecomposition
                L = [CPA,CD,CI,CS,CJ,CO,CI,CN,CT,CDD,CE,CC,CO,CM,CP,CO,CS,CI,CT,CI,CO,CN,CW|RL],
                char_code('(',CPA),
                char_code('d',CD),
                char_code('i',CI),
                char_code('s',CS),
                char_code('j',CJ),
                char_code('o',CO),
                char_code('n',CN),
                char_code('t',CT),
                char_code('D',CDD),
                char_code('e',CE),
                char_code('c',CC),
                char_code('m',CM),
                char_code('p',CP),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'($disjointDecomposition '),
                skipWhiteSpaces(RL,NRL1),
                printFormatAtom(SX,NRL1,NRL2),
                print_to_screen(SX,' '),
                skipWhiteSpaces(NRL2,NRL3),
                NRL3 = [CARROBA|_],
                (
                 (
                  char_code('@',CARROBA),
                  !,
                  atom_codes(LC,NRL3),
                  print_to_screen(SX,LC)
                 );
                 (
                  print_to_screen(SX,'@('),
                  printFormatRowList(SX,NRL3),
                  print_to_screen(SX,'))')
                 )
                ),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % contraryAttribute
                L = [CPA,CC,CO,CN,CT,CR,CA,CR,CY,CAA,CT,CT,CR,CI,CB,CU,CT,CE,CW|RL],
                char_code('(',CPA),
                char_code('c',CC),
                char_code('o',CO),
                char_code('n',CN),
                char_code('t',CT),
                char_code('r',CR),
                char_code('a',CA),
                char_code('y',CY),
                char_code('A',CAA),
                char_code('i',CI),
                char_code('b',CB),
                char_code('u',CU),
                char_code('e',CE),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'(contraryAttribute '),
                skipWhiteSpaces(RL,NRL1),
%                printFormatAtom(SX,NRL1,NRL2),
%                print_to_screen(SX,' '),
%                skipWhiteSpaces(NRL2,NRL3),
%                NRL3 = [CARROBA|_],
                 NRL1 = [CARROBA|_],
                (
                 (
                  char_code('@',CARROBA),
                  !,
%                  atom_codes(LC,NRL3),
                  atom_codes(LC,NRL1),
                  print_to_screen(SX,LC)
                 );
                 (
                  print_to_screen(SX,'@('),
%                  printFormatRowList(SX,NRL3),
                  printFormatRowList(SX,NRL1),
                  print_to_screen(SX,'))')
                 )
                ),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % exhaustiveAttribute
                L = [CPA,CE,CX,CH,CA,CU,CS,CT,CI,CV,CE,CAA,CT,CT,CR,CI,CB,CU,CT,CE,CW|RL],
                char_code('(',CPA),
                char_code('e',CE),
                char_code('x',CX),
                char_code('h',CH),
                char_code('a',CA),
                char_code('u',CU),
                char_code('s',CS),
                char_code('t',CT),
                char_code('i',CI),
                char_code('v',CV),
                char_code('A',CAA),
                char_code('r',CR),
                char_code('b',CB),
                char_code(' ',CW),
                !,
                print_to_screen(SX,'\n'),
                print_to_screen(SX,'(exhaustiveAttribute '),
                skipWhiteSpaces(RL,NRL1),
                printFormatAtom(SX,NRL1,NRL2),
                print_to_screen(SX,' '),
                skipWhiteSpaces(NRL2,NRL3),
                NRL3 = [CARROBA|_],
                (
                 (
                  char_code('@',CARROBA),
                  !,
                  atom_codes(LC,NRL3),
                  print_to_screen(SX,LC)
                 );
                 (
                  print_to_screen(SX,'@('),
                  printFormatRowList(SX,NRL3),
                  print_to_screen(SX,'))')
                 )
                ),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatFile(S,SX,NN,NL).
formatFile(S,SX,N,L):-
                % Axiom
                print_to_screen(SX,'\n'),
                atom_codes(LC,L),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatAxiom(S,SX,NN,NL).

%%% formatDocumentation(in S, in SX, in N, in D, in L)
%%%             S: stream of the input file
%%%             SX: stream of the output file
%%%             N: index of lines
%%%             D: index of dashes
%%%             L: current line

formatDocumentation(S,SX,N,2,L):-
                !,
                print_to_screen(SX,'\n'),
                formatFile(S,SX,N,L).
formatDocumentation(S,SX,N,I,L):-
                I < 2,
                isEmptyLine(L),
                !,
                print_to_screen('Error at line: '),
                print_to_screen(N),
                print_to_screen('\n'),
                close(S),
                close(SX),
                fail.
formatDocumentation(S,SX,N,I,L):-
                I < 2,
                !,
                numberOfMarks(L,J),
                atom_codes(LC,L),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                NI is I + J,
                read_next_line(S,NL),
                NN is N + 1,
                formatDocumentation(S,SX,NN,NI,NL).
formatDocumentation(S,SX,N,_,_):-
                print_to_screen('Error at line: '),
                print_to_screen(N),
                print_to_screen('\n'),
                close(S),
                close(SX),
                fail.

%%% formatComment(in S, in SX, in N, in L)
%%%             S: stream of the input file
%%%             SX: stream of the output file
%%%             N: index of lines
%%%             L: current line

formatComment(S,SX,N,L):-
                % Comment
                L = [C|_],
                char_code(';',C),
                !,
                atom_codes(LC,L),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatComment(S,SX,NN,NL).
formatComment(S,SX,N,L):-
                formatFile(S,SX,N,L).

%%% formatAxiom(in S, in SX, in N, in L)
%%%             S: stream of the input file
%%%             SX: stream of the output file
%%%             N: index of lines
%%%             L: current line

formatAxiom(S,SX,N,end_of_file):-
                !,
                formatFile(S,SX,N,end_of_file).
formatAxiom(S,SX,N,L):-
                % Empty line
                isEmptyLine(L),
                !,
                formatFile(S,SX,N,L).
formatAxiom(S,SX,N,L):-
                % Comment
                L = [C|_],
                char_code(';',C),
                !,
                formatFile(S,SX,N,L).
formatAxiom(S,SX,N,L):-
                % New axiom
                L = [C|_],
                char_code('(',C),
                !,
                formatFile(S,SX,N,L).
formatAxiom(S,SX,N,L):-
                atom_codes(LC,L),
                print_to_screen(SX,LC),
                print_to_screen(SX,'\n'),
                read_next_line(S,NL),
                NN is N + 1,
                formatAxiom(S,SX,NN,NL).

%%% numberOfMarks(in L, out N)
%%%             L: a string
%%%             N: number of dashes '"' in L

numberOfMarks([],0).
numberOfMarks([C|L],NN):-
                char_code('"',C),
                !,
                numberOfMarks(L,N),
                NN is N + 1.
numberOfMarks([_|L],N):-
                numberOfMarks(L,N).

%%% skipWhiteSpaces(in LC, out NLC)
%%%             LC: a list of characters
%%%             NLC: the list of characters obtained from LC by removing the initial white spaces

skipWhiteSpaces([C|LC],NLC):-
                char_code(' ',C),
                !,
                skipWhiteSpaces(LC,NLC).
skipWhiteSpaces(LC,LC).

%%% printFormatAtom(in SX, in LC, out NLC)
%%%             SX: output stream
%%%             LC: a list of characters
%%%             NLC: the list of characters after printing the first atom
%%%             Action: the first atom is printed in SX

printFormatAtom(_,[C|LC],LC):-
                char_code(' ',C),
                !.
printFormatAtom(SX,[C|LC],NLC):-
                char_code(T,C),
                print_to_screen(SX,T),
                printFormatAtom(SX,LC,NLC).

%%% printFormatRowList(in SX, in LC)
%%%             SX: output stream
%%%             LC: a list of characters
%%%             Action: the elements of the row list are printed in SX

printFormatRowList(_,[C|_]):-
                char_code(')',C),
                !.
printFormatRowList(SX,[C|LC]):-
                char_code(' ',C),
                !,
                print_to_screen(SX,','),
                skipWhiteSpaces(LC,NLC),
                printFormatRowList(SX,NLC).
printFormatRowList(SX,[C|LC]):-
                char_code(T,C),
                print_to_screen(SX,T),
                printFormatRowList(SX,LC).
