% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Part III:		Input													% %
% %																				% %
% %				All the predicates to read axioms and store information			% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %


%%% readAxioms
%%%		Action: read axioms from default files

readAxioms:-
		getOntologyList(LF),
		readAxioms(LF).

%%% readAxioms(in LF)
%%%		LF: list of input files (names without extension)

readAxioms([]).
readAxioms([F|LF]):-
		print_to_screen('\t'),
		print_to_screen('Current file --> '),
		print_to_screen(F),
		print_to_screen('\n'),
		atom_concat(F,'.kif',NF),												% Add the extension to the file name
		open(NF, read, S),														% Open the file
		read_next_line(S,L),													% Read the first line of the file
		parseAxioms(F,S,1,1,L),													% Parse the file
		close(S),																% Close the file
		!,
		readAxioms(LF).															% Read the rest of files

%%% parseAxioms(in F, in S, in C, in I, in L)
%%%		F: name of the file
%%%		S: stream of the file
%%%		C: index of axioms
%%%		I: index of the first line of the axiom
%%%		L: first line of the current axiom

parseAxioms(F,S,C,I,L):-
		readAxiom(F,S,I,L,IFL,ICL,NT,A,NL),										% Read axiom
%leash([]),trace,
		parseAxiom(F,S,C,IFL,ICL,NT,A,NL).										% Parse axiom

%%% readAxiom(in F, in S, in I, in L, out IFL, out ICL, out NT, out A, out NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		I: index of lines
%%%		L: first line of the next axiom
%%%		IFL: index of the first line of the axiom
%%%		ICL: index of the last line of the axiom
%%%		NT: text of the axiom
%%%		A: the axiom
%%%		NL: new first line of the next axiom

readAxiom(F,S,I,L,IFL,ICL,NT,A,NL):-
		(
		 (
		  L = end_of_file,														% End of file
		  !,
		  IFL = I,																% Return the index of the first line (any)
		  ICL = I,																% Return the index of the last line (any)
		  NT = [],																% Return the text of the axiom
		  A = end_of_file,														% Return the axiom (end_of_file)
		  NL = []																% Return the first line of the next axiom (empty line)
		 );
		 (
		  isEmptyLine(L),														% Empty line
		  !,
		  NI is I + 1,															% Update the index of lines
		  saveAxiom(-2,F,I,I,text,true),										% Store the line
		  read_next_line(S,RL),													% Read the next line
		  readAxiom(F,S,NI,RL,IFL,ICL,NT,A,NL)									% Recursive call to "read_axiom/9"
		 );
/*		 (
		  L = [C|_],															% Get the first character
		  char_code(';',C),														% Comment
		  !,
		  NI is I + 1,															% Update the index of lines
		  saveAxiom(-1,F,I,I,text,true),										% Store the line
		  read_next_line(S,RL),													% Read next line
		  readAxiom(F,S,NI,RL,IFL,ICL,NT,A,NL)									% Recursive call to "read_axiom/9"
		 );
*/
		 (
		  L = [CP,DOLLAR,CD,CO,CC,CU,CM,CE,CN,CT,CA,CT,CI,CO,CN|_],				% Get the first characters
		  char_code('(',CP),
		  char_code('$',DOLLAR),
		  char_code('d',CD),
		  char_code('o',CO),
		  char_code('c',CC),
		  char_code('u',CU),
		  char_code('m',CM),
		  char_code('e',CE),
		  char_code('n',CN),
		  char_code('t',CT),
		  char_code('a',CA),
		  char_code('i',CI),													% New documentation
		  !,
		  read_rest_of_documentation(F,S,I,[L],L,NI,NT,A,NL),					% Call to "read_rest_of_documentation/9"
		  IFL = I,																% Return the index of the first line of the axiom
		  ICL = NI																% Return the index of the last line of the axiom
		 );
		 (
		  L = [CP,DOLLAR,CC,CO,CM,CM,CE,CN,CT|_],								% Get the first characters
		  char_code('(',CP),
		  char_code('$',DOLLAR),
		  char_code('c',CC),
		  char_code('o',CO),
		  char_code('m',CM),
		  char_code('e',CE),
		  char_code('n',CN),
		  char_code('t',CT),
		  !,
		  read_rest_of_comment(F,S,I,[L],L,NI,NT,A,NL),							% Call to "read_rest_of_documentation/9"
		  IFL = I,																% Return the index of the first line of the axiom
		  ICL = NI																% Return the index of the last line of the axiom
		 );
		 (
		  L = [CP,CT,CE,CR,CM,CF,CO,CR,CM,CA,CT|_],								% Get the first characters
		  char_code('(',CP),
		  char_code('t',CT),
		  char_code('e',CE),
		  char_code('r',CR),
		  char_code('m',CM),
		  char_code('F',CF),
		  char_code('o',CO),
		  char_code('a',CA),
		  char_code('t',CT),													% New documentation -> termFormat
		  !,
		  NI is I + 1,															% Update the index of lines
		  saveAxiom(-3,F,I,I,'$documentation',true),							% Store the line
		  read_next_line(S,RL),													% Read next line
		  readAxiom(F,S,NI,RL,IFL,ICL,NT,A,NL)									% Recursive call to "read_axiom/9"
		 );
		 (
		  L = [CB,CS,CY,CN,CO,CN,CY,CM,CO,CU,CS,CEE,CX,CT,CE,CR,CN,CA,CL,CCC,CO,CN,CC,CE,CP,CT|_],
																				% Get the first characters
		  char_code('(',CB),
		  char_code('s',CS),
		  char_code('y',CY),
		  char_code('n',CN),
		  char_code('o',CO),
		  char_code('m',CM),
		  char_code('u',CU),
		  char_code('s',CS),
		  char_code('E',CEE),
		  char_code('x',CX),
		  char_code('t',CT),
		  char_code('e',CE),
		  char_code('r',CR),
		  char_code('a',CA),
		  char_code('l',CL),
		  char_code('C',CCC),
		  char_code('c',CC),
		  char_code('p',CP),													% New documentation -> synonymousExternalConcept
		  !,
		  NI is I + 1,															% Update the index of lines
		  saveAxiom(-3,F,I,I,'$documentation',true),							% Store the line
		  read_next_line(S,RL),													% Read next line
		  readAxiom(F,S,NI,RL,IFL,ICL,NT,A,NL)									% Recursive call to "read_axiom/9"
		 );
		 (
		  L = [C|_],															% Get the first character
		  char_code('(',C),														% New axiom
		  !,
		  read_rest_of_axiom(F,S,I,[L],L,NI,NT,A,NL),							% Call to "read_rest_of_axiom/9"
		  IFL = I,																% Return the index of the first line of the axiom
		  ICL = NI																% Return the index of the last line of the axiom
		 );
		 (
		  print_to_screen('Incorrect axiom at line: '),									% The axiom is not correct
		  print_to_screen(I),
		  print_to_screen('\n'),
		  fail
		 )
		).

%%% read_rest_of_documentation(in F, in S, in I, in T, in PL, out NI, out NT, out, A, out NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		I: index of the current line of the current axiom
%%%		T: text of the axiom that has been read
%%%		PL: previous lines of the current axiom
%%%		NI: index of the last line of the axiom
%%%		NT: text of axioms
%%%		A: the axiom
%%%		NL: new first line of the next axiom

read_rest_of_documentation(F,S,I,T,PL,NI,NT,A,NL):-
		read_next_line(S,L),													% Read the next line
		(
		 (
		  L = end_of_file,														% End of file
		  !,
		  NI is I,																% Return the index of the last line of the documentation
		  NT = T,																% Return the text of the documentation
		  A = PL,																% Return the documenation
		  NL = L																% Return the first line of the next axiom
		 );
		 (
		  isEmptyLine(L),														% Empty line
		  !,
		  NI is I,																% Return the index of the last line of the documentation
		  NT = T,																% Return the text of the documentation
		  A = PL,																% Return the documentation
		  NL = L																% Return the first line of the next axiom
		 );
		 (
		  RI is I + 1,															% New index of the current line of the documentation
		  append(T,[L],RT),														% Current text of the documenation
		  append(PL,L,NPL),														% Current documentation
		  read_rest_of_documentation(F,S,RI,RT,NPL,NI,NT,A,NL),					% Recursive call to "read_rest_of_documentation/9"
		  !
		 );
		 (
		  print_to_screen('Incorrect axiom at line: '),									% The axiom is not correct
		  print_to_screen(I),
		  fail
		 )
		).

%%% read_rest_of_comment(in F, in S, in I, in T, in PL, out NI, out NT, out, A, out NL)
%%%             F: name of the file
%%%             S: stream of the file
%%%             I: index of the current line of the current axiom
%%%             T: text of the axiom that has been read
%%%             PL: previous lines of the current axiom
%%%             NI: index of the last line of the axiom
%%%             NT: text of axioms
%%%             A: the axiom
%%%             NL: new first line of the next axiom

read_rest_of_comment(F,S,I,T,PL,NI,NT,A,NL):-
		read_next_line(S,L),													% Read the next line
		(
		 (
		  L = [C|_],															% Get the first character
		  char_code(')',C),														% End of comment
		  !,
		  NI is I+1,															% Return the index of the last line of the comment
		  append(T,[L],NT),														% Return the text of the comment
		  append(PL,L,A),														% Return the comment
		  read_next_line(S,NL)													% Return the first line of the next axiom
		 );
		 (
		  L = [C|_],															% Get the first character
		  char_code(';',C),														% Comment
		  RI is I + 1,															% New index of the current line of the comment
		  append(T,[L],RT),														% Current text of the comment
		  append(PL,L,NPL),														% Current comment
		  read_rest_of_comment(F,S,RI,RT,NPL,NI,NT,A,NL),						% Recursive call to "read_rest_of_comment/9"
		  !
		 );
		 (
		  print_to_screen('Incorrect axiom at line: '),									% The axiom is not correct
		  print_to_screen(I),
		  fail
		 )
		).

%%% read_rest_of_axiom(in F, in S, in I, in T, in PL, out NI, out NT, out, A, out NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		I: index of the current line of the current axiom
%%%		T: text of the axiom that has been read
%%%		PL: previous lines of the current axiom
%%%		NI: index of the last line of the axiom
%%%		NT: text of axioms
%%%		A: the axiom
%%%		NL: new first line of the next axiom

read_rest_of_axiom(F,S,I,T,PL,NI,NT,A,NL):-
		read_next_line(S,L),													% Read the next line
		(
		 (
		  L = end_of_file,														% End of file
		  !,
		  NI is I,																% Return the index of the last line of the axiom
		  NT = T,																% Return the text of the axiom
		  A = PL,																% Return the axiom
		  NL = L																% Return the first line of the next axiom
		 );
		 (
		  isEmptyLine(L),														% Empty line
		  !,
		  NI is I,																% Return the index of the last line of the axiom
		  NT = T,																% Return the text of the axiom
		  A = PL,																% Return the axiom
		  NL = L																% Return the first line of the next axiom
		 );
		 (
		  L = [C|_],															% Get the first character
		  char_code('(',C),														% The beginning of a new axiom
		  !,
		  NI is I,																% Return the index of the last line of the axiom
		  NT = T,																% Return the text of the axiom
		  A = PL,																% Return the axiom
		  NL = L																% Return the first line of the next axiom
		 );
		 (
		  L = [C|_],															% Get the first character
		  char_code(';',C),														% Comment
		  !,
		  NI is I,																% Return the index of the last line of the axiom
		  NT = T,																% Return the text of the axiom
		  A = PL,																% Return the axiom
		  NL = L																% Return the first line of the next axiom
		 );
		 (
		  RI is I + 1,															% New index of the current line of the axiom
		  append(T,[L],RT),														% Current text of the axiom
		  append(PL,L,NPL),														% Current axiom
		  read_rest_of_axiom(F,S,RI,RT,NPL,NI,NT,A,NL),							% Recursive call to "read_rest_of_axiom/9"
		  !
		 );
		 (
		  print_to_screen('Incorrect axiom at line: '),									% The axiom is not correct
		  print_to_screen(I),
		  fail
		 )
		).

%%% read_next_line(in S, out L)
%%%		S: stream of the file
%%%		L: next line of the file

read_next_line(S,L):-
		read_line_to_codes(S,L).												% SWI Prolog

%%% parseAxiom(in F, in S, in C, in IFL, in ICL, in T, in A, in NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		C: index of axioms
%%%		IFL: index of the first line of the axiom
%%%		ICL: index of the current line of the current axiom
%%%		T: text of the axiom
%%%		A: the axiom
%%%		NL: first line of the next axiom

parseAxiom(F,_,C,_,L,_,end_of_file,_):-
		!,
		NC is C - 1,
		setNumberOfAxioms(F,NC),
		NL is L - 1,
		setNumberOfLines(F,NL).
parseAxiom(F,S,C,IFL,ICL,_,A,L):-
		formula(LC,NF,A,[]),													% Parse the axiom
		!,
		getAxiomCode(LC,M),														% Get the code assigned to the axiom
		(
		 (
		  M = -4,																% Comment
		  !,
		  saveAxiom(M,F,IFL,ICL,'$comment',true),								% Store the axiom
		  NC = C																% The index of axioms does not change
		 );
		 (
		  M = -3,																% Documentation
		  !,
		  saveAxiom(M,F,IFL,ICL,'$documentation',true),							% Store the axiom
		  NC = C																% The index of axioms does not change
		 );
		 (
		  M >= 0,
		  !,
		  getAxiomKey(F,C,K),													% Assign a key to the axiom
		  saveAxiom(M,F,IFL,ICL,K,NF),											% Store the axiom
		  NC is C + 1															% New index of axioms
		 );
		 (
		  print_to_screen('Error: code is non-valid (have to be treated)'),				% Code non-valid (have to be treated!!!)
		  print_to_screen('\n'),
		  print_to_screen('Lines '),
		  print_to_screen(IFL),
		  print_to_screen('-'),
		  print_to_screen(ICL),
		  fail
		 )
		),
		NI is ICL + 1,															% New index of lines
		!,
		parseAxioms(F,S,NC,NI,L).													% Parse the rest of the file
parseAxiom(_,_,_,IFL,ICL,T,_,_):-
		print_to_screen('Incorrect axiom in lines '),
		print_to_screen(IFL),
		print_to_screen('-'),
		print_to_screen(ICL),
		print_to_screen('\n'),
		print_text_axiom(T),
		fail.

%%% getAxiomCode(in LC, out C)
%%%		LC: a list of codes
%%%		C: the main code

getAxiomCode([],0):-
		!.
getAxiomCode(LC,C):-
		max_list(LC,C).

%%% getAxiomKey(in F, in C, out K)
%%%		F: name of the file
%%%		C: index of axioms
%%%		K: key of the axiom

getAxiomKey(F,C,K):-
		number_codes(C,LC),
		atom_codes('A',[H]),
		atom_codes(NC,[H|LC]),
		atom_concat(F,NC,K).

%%% isEmptyLine(in L)
%%%		L: a list of character codes
%%%		isEmpty(L) succeeds iff either L is empty or all the characters in L are white spaces (' ') and tabs ('\t').

isEmptyLine([]).
isEmptyLine([C|L]):-
		(
		 (
		  char_code(' ',C)
		 );
		 (
		  char_code('\t',C)
		 )
		),
		isEmptyLine(L).


% % % % % % % % % % % % % %
% % % % % % % % % % % % % %
% %						% %
% %		The parser		% %
% %						% %
% % % % % % % % % % % % % %
% % % % % % % % % % % % % %

% % Code table

% -4 -> Comment
% -3 -> Documentation
% -2 -> Empty line
% -1 -> Comment
%  0 -> FO axiom without rowList and rowOperator
%  1 -> FO axiom with rowList
%  2 -> FO axiom with rowOperator
%  3 -> Domain
%  4 -> Schema
%  5 -> Axiom with functions or non-FO axiom

% % Rules for formula % %

formula(LC,F) --> nextToken, "($comment", !, any,
				{ LC = [-4], F = '$comment' }.

formula(LC,F) --> nextToken, "($documentation", !, any,
				{ LC = [-3], F = '$documentation' }.

formula(NLC,NF) --> nextToken, "(", nextToken, "<=>", formula(LC1,F1), formula(LC2,F2), nextToken, ")", !, nextToken, 
				{ append(LC1,LC2,NLC), NF = equiv(F1,F2) }.

formula(NLC,NF) --> nextToken, "(", nextToken, "=>", formula(LC1,F1), formula(LC2,F2), nextToken, ")", !, nextToken, 
				{ append(LC1,LC2,NLC), NF = implies(F1,F2) }.

formula(NLC,NF) --> nextToken, "(", nextToken, "or", formula(LC1,F1), orFormula(LC2,F2), nextToken, ")", !, nextToken, 
				{ append(LC1,LC2,NLC), NF = or([F1|F2]) }.

formula(NLC,NF) --> nextToken, "(", nextToken, "and", formula(LC1,F1), andFormula(LC2,F2), nextToken, ")", !, nextToken, 
				{ append(LC1,LC2,NLC), NF = and([F1|F2]) }.

formula(LC,NF) --> nextToken, "(", nextToken, "@or(", rowVariable(ROW), nextToken, ",", singleVariable(HEAD), nextToken, ",", rowVariable(TAIL), nextToken, ")", formula(_,F), nextToken, ")", !, nextToken, 
				{ LC = [2], NF =..['@or',ROW,HEAD,TAIL,F] }.

formula(LC,NF) --> nextToken, "(", nextToken, "@and(", rowVariable(ROW), nextToken, ",", singleVariable(HEAD), nextToken, ",", rowVariable(TAIL), nextToken, ")", formula(_,F), nextToken, ")", !, nextToken, 
				{ LC = [2], NF =..['@and',ROW,HEAD,TAIL,F] }.

formula(NLC,NF) --> nextToken, "(", nextToken, "not", formula(LC,F), nextToken, ")", !, nextToken, 
				{ NLC = LC, NF = not(F) }.

formula(NLC,NF) --> nextToken, "(", nextToken, "exists*", nextToken, "(", listOfVariables(LC1,LV), nextToken, ")", formula(LC2,F), nextToken, ")", !, nextToken, 
				{ append(LC1,LC2,NLC), NF = 'exists*'(LV,F) }.

formula(NLC,NF) --> nextToken, "(", nextToken, "forall*", nextToken, "(", listOfVariables(LC1,LV), nextToken, ")", formula(LC2,F), nextToken, ")", nextToken, 
				{ append(LC1,LC2,NLC), NF = 'forall*'(LV,F) }.

formula(NLC,NF) --> nextToken, "(", nextToken, "exists", nextToken, "(", listOfVariables(LC1,LV), nextToken, ")", formula(LC2,F), nextToken, ")", !, nextToken, 
				{ append(LC1,LC2,NLC), NF = exists(LV,F) }.

formula(NLC,NF) --> nextToken, "(", nextToken, "forall", nextToken, "(", listOfVariables(LC1,LV), nextToken, ")", formula(LC2,F), nextToken, ")", nextToken, 
				{ append(LC1,LC2,NLC), NF = forall(LV,F) }.

formula(LC,NF) --> nextToken, "(", nextToken, "$domain", constant(R), integer(I), type(T), nextToken, ")", !, nextToken,
				{ LC = [3], I > 0, NF =..['$domain',R,I,T] }.

formula(LC,NF) --> nextToken, "(", nextToken, "$domainSubclass", constant(R), integer(I), type(T), nextToken, ")", !, nextToken,	%%% ONLY FOR SUMO %%%
				{ LC = [3], I > 0, NF =..['$domainSubclass',R,I,T] }.

formula(LC,NF) --> nextToken, "(", nextToken, "$range", constant(R), type(T), nextToken, ")", !, nextToken,	%%% ONLY FOR SUMO %%%
				{ LC = [3], NF =..['$range',R,T] }.

formula(LC,NF) --> nextToken, "(", nextToken, "$rangeSubclass", constant(R), type(T), nextToken, ")", !, nextToken,	%%% ONLY FOR SUMO %%%
				{ LC = [3], NF =..['$rangeSubclass',R,T] }.

formula(LC,NF) --> nextToken, "(", nextToken, "$schema", constant(CLASS), predicate(AUXR), integer(I), varPredicate(R), form(R,F), nextToken, ")", !, nextToken,
				{ LC = [4], I > 0, NF =..['$schema',CLASS,AUXR,I,R,F] }.

formula(NLC,NF) --> nextToken, "(", nextToken, "equal", term(LC1,T1), term(LC2,T2), nextToken, ")", !, nextToken, 
				{ append(LC1,LC2,NLC), NF = equal(T1,T2) }.

formula(LC,F) --> atom(LC,F), !,  nextToken.


%%% Rules for form %%%

form(R,NF) --> nextToken, "(", nextToken, "<=>", form(R,F1), form(R,F2), nextToken, ")", !, nextToken, 
				{ NF = equiv(F1,F2) }.

form(R,NF) --> nextToken, "(", nextToken, "=>", form(R,F1), form(R,F2), nextToken, ")", !, nextToken, 
				{ NF = implies(F1,F2) }.

form(R,NF) --> nextToken, "(", nextToken, "or", form(R,F1), orForm(R,F2), nextToken, ")", !, nextToken, 
				{ NF = or([F1|F2]) }.

form(R,NF) --> nextToken, "(", nextToken, "and", form(R,F1), andForm(R,F2), nextToken, ")", !, nextToken, 
				{ NF = and([F1|F2]) }.

form(R,NF) --> nextToken, "(", nextToken, "not", form(R,F), nextToken, ")", !, nextToken, 
				{ NF = not(F) }.

form(R,NF) --> nextToken, "(", nextToken, "exists*", nextToken, "(", listOfSingleVariables(LV), nextToken, ")", form(R,F), nextToken, ")", !, nextToken, 
				{ NF = 'exists*'(LV,F) }.

form(R,NF) --> nextToken, "(", nextToken, "forall*", nextToken, "(", listOfSingleVariables(LV), nextToken, ")", form(R,F), nextToken, ")", nextToken, 
				{ NF = 'forall*'(LV,F) }.

form(R,NF) --> nextToken, "(", nextToken, "exists", nextToken, "(", listOfSingleVariables(LV), nextToken, ")", form(R,F), nextToken, ")", !, nextToken, 
				{ NF = exists(LV,F) }.

form(R,NF) --> nextToken, "(", nextToken, "forall", nextToken, "(", listOfSingleVariables(LV), nextToken, ")", form(R,F), nextToken, ")", nextToken, 
				{ NF = forall(LV,F) }.

form(R,NF) --> nextToken, "(", nextToken, "equal", nonRowTerm(R,T1), nonRowTerm(R,T2), nextToken, ")", !, nextToken, 
				{ NF = equal(T1,T2) }.

form(R,F) --> nonRowAtom(R,F), !,  nextToken.


%%% Rules for any %%%

any --> [_], !, any.
any --> "".


%%% Rules for nextToken %%%

nextToken --> " ", !, nextToken.

nextToken --> [C],
				{ char_code('\t',C) },
				!, nextToken.

nextToken --> "".


%%% Rules for orFormula %%%

orFormula(NLC,NF) --> formula(LC1,F1), orFormula(LC2,F2), !,
				{ append(LC1,LC2,NLC), NF = [F1|F2] }.

orFormula(LC,[F]) --> formula(LC,F).


%%% Rules for andFormula %%%

andFormula(NLC,NF) --> formula(LC1,F1), andFormula(LC2,F2), !,
				{ append(LC1,LC2,NLC), NF = [F1|F2] }.

andFormula(LC,[F]) --> formula(LC,F).


%%% Rules for orForm %%%

orForm(R,NF) --> form(R,F1), orForm(R,F2), !,
				{ NF = [F1|F2] }.

orForm(R,[F]) --> form(R,F).


%%% Rules for andForm %%%

andForm(R,NF) --> form(R,F1), andForm(R,F2), !,
				{ NF = [F1|F2] }.

andForm(R,[F]) --> form(R,F).


%%% Rules for listOfVariables %%%

listOfVariables(NLC,[V|LV]) --> anyKindOfVariable(LC1,V), !, listOfVariables(LC2,LV),
				{ append(LC1,LC2,NLC) }.

listOfVariables([],[]) --> "".


%%% Rules for listOfSingleVariables %%%

listOfSingleVariables([V|LV]) --> singleVariable(V), !, listOfSingleVariables(LV).

listOfSingleVariables([]) --> "".


%%% Rules for term %%%

term(LC,L) --> nextToken, "@(", constant(T), restOfRowList(R),
				{ L =..['#row',[T|R]], LC = [1] }.

term(LC,V) --> anyKindOfVariable(LC,V), !.

term(LC,C) --> constant(C), !,
				{ addInputConstant(C), LC = [] }.

term(LC,T) --> formula(_,T),
				{ LC = [5] }.


%%% Rules for nonRowTerm %%%

nonRowTerm(_,V) --> singleVariable(V), !.

nonRowTerm(_,C) --> constant(C),
				{ addInputConstant(C) }.

nonRowTerm(R,R) --> varPredicate(R).


%%% Rules for restOfRowList %%%

restOfRowList([]) --> nextToken, ")", !.

restOfRowList([T|R]) --> nextToken, ",", constant(T), restOfRowList(R).


%%% Rules for listOfArguments %%%

listOfArguments(NLC,[T|NT]) --> term(LC1,T), !, listOfArguments(LC2,NT),
				{ append(LC1,LC2,NLC)}.

listOfArguments([],[]) --> "".


%%% Rules for listOfNonRowArguments %%%

listOfNonRowArguments(R,[T|NT]) --> nonRowTerm(R,T), !, listOfNonRowArguments(R,NT).

listOfNonRowArguments(_,[]) --> "".


%%% Rules for atom %%%

atom(NLC,NF) --> nextToken, "(", predicate(P), listOfArguments(NLC,T), nextToken, ")",
				{ length(T,A), addInputPredicate(P,A), NF =..[P|T] }.


%%% Rules for nonRowAtom %%%

nonRowAtom(R,NF) --> nextToken, "(", predicate(P), listOfNonRowArguments(R,T), nextToken, ")",
				{ length(T,A), addInputPredicate(P,A), NF =..[P|T] }.

nonRowAtom(R,NF) --> nextToken, "(", varPredicate(R), listOfNonRowArguments(R,T), nextToken, ")",
				{ NF =..[R|T] }.


%%% Rules for anyKindOfVariable %%%

anyKindOfVariable([],V) --> singleVariable(V).

anyKindOfVariable([1],RV) --> rowVariable(RV).


%%% Rules for singleVariable %%%

singleVariable(V) --> nextToken, [C],
				{ char_code('?',C) },
				[F],
				{ char_code('A',CFirst), char_code('Z',CLast), char_code('_',CDown), ( ( F >= CFirst, F =< CLast, ! ) ; F = CDown ) },
				identifier(I),
				{ atom_codes(N,[F|I]), V =..['#var',N] }.


%%% Rules for rowVariable %%%

rowVariable(RV) --> nextToken, [C],
				{ char_code('@',C) },
				[F],
				{ char_code('A',CFirst), char_code('Z',CLast), char_code('_',CDown), ( ( F >= CFirst, F =< CLast, ! ) ; F = CDown ) },
				identifier(I),
				{ atom_codes(N,[F|I]), RV =..['#rowVar',N] }.


%%% Rules for varPredicate %%%

% varPredicate(VP) --> nextToken, [C],
%				{ char_code('!',C) },
%				identifier(I),
%				{ atom_codes(N,I), VP =..['#pred',N] }.

varPredicate(VP) --> nextToken, [C],
				{ char_code('!',C) },
				identifier(I),
				{ atom_codes(VP,[C|I]) }.

%%% Rules for type %%%

type(T) --> nextToken, [C], 
				{ char_code('@',C) },
				!,
				constant(N),
				{ T =..['#row',N] }.

type(T) --> constant(T).


%%% Rules for constant %%%

constant(K) --> nextToken, [C], 
				{ char_code('a',CFirst), char_code('z',CLast), C >= CFirst, C =< CLast },
				!,
				identifier(LC),
				{ atom_codes(K,[C|LC]) }.
constant(K) --> nextToken, [C], 
				{ char_code('A',CFirst), char_code('Z',CLast), C >= CFirst, C =< CLast },
				!,
				identifier(LC),
				{ atom_codes(K,[C|LC]) }.
constant(K) --> nextToken, [C], 
				{ char_code('0',CFirst), char_code('9',CLast), C >= CFirst, C =< CLast },
				!,
				identifier(LC),
				{ atom_codes(K,[C|LC]) }.
constant(K) --> nextToken, [C], 
				{ char_code('$',C) },
				identifier(LC),
				{ atom_codes(K,[C|LC]) }.

%%% Rules for integer %%%

integer(I) --> listOfDigits([D|RD]),
				{ number_codes(I,[D|RD]) }.


%%% Rules for ListOfDigits %%%

listOfDigits([D|RD]) --> nextToken, [D],
				{ char_code('0',CFirst), char_code('9',CLast), D >= CFirst, D =< CLast },
				!, listOfDigits(RD).

listOfDigits([]) --> "".


%%% Rules for predicate %%%

predicate(P) --> constant(P).


%%% Rules for identifier %%%

identifier([C|LC]) --> [C], 
				{ char_code('a',CFirst), char_code('z',CLast), C >= CFirst, C =< CLast },
				!,
				identifier(LC).
identifier([C|LC]) --> [C], 
				{ char_code('A',CFirst), char_code('Z',CLast), C >= CFirst, C =< CLast },
				!,
				identifier(LC).
identifier([C|LC]) --> [C], 
				{ char_code('0',CFirst), char_code('9',CLast), C >= CFirst, C =< CLast },
				!,
				identifier(LC).
identifier([C|LC]) --> [C], 
				{ char_code('_',C) },
				!,
				identifier(LC).

identifier([]) --> "".


% % % % % % % % % % % % % %
% % % % % % % % % % % % % %
% %						% %
% %		End of parser	% %
% %						% %
% % % % % % % % % % % % % %
% % % % % % % % % % % % % %
