% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Part III:		Input													% %
% %																				% %
% %				All the predicates to read goals and store information			% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %


%%% readGoals
%%%		Action: read goals from default files

readGoals:-
		getGoalList(LF),
		readGoals(LF).

%%% readGoals(in LF)
%%%		LF: list of input files (names without extension)

readGoals([]).
readGoals([F|LF]):-
		print_to_screen('\t'),
		print_to_screen('Current file --> '),
		print_to_screen(F),
		print_to_screen('\n'),
		atom_concat(F,'.kif',NF),												% Add the extension to the file name
		open(NF, read, S),														% Open the file
		read_next_line(S,L),													% Read the first line of the file
		parseGoals(F,S,1,1,L),													% Parse the file
		close(S),																% Close the file
		!,
		readGoals(LF).															% Read the rest of files

%%% parseGoals(in F, in S, in C, in I, in L)
%%%		F: name of the file
%%%		S: stream of the file
%%%		C: index of goals
%%%		I: index of the first line of the goal
%%%		L: first line of the current goal

parseGoals(F,S,C,I,L):-
		readGoal(F,S,I,L,IFL,ICL,NT,A,NL),										% Read goal
		parseGoal(F,S,C,IFL,ICL,NT,A,NL).										% Parse goal

%%% readGoal(in F, in S, in I, in L, out IFL, out ICL, out NT, out A, out NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		I: index of lines
%%%		L: first line of the next goal
%%%		IFL: index of the first line of the goal
%%%		ICL: index of the last line of the goal
%%%		NT: text of the goal
%%%		A: the goal
%%%		NL: new first line of the next goal

readGoal(F,S,I,L,IFL,ICL,NT,A,NL):-
		(
		 (
		  L = end_of_file,														% End of file
		  !,
		  IFL = I,																% Return the index of the first line (any)
		  ICL = I,																% Return the index of the last line (any)
		  NT = [],																% Return the text of the goal
		  A = end_of_file,														% Return the goal (end_of_file)
		  NL = []																% Return the first line of the next goal (empty line)
		 );
		 (
		  isEmptyLine(L),														% Empty line
		  !,
		  NI is I+1,
		  read_next_line(S,RL),													% Read the next line
		  readGoal(F,S,NI,RL,IFL,ICL,NT,A,NL)									% Recursive call to "read_goal/9"
		 );
		 (
		  L = [C|_],															% Get the first character
		  char_code(';',C),														% Comment
		  !,
		  NI is I+1,
		  read_next_line(S,RL),													% Read next line
		  readGoal(F,S,NI,RL,IFL,ICL,NT,A,NL)									% Recursive call to "read_goal/9"
		 );
		 (
		  L = [C|_],															% Get the first character
		  char_code('(',C),														% New goal
		  !,
		  read_rest_of_goal(F,S,I,[L],L,NI,NT,A,NL),							% Call to "read_rest_of_goal/9"
		  IFL = I,																% Return the index of the first line of the goal
		  ICL = NI																% Return the index of the last line of the goal
		 );
		 (
		  print_to_screen('Incorrect goal at line: '),									% The goal is not correct
		  print_to_screen(I),
		  print_to_screen('\n'),
		  fail
		 )
		).

%%% read_rest_of_goal(in F, in S, in I, in T, in PL, out NI, out NT, out, A, out NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		I: index of the current line of the current goal
%%%		T: text of the goal that has been read
%%%		PL: previous lines of the current goal
%%%		NI: index of the last line of the goal
%%%		NT: text of goals
%%%		A: the goal
%%%		NL: new first line of the next goal

read_rest_of_goal(F,S,I,T,PL,NI,NT,A,NL):-
		read_next_line(S,L),													% Read the next line
		(
		 (
		  L = end_of_file,														% End of file
		  !,
		  NI is I,																% Return the index of the last line of the goal
		  NT = T,																% Return the text of the goal
		  A = PL,																% Return the goal
		  NL = L																% Return the first line of the next goal
		 );
		 (
		  isEmptyLine(L),														% Empty line
		  !,
		  NI is I,																% Return the index of the last line of the goal
		  NT = T,																% Return the text of the goal
		  A = PL,																% Return the goal
		  NL = L																% Return the first line of the next goal
		 );
		 (
		  L = [C|_],															% Get the first character
		  char_code('(',C),														% The beginning of a new goal
		  !,
		  NI is I,																% Return the index of the last line of the goal
		  NT = T,																% Return the text of the goal
		  A = PL,																% Return the goal
		  NL = L																% Return the first line of the next goal
		 );
		 (
		  L = [C|_],															% Get the first character
		  char_code(';',C),														% Comment
		  !,
		  NI is I,																% Return the index of the last line of the goal
		  NT = T,																% Return the text of the goal
		  A = PL,																% Return the goal
		  NL = L																% Return the first line of the next goal
		 );
		 (
		  RI is I + 1,															% New index of the current line of the goal
		  append(T,[L],RT),														% Current text of the goal
		  append(PL,L,NPL),														% Current goal
		  read_rest_of_goal(F,S,RI,RT,NPL,NI,NT,A,NL),							% Recursive call to "read_rest_of_goal/9"
		  !
		 );
		 (
		  print_to_screen('Incorrect goal at line: '),									% The goal is not correct
		  print_to_screen(I),
		  fail
		 )
		).

%%% parseGoal(in F, in S, in C, in IFL, in ICL, in T, in A, in NL)
%%%		F: name of the file
%%%		S: stream of the file
%%%		C: index of goals
%%%		IFL: index of the first line of the goal
%%%		ICL: index of the current line of the current goal
%%%		T: text of the goal
%%%		A: the goal
%%%		NL: first line of the next goal

parseGoal(F,_,C,_,_,_,end_of_file,_):-
		!,
		NC is C - 1,
		setNumberOfGoals(F,NC).
parseGoal(F,S,C,IFL,ICL,_,A,L):-
		formula(_,NF,A,[]),														% Parse the goal
		!,
		getGoalKey(F,C,K),														% Assign a key to the goal
		saveGoal(F,IFL,ICL,K,NF),												% Store the goal
		NC is C + 1,															% New index of goals
		NI is ICL + 1,															% New index of lines
		parseGoals(F,S,NC,NI,L).												% Parse the rest of the file
parseGoal(_,_,_,IFL,ICL,T,_,_):-
		print_to_screen('Incorrect goal in lines '),
		print_to_screen(IFL),
		print_to_screen('-'),
		print_to_screen(ICL),
		print_to_screen('\n'),
		print_text_goal(T),
		fail.

%%% getGoalKey(in F, in C, out K)
%%%		F: name of the file
%%%		C: index of goals
%%%		K: key of the goal

getGoalKey(F,C,K):-
		number_codes(C,LC),
		extendToThousand(LC,NLC),
		atom_codes(NC,NLC),
		atom_concat(F,NC,K).

%%% extendToThousand(in LC, out NLC)
%%%		LC: list of characters that represents a number
%%%		NLC: list of characters that represents the same number with at least 4 digits

extendToThousand(LC,LC):-
		length(LC,N),
		N >= 4,
		!.
extendToThousand(LC,NLC):-
		atom_codes('0',[C]),
		extendToThousand([C|LC],NLC).
