% %
% %     Redundant atoms in conjunctions are removed when possible
% %

simplifyAxioms:-
		findall((F,C),getNumberOfAxioms(F,C),LFC),
		simplifyAxiomsListOfFiles(LFC,0).

%%% simplifyAxioms(in M)
%%%		M: mode
%%%			mode 0 -> negation is distributed (default)
%%%			mode 1 -> negation is NOT distributed

simplifyAxioms(M):-
		findall((F,C),getNumberOfAxioms(F,C),LFC),
		simplifyAxiomsListOfFiles(LFC,M).

%%% simplifyGoal
%%%		Use default file

simplifyGoal:-
		simplifyAxiomsFile('goal',0,1,0).

%%% simplifyAxiomsListOfFiles(in LFC, in M)
%%%		LFC: list of pairs (F,C) where
%%%			F is the name of the file
%%%			C is the number of axioms in F
%%%		M: mode
%%%			mode 0 -> negation is distributed (default)
%%%			mode 1 -> negation is NOT distributed
%%%		Action: simplify each axiom in the list of files

simplifyAxiomsListOfFiles([],_).
simplifyAxiomsListOfFiles([(F,C)|LFC],M):-
		simplifyAxiomsFile(F,0,C,M),
		simplifyAxiomsListOfFiles(LFC,M).

%%% simplifyAxiomsFile(in F, in I, in N, in M)
%%%		F: file name
%%%		I: index of the last treated axiom
%%%		N: number of axioms
%%%		M: mode
%%%			mode 0 -> negation is distributed (default)
%%%			mode 1 -> negation is NOT distributed
%%%		Action: simplify each axiom in F

simplifyAxiomsFile(_,N,N,_):-
		!.
simplifyAxiomsFile(F,I,N,M):-
		NI is I + 1,
		getAxiomKey(F,NI,K),
		!,
		(
		 (
		  1 is NI mod 100,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t'),
		  print_to_screen('Current axiom --> '),
		  print_to_screen(K),
		  print_to_screen(' 0')
		 );
		 (
		  1 is NI mod 10,
		  !,
		  NI1 is NI // 10,
		  NI2 is NI1 mod 10,
		  print_to_screen(' '),
		  print_to_screen(NI2)
		 );
		 (
		  true
		 )
		),
		getAxiom(K,C,A),
		(
		 (
		  C >= 0,
		  C =< 2,
		  !,
		  simplifyAxiom(A,M,NA),
		  setAxiom(K,C,NA)
		 );
		 (
		  true
		 )
		),
		simplifyAxiomsFile(F,NI,N,M).

%%% simplifyAxiom(in A, in M, out NA)
%%%		A: an axiom
%%%		M: mode
%%%			mode 0 -> negation is distributed (default)
%%%			mode 1 -> negation is NOT distributed
%%%		NA: the axiom where redundant atoms are removed when possible

% Default

simplifyAxiom(A,NA):-
		simplifyAxiom(A,0,NA).

simplifyAxiom(exists([],F),M,NF):-
		!,
		simplifyAxiom(F,M,NF).
simplifyAxiom(exists(LV,F),M,NF):-
		!,
		simplifyAxiom(F,M,SF),
		simplifyQuantifier(LV,SF,NLV),
		(
		 (
		  NLV = [],
		  !,
		  NF = SF
		 );
		 (
		  NF = exists(NLV,SF)
		 )
		).
simplifyAxiom(forall([],F),M,NF):-
		!,
		simplifyAxiom(F,M,NF).
simplifyAxiom(forall(LV,F),M,NF):-
		!,
		simplifyAxiom(F,M,SF),
		simplifyQuantifier(LV,SF,NLV),
		(
		 (
		  NLV = [],
		  !,
		  NF = SF
		 );
		 (
		  NF = forall(NLV,SF)
		 )
		).
simplifyAxiom('exists*'([],F),M,NF):-
		!,
		simplifyAxiom(F,M,NF).
simplifyAxiom('exists*'(LV,F),M,NF):-
		!,
		simplifyAxiom(F,M,SF),
		simplifyQuantifier(LV,SF,NLV),
		(
		 (
		  NLV = [],
		  !,
		  NF = SF
		 );
		 (
		  NF = 'exists*'(NLV,SF)
		 )
		).
simplifyAxiom('forall*'([],F),M,NF):-
		!,
		simplifyAxiom(F,M,NF).
simplifyAxiom('forall*'(LV,F),M,NF):-
		!,
		simplifyAxiom(F,M,SF),
		simplifyQuantifier(LV,SF,NLV),
		(
		 (
		  NLV = [],
		  !,
		  NF = SF
		 );
		 (
		  NF = 'forall*'(NLV,SF)
		 )
		).
simplifyAxiom(and(LF),M,NF):-
		!,
		simplifyAxiomAnd(LF,M,LF1),
		simplifyListOfAxioms(LF1,M,NLF),
		(
		 (
		  length(NLF,1),
		  !,
		  [NF] = NLF
		 );
		 (
		  NF = and(NLF)
		 )
		).
simplifyAxiom(or(LF),M,NF):-
		!,
		simplifyAxiomOr(LF,M,LF1),
		simplifyListOfAxioms(LF1,M,NLF),
		(
		 (
		  length(NLF,1),
		  !,
		  [NF] = NLF
		 );
		 (
		  NF = or(NLF)
		 )
		).
simplifyAxiom(implies(F1,F2),M,implies(NF1,NF2)):-
		!,
		simplifyAxiom(F1,M,NF1),
		simplifyAxiom(F2,M,NF2).
simplifyAxiom(equiv(F1,F2),M,equiv(NF1,NF2)):-
		!,
		simplifyAxiom(F1,M,NF1),
		simplifyAxiom(F2,M,NF2).
simplifyAxiom(not(F),0,NF):-
		!,
		distributeNegation(F,F1,C),
		(
		 (
		  C > 0,
		  !,
		  simplifyAxiom(F1,0,NF)
		 );
		 (
		  NF = not(F)
		 )
		).
simplifyAxiom(not(F),1,not(NF)):-
		!,
		simplifyAxiom(F,1,NF).
simplifyAxiom(F,_,F).

%%% simplifyQuantifier(in LV, in F, out NLV)
%%%		LV: alist of variables
%%%		F: a formula
%%%		NLV: the list of variables that is obtained by removing from LV the variables non-occurring in F

simplifyQuantifier([],_,[]).
simplifyQuantifier([V|LV],F,[V|NLV]):-
		contains_term(V,F),
		!,
		simplifyQuantifier(LV,F,NLV).
simplifyQuantifier([_|LV],F,NLV):-
		simplifyQuantifier(LV,F,NLV).

%%% simplifyAxiomOr(in LF, in M, out NLF)
%%%		LF: a list of formulas
%%%		M: mode
%%%			mode 0 -> negation is distributed (default)
%%%			mode 1 -> negation is NOT distributed
%%%		NLF: the list that is obtained from simplifying disjunction

simplifyAxiomOr([],_,[]).
simplifyAxiomOr([or(F)|LF],M,NLF):-
		!,
		simplifyAxiomOr(F,M,F1),
		simplifyAxiomOr(LF,M,LF1),
		append(F1,LF1,NLF).
simplifyAxiomOr([F|LF],M,[F|NLF]):-
		simplifyAxiomOr(LF,M,NLF).

%%% simplifyListOfAxioms(in LF, in M, out NLF)
%%%		LF: list of formulas
%%%		M: mode
%%%			mode 0 -> negation is distributed (default)
%%%			mode 1 -> negation is NOT distributed
%%%		NLF: the list of simplified formulas

simplifyListOfAxioms([],_,[]).
simplifyListOfAxioms([F|LF],M,[NF|NLF]):-
		simplifyAxiom(F,M,NF),
		simplifyListOfAxioms(LF,M,NLF).

%%% simplifyAxiomAnd(in LF, in M, out NLF)
%%%		LF: a list of formulas
%%%		M: mode
%%%			mode 0 -> negation is distributed (default)
%%%			mode 1 -> negation is NOT distributed
%%%		NLF: the list that is obtained from simplifying conjunction

simplifyAxiomAnd([],_,[]):-
		!.
simplifyAxiomAnd(LF,M,NLF):-
		simplifyAxiomAnd([],LF,M,NLF).

%%% simplifyAxiomAnd(in LF1, in LF2, in M, out NLF)
%%%		LF1,LF2: lists of formulas
%%%		M: mode
%%%			mode 0 -> negation is distributed (default)
%%%			mode 1 -> negation is NOT distributed
%%%		NLF: the list that is obtained from simplifying LF1 & LF2

simplifyAxiomAnd(LF,[],_,LF).
simplifyAxiomAnd(LF1,['$instance'(I,C)|LF2],M,NLF):-
		!,
		simplifyConjunctionOfFormulas('$instance',I,C,LF2,NLF1,NLF2),
		append(LF1,NLF1,NLF3),
		simplifyAxiomAnd(NLF3,NLF2,M,NLF).
simplifyAxiomAnd(LF1,['$subclass'(SC,C)|LF2],M,NLF):-
		!,
		simplifyConjunctionOfFormulas('$subclass',SC,C,LF2,NLF1,NLF2),
		append(LF1,NLF1,NLF3),
		simplifyAxiomAnd(NLF3,NLF2,M,NLF).
simplifyAxiomAnd(LF1,[F|LF2],M,NLF):-
		append(LF1,[F],NLF3),
		simplifyAxiomAnd(NLF3,LF2,M,NLF).

%%% simplifyConjunctionOfFormulas(in P, in V, in C, in LF, out NLF1, out NLF2)
%%%		P: a predicate symbol
%%%		V: a term (variable expected)
%%%		C: a class
%%%		LF: a list of formulas
%%%		NLF1, NLF2: the list of formulas NLF1 & NLF2 that is obtained from simplifying P(V,C) & LF

simplifyConjunctionOfFormulas(P,V,C,[],[NF],[]):-
		NF =..[P,V,C].
simplifyConjunctionOfFormulas(P,V,C,[F|LF],[],[F|LF]):-
		F =..[P,V,SC],
		isSubclass(SC,C),
		!.
simplifyConjunctionOfFormulas(P,V,C,[F|LF],NLF1,NLF2):-
		F =..[P,V,SC],
		isSubclass(C,SC),
		!,
		simplifyConjunctionOfFormulas(P,V,C,LF,NLF1,NLF2).
simplifyConjunctionOfFormulas(P,V,C,[F|LF],NLF1,[F|NLF2]):-
		simplifyConjunctionOfFormulas(P,V,C,LF,NLF1,NLF2).

%%% distributeNegation(in F, out NF, out T)
%%%		F: a formula
%%%		NF: the formula that is obtained from distributing negation on not F
%%%		T: the number of transformation perfomed for distributing negation

distributeNegation(exists(LV,F),forall(LV,NF),NT):-
		!,
		distributeNegation(F,NF,T),
		NT is T+1.
distributeNegation(forall(LV,F),exists(LV,NF),NT):-
		!,
		distributeNegation(F,NF,T),
		NT is T+1.
distributeNegation('exists*'(LV,F),'forall*'(LV,NF),NT):-
		!,
		distributeNegation(F,NF,T),
		NT is T+1.
distributeNegation('forall*'(LV,F),'exists*'(LV,NF),NT):-
		!,
		distributeNegation(F,NF,T),
		NT is T+1.
distributeNegation(and(LF),or(NLF),NT):-
		!,
		distributeNegationListOfFormulas(LF,NLF,T),
		NT is T+1.
distributeNegation(or(LF),and(NLF),NT):-
		!,
		distributeNegationListOfFormulas(LF,NLF,T),
		NT is T+1.
distributeNegation(implies(F1,F2),and(NLF),NT):-
		!,
		distributeNegationListOfFormulas([not(F1),F2],NLF,T),
		NT is T+1.
distributeNegation(equiv(F1,F2),or([NF1,NF2]),NT):-
		!,
		distributeNegation(implies(F1,F2),NF1,T1),
		distributeNegation(implies(F2,F1),NF2,T2),
		NT is T1+T2+1.
distributeNegation(not(F),F,1):-
		!.
distributeNegation(F,not(F),0).

%%% distributeNegationListOfFormulas(in LF, out NLF, out T)
%%%		LF: a list of formulas
%%%		NLF: the list of formulas that is obtained from distributing negation on not LF
%%%		T: the number of transformation perfomed for distributing negation

distributeNegationListOfFormulas([],[],0).
distributeNegationListOfFormulas([F|LF],[NF|NLF],NT):-
		distributeNegation(F,NF,T1),
		distributeNegationListOfFormulas(LF,NLF,T2),
		NT is T1+T2+1.

%%% distributeQuantifiers(in F, out NF)
%%%		F: a formula
%%%		NF: the formula that is obtained from distributing quantifiers on F

distributeQuantifiers(exists(LV,or(LF)),or(NLF)):-
		!,
		distributeExistsListOfFormulas(LV,LF,NLF).
distributeQuantifiers(exists(LV,implies(F1,F2)),or(NLF)):-
		!,
		distributeExistsListOfFormulas(LV,[not(F1),F2],NLF).
distributeQuantifiers(forall(LV,and(LF)),and(NLF)):-
		!,
		distributeForallListOfFormulas(LV,LF,NLF).
distributeQuantifiers(forall(LV,equiv(F1,F2)),and(NLF)):-
		!,
		distributeForallListOfFormulas(LV,[implies(F1,F2),implies(F2,F1)],NLF).

distributeQuantifiers('exists*'(LV,or(LF)),or(NLF)):-
		!,
		distributeExistsSListOfFormulas(LV,LF,NLF).
distributeQuantifiers('exists*'(LV,implies(F1,F2)),or(NLF)):-
		!,
		distributeExistsSListOfFormulas(LV,[not(F1),F2],NLF).
distributeQuantifiers('forall*'(LV,and(LF)),and(NLF)):-
		!,
		distributeForallSListOfFormulas(LV,LF,NLF).
distributeQuantifiers('forall*'(LV,equiv(F1,F2)),and(NLF)):-
		!,
		distributeForallSListOfFormulas(LV,[implies(F1,F2),implies(F2,F1)],NLF).
distributeQuantifiers(and(LF),and(NLF)):-
		!,
		distributeQuantifiersListOfFormulas(LF,NLF).
distributeQuantifiers(or(LF),or(NLF)):-
		!,
		distributeQuantifiersListOfFormulas(LF,NLF).
distributeQuantifiers(implies(F1,F2),implies(NF1,NF2)):-
		!,
		distributeQuantifiers(F1,NF1),
		distributeQuantifiers(F2,NF2).
distributeQuantifiers(equiv(F1,F2),equiv(NF1,NF2)):-
		!,
		distributeQuantifiers(F1,NF1),
		distributeQuantifiers(F2,NF2).
distributeQuantifiers(not(F),not(NF)):-
		!,
		distributeQuantifiers(F,NF).
distributeQuantifiers(F,F).

%%% distributeExistsListOfFormulas(in LV, in LF, out NLF)
%%%		LV: a list of variables
%%%		LF: a list of formulas
%%%		NLF: the list of formulas that is obtained from distributing quantifiers on exists LV: LF

distributeExistsListOfFormulas(_,[],[]).
distributeExistsListOfFormulas(LV,[F|LF],[NF|NLF]):-
		distributeQuantifiers(exists(LV,F),NF),
		distributeExistsListOfFormulas(LV,LF,NLF).

%%% distributeForallListOfFormulas(in LV, in LF, out NLF)
%%%		LV: a list of variables
%%%		LF: a list of formulas
%%%		NLF: the list of formulas that is obtained from distributing quantifiers on forall LV: LF

distributeForallListOfFormulas(_,[],[]).
distributeForallListOfFormulas(LV,[F|LF],[NF|NLF]):-
		distributeQuantifiers(forall(LV,F),NF),
		distributeForallListOfFormulas(LV,LF,NLF).

%%% distributeExistsSListOfFormulas(in LV, in LF, out NLF)
%%%		LV: a list of variables
%%%		LF: a list of formulas
%%%		NLF: the list of formulas that is obtained from distributing quantifiers on exists* LV: LF

distributeExistsSListOfFormulas(_,[],[]).
distributeExistsSListOfFormulas(LV,[F|LF],[NF|NLF]):-
		distributeQuantifiers('exists*'(LV,F),NF),
		distributeExistsSListOfFormulas(LV,LF,NLF).

%%% distributeForallSListOfFormulas(in LV, in LF, out NLF)
%%%		LV: a list of variables
%%%		LF: a list of formulas
%%%		NLF: the list of formulas that is obtained from distributing quantifiers on forall* LV: LF

distributeForallSListOfFormulas(_,[],[]).
distributeForallSListOfFormulas(LV,[F|LF],[NF|NLF]):-
		distributeQuantifiers('forall*'(LV,F),NF),
		distributeForallSListOfFormulas(LV,LF,NLF).

%%% distributeQuantifiersListOfFormulas(in LF, out NLF)
%%%		LF: a list of formulas
%%%		NLF: the list of formulas that is obtained from distributing quantifiers on LF

distributeQuantifiersListOfFormulas([],[]).
distributeQuantifiersListOfFormulas([F|LF],[NF|NLF]):-
		distributeQuantifiers(F,NF),
		distributeQuantifiersListOfFormulas(LF,NLF).
