% %
% % Type axioms
% %
% %     Type information of variables is included in each axiom

typeAxioms:-
		findall((F,C),(getNumberOfAxioms(F,C), F \= 'automaticCheck'),LFC),
%print_to_screen(LFC),
		typeAxiomsListOfFiles(LFC).

%%% typeAxiomsListOfFiles(in LFC)
%%%		LFC: list of pairs (F,C) where
%%%%			F is the name of the file
%%%%			C is the number of axioms in F
%%%		Action: axioms are typed

typeAxiomsListOfFiles([]).
typeAxiomsListOfFiles([(F,C)|LFC]):-
		typeAxiomsFile(F,0,C),
		typeAxiomsListOfFiles(LFC).

%%% typeAxiomsFile(in F, in I, in N)
%%%		F: file name
%%%		I: index of the last treated axiom
%%%		N: number of axioms
%%%		Action: axioms are typed

typeAxiomsFile(_,N,N):-
		!.
typeAxiomsFile(F,I,N):-
		NI is I + 1,
		getAxiomKey(F,NI,K),
		!,
%print_to_screen(K),
%print_to_screen('\n'),
		(
		 (
		  1 is NI mod 100,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t'),
		  print_to_screen('Current axiom --> '),
		  print_to_screen(K),
		  print_to_screen(' 0')
		 );
		 (
		  1 is NI mod 10,
		  !,
		  NI1 is NI // 10,
		  NI2 is NI1 mod 10,
		  print_to_screen(' '),
		  print_to_screen(NI2)
		 );
		 (
		  true
		 )
		),
/*
(
 (
  K = 'orWhiteBoxA31',
  !,
  leash([]), trace
 );
 (
  notrace
 )
),
*/
		getAxiom(K,C,A),
		(
		 (
		  C \= 0,
		  !
		 );
		 (
		  C = 0,
		  typeAxiom(A,NA),
		  setAxiom(K,C,NA)
		 )
		),
		typeAxiomsFile(F,NI,N).

%%% typeAxiom(in A, out NA)
%%%		A: an axiom
%%%		NA: the typed axiom

typeAxiom(A,NA):-
		typeFormula(A,NA,[]).

%%% typeFormula(in F, out NF, out LVT)
%%%		F: a formula
%%%		NF: the typed formula
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types 

typeFormula(exists(LV,F),'exists*'(LV,NF),NLVT2):-
		!,
		typeFormula(F,F1,LVT),
		extractTypeInformation(LV,LVT,NLVT1,NLVT2),
		variablesTypeFormula(NLVT1,F2),
		(
		 (
		  F2 = [],
		  !,
		  NF = F1
		 );
		 (
		  NF = and([and(F2),F1])
		 )
		).
typeFormula(forall(LV,F),'forall*'(LV,NF),NLVT2):-
		!,
		typeFormula(F,F1,LVT),
		extractTypeInformation(LV,LVT,NLVT1,NLVT2),
		variablesTypeFormula(NLVT1,F2),
		(
		 (
		  F2 = [],
		  !,
		  NF = F1
		 );
		 (
		  NF = implies(and(F2),F1)
		 )
		).
typeFormula('exists*'(LV,F),'exists*'(LV,NF),NLVT):-
		!,
		typeFormula(F,NF,LVT),
		extractTypeInformation(LV,LVT,_,NLVT).
typeFormula('forall*'(LV,F),'forall*'(LV,NF),NLVT):-
		!,
		typeFormula(F,NF,LVT),
		extractTypeInformation(LV,LVT,_,NLVT).
typeFormula(and(LF),and(NLF),NLVT):-
		!,
		typeListOfFormulas(LF,NLF,NLVT).
typeFormula(or(LF),or(NLF),NLVT):-
		!,
		typeListOfFormulas(LF,NLF,NLVT).
typeFormula(implies(F1,F2),implies(NF1,NF2),NLVT):-
		!,
		typeFormula(F1,NF1,LVT1),
		typeFormula(F2,NF2,LVT2),
		joinTypeInformation(LVT1,LVT2,NLVT).
typeFormula(equiv(F1,F2),equiv(NF1,NF2),NLVT):-
		!,
		typeFormula(F1,NF1,LVT1),
		typeFormula(F2,NF2,LVT2),
		joinTypeInformation(LVT1,LVT2,NLVT).
typeFormula(not(F),not(NF),NLVT):-
		!,
		typeFormula(F,NF,NLVT).
%typeFormula(equal(T1,T2),equal(T1,T2),[]):-
%		!.
typeFormula(equal(T1,T2),equal(T1,T2),NLVT):-
		!,
		typeEqualFormula(T1,T2,NLVT).
typeFormula(A,A,NLVT):-
		A =..[P|ST],
		length(ST,N),
		typeListOfTerms(ST,P,0,N,NLVT).

%%% extractTypeInformation(in LV, in LVT, out NLVT1, out NLVT2)
%%%		LV: a list of variables
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types
%%%		NLVT1: the list of pairs (V,LT) from LVT where V belongs to LV
%%%		NLVT2: the list of pairs (V,LT) from LVT where V does not belong to LV

extractTypeInformation(_,[],[],[]).
extractTypeInformation(LV,[(V,T)|LVT],[(V,T)|NLVT1],NLVT2):-
		member(V,LV),
		!,
		extractTypeInformation(LV,LVT,NLVT1,NLVT2).
extractTypeInformation(LV,[VT|LVT],NLVT1,[VT|NLVT2]):-
		extractTypeInformation(LV,LVT,NLVT1,NLVT2).

%%% variablesTypeFormula(in LVT, out LF)
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types
%%%		LF: the list of formula that is obtained from LVT

variablesTypeFormula([],[]).
variablesTypeFormula([(V,LT)|LVT],NLF):-
		variableTypeFormula(V,LT,LF1),
		variablesTypeFormula(LVT,LF2),
		append(LF1,LF2,NLF).

%%% variableTypeFormula(in V, in LT, out LF)
%%%		V: a variable
%%%		LT: a list of types
%%%		LF: the list of formula that is obtained from V and LT

variableTypeFormula(_,[],[]).
%variableTypeFormula(V,[('$object','$class')|LT],[F|LF]):-
variableTypeFormula(V,[('$object','$class')|LT],LF):-
		!,
		variableTypeFormula(V,LT,LF).
variableTypeFormula(V,[('$object','$object')|LT],LF):-
		!,
		variableTypeFormula(V,LT,LF).
variableTypeFormula(V,[('$object','$ObjectOrClass')|LT],LF):-
		!,
		variableTypeFormula(V,LT,LF).

variableTypeFormula(V,[('$object','Entity')|LT],LF):-
		!,
		variableTypeFormula(V,LT,LF).

variableTypeFormula(V,[('$object',T)|LT],[F|LF]):-
		!,
		F =..['$instance',V,T],
		variableTypeFormula(V,LT,LF).
variableTypeFormula(V,[('$class',T)|LT],[F|LF]):-
		F =..['$subclass',V,T],
		variableTypeFormula(V,LT,LF).

%%% typeListOfFormulas(in LF, out NLF, out LVT)
%%%		LF: a list of formulas
%%%		NLF: the list of formulas that is obtained from typing LF
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types

typeListOfFormulas([],[],[]).
typeListOfFormulas([F|LF],[NF|NLF],NLVT):-
		typeFormula(F,NF,LVT1),
		typeListOfFormulas(LF,NLF,LVT2),
		joinTypeInformation(LVT1,LVT2,NLVT).

%%% typeEqualFormula(in T1, in T2, out LVT)
%%%		T1, T2: terms
%%%		NLF: the list of formulas that is obtained from typing LF
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types

typeEqualFormula(T1,T2,NLVT):-
		isSingleVariable(T1),
		compound(T2),
		T2 =..[F|_],
		getRange(F,T,0),
		!,
		typeTerm(T2,LVT),
		addTypeInformation(T1,('$object',T),LVT,NLVT).
typeEqualFormula(T1,T2,NLVT):-
		isSingleVariable(T1),
		compound(T2),
		T2 =..[F|_],
		getRangeSubclass(F,T,0),
		!,
		typeTerm(T2,LVT),
		addTypeInformation(T1,('$class',T),LVT,NLVT).
typeEqualFormula(T1,T2,NLVT):-
		isSingleVariable(T2),
		compound(T1),
		T1 =..[F|_],
		getRange(F,T,0),
		!,
		typeTerm(T1,LVT),
		addTypeInformation(T2,('$object',T),LVT,NLVT).
typeEqualFormula(T1,T2,NLVT):-
		isSingleVariable(T2),
		compound(T1),
		T1 =..[F|_],
		getRangeSubclass(F,T,0),
		!,
		typeTerm(T1,LVT),
		addTypeInformation(T2,('$class',T),LVT,NLVT).
typeEqualFormula(T1,T2,NLVT):-
		typeTerm(T1,LVT1),
		typeTerm(T2,LVT2),
		joinTypeInformation(LVT1,LVT2,NLVT).

%%% typeTerm(in T, out LVT)
%%%		T: term
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types

typeTerm(T,[]):-
		isSingleVariable(T),
		!.
%typeTerm(T,[]):-
%		ground(T),
%		!.
typeTerm(T,LVT):-
		compound(T),
		!,
		T =..[FS|NLT],
		length(NLT,NN),
		typeListOfSubterms(NLT,FS,0,NN,LVT).
typeTerm(_,[]).

%%% typeListOfTerms(in LT, in P, in I, in N, out NLVT)
%%%		LT: a list of terms
%%%		P: a predicate
%%%		I: index of the last argument that has been treated
%%%		N: number of arguments of P
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types

typeListOfTerms([],_,N,N,[]).
typeListOfTerms([V|LT],P,I,N,NLVT):-
		isSingleVariable(V),
		NI is I + 1,
		getDomain(P,NI,T,0),
		!,
		typeListOfTerms(LT,P,NI,N,LVT),
		addTypeInformation(V,('$object',T),LVT,NLVT).
typeListOfTerms([V|LT],P,I,N,NLVT):-
		isSingleVariable(V),
		NI is I + 1,
		getDomainSubclass(P,NI,T,0),
		!,
		typeListOfTerms(LT,P,NI,N,LVT),
		addTypeInformation(V,('$class',T),LVT,NLVT).
typeListOfTerms([T|LT],P,I,N,NLVT):-
		compound(T),
		!,
		T =..[FS|NLT],
		length(NLT,NN),
		typeListOfSubterms(NLT,FS,0,NN,LVT1),
		NI is I + 1,
		typeListOfTerms(LT,P,NI,N,LVT2),
		joinTypeInformation(LVT1,LVT2,NLVT).
typeListOfTerms([_|LT],P,I,N,NLVT):-
		NI is I + 1,
		typeListOfTerms(LT,P,NI,N,NLVT).

%%% typeListOfSubterms(in LT, in F, in I, in N, out NLVT)
%%%		LT: a list of subterms
%%%		F: a function
%%%		I: index of the last argument that has been treated
%%%		N: number of arguments of F
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types

typeListOfSubterms([],_,N,N,[]).
typeListOfSubterms([V|LT],P,I,N,NLVT):-
		isSingleVariable(V),
		NI is I + 1,
		getDomain(P,NI,T,0),
		!,
		typeListOfSubterms(LT,P,NI,N,LVT),
		addTypeInformation(V,('$object',T),LVT,NLVT).
typeListOfSubterms([V|LT],P,I,N,NLVT):-
		isSingleVariable(V),
		NI is I + 1,
		getDomainSubclass(P,NI,T,0),
		!,
		typeListOfSubterms(LT,P,NI,N,LVT),
		addTypeInformation(V,('$class',T),LVT,NLVT).
typeListOfSubterms([T|LT],P,I,N,NLVT):-
		compound(T),
		!,
		T =..[FS|NLT],
		length(NLT,NN),
		typeListOfSubterms(NLT,FS,0,NN,LVT1),
		NI is I + 1,
		typeListOfTerms(LT,P,NI,N,LVT2),
		joinTypeInformation(LVT1,LVT2,NLVT).
typeListOfSubterms([_|LT],P,I,N,NLVT):-
		NI is I + 1,
		typeListOfSubterms(LT,P,NI,N,NLVT).

%%% joinTypeInformation(in LVT1, in LVT2, out NLVT)
%%%		LVT1,LVT2: lists of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types
%%%		NLVT: the list that is obtained from joining LVT1 and LVT2

joinTypeInformation(LVT,[],LVT).
joinTypeInformation(LVT1,[(V,LT)|LVT2],NLVT):-
		addListOfTypeInformation(V,LT,LVT1,NLVT1),
		joinTypeInformation(NLVT1,LVT2,NLVT).

%%% addListOfTypeInformation(in V, in LT, in LVT, out NLVT)
%%%		V: a single variable
%%%		LT: a list of types
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types
%%%		NLVT: the list that is obtained from adding to LVT the type information in LT

addListOfTypeInformation(_,[],LVT,LVT).
addListOfTypeInformation(V,[T|LT],LVT,NLVT):-
		addTypeInformation(V,T,LVT,LVT1),
		addListOfTypeInformation(V,LT,LVT1,NLVT).

%%% addTypeInformation(in V, in T, in LVT, out NLVT)
%%%		V: a single variable
%%%		T: a type
%%%		LVT: a list of pairs (V,LT) where
%%%				V is a single variable
%%%				LT is a list of types
%%%		NLVT: the list that is obtained from adding to LVT the type information in T

addTypeInformation(V,T,[],[(V,[T])]).
addTypeInformation(V,T,[(V,LT)|LVT],[(V,NLT)|LVT]):-
		!,
		addTypeInformationToList(T,LT,NLT).
addTypeInformation(V,T,[H|LVT],[H|NLVT]):-
		addTypeInformation(V,T,LVT,NLVT).

%%% addTypeInformationToList(in T, in LT, out NLT)
%%%		T: a type
%%%		LT: a list of types
%%%		NLT: the list of types that is obtained from adding T to LT

addTypeInformationToList(T,[],[T]).
addTypeInformationToList(T,[T|LT],[T|LT]):-
		!.
addTypeInformationToList((M,T),[(M,ST)|LT],[(M,ST)|LT]):-
		isSubclass(ST,T),
		!.
addTypeInformationToList((M,ST),[(M,T)|LT],[(M,ST)|LT]):-
		isSubclass(ST,T),
		!.
addTypeInformationToList(T,[H|LT],[H|NLT]):-
		addTypeInformationToList(T,LT,NLT).
