% %
% % Transfomation 2: verify Axioms
% %
% %	1. constant and predicate symbols are disjoint
% %	2. each predicate is used with just one arity
% %	3. "$schema/5" are verified: the auxiliar relation cannot be defined in the ontology and can only be used once
% %	4. all defined predicates have domain information
% %	5. all the constants have been defined
% %	6. predicates are predefined or defined according to one schema
% %	7. all the variables are quantified (only closed formulas are allowed), but only once
% %	8. row variables are placed in argument with row type
% %	9. in each argument with row type either a row variable or multi-length list occurs

verifyAxioms(NW):-
		% Check 1: constant and predicate symbols are disjoint
		findall((S,A),(getInputPredicate(S,A),getInputConstant(S),not(isSchemaDefinedPredicate(S,A))),L1),
		length(L1,W1),
		(
		 (
		  W1 = 0
		 );
		 (
		  W1 \= 0,
		  print_to_screen('\n\t\t'),
		  print_to_screen('Error: the following symbols are used both as predicate and constant symbols --> '),
		  print_to_screen('\n'),
		  writePredicateAndConstantSymbols(L1)
		 )
		),
		% Check 2: each predicate is used with just one arity
		findall((S,A1,A2),(getInputPredicate(S,A1),getInputPredicate(S,A2), A1 < A2),L2),
		length(L2,W2),
		(
		 (
		  W2 = 0
		 );
		 (
		  W2 \= 0,
		  print_to_screen('\n\t\t'),
		  print_to_screen('Error: the following predicate symbols are used with several arities --> '),
		  print_to_screen('\n'),
		  writePredicateWithSeveralArities(L2)
		 )
		),
		% Check 3: the auxiliar relation in "$schema/5" cannot be defined in the ontology and can only be used once
		getSchemaAuxiliarRelations(LSA),
		(
		 (
		  no_doubles(LSA),
		  !,
		  W31 = 0
		 );
		 (
		  repeated_elements(LSA,RE),
		  W31 = 1,
		  print_to_screen('\n\t\t'),
		  print_to_screen('Error: the following relations are used in several schemas -->  '),
		  writeSchemaRepeatedAuxiliarRelations(RE)
		 )
		),
		verifySchemaRelations(LSA,W32),
		W3 is W31 + W32,

		% %
		% % initialize internal information
		% %
		print_to_screen('\n'),
		print_to_screen('\t'),
		print_to_screen('Initializing internal information ... '),
		initializeInternalInformation,
%trace,
%fail,
		print_to_screen('Done!'),
		
		% %
		% % discarding  conflictive axioms
		% %
		print_to_screen('\n'),
		print_to_screen('\t'),
		print_to_screen('Discarding non-FO axioms ...'),
		discardAxioms,
		print_to_screen('Done!'),
		print_to_screen('\n'),
		print_to_screen('\n'),
		findall((F,C),numberOfAxioms(F,C),LFC),
		verifyAxiomsListOfFiles(LFC,LW),

		NW is W1 + W2 + W3 + LW.

%%% verifyGoal(out NW)
%%%		   NW: the number of errors
%%%		   Use default file

verifyGoal(NW):-
		verifyAxiomsFile('goal',0,1,NW).

%%% verifySchemaRelations(in LSA, out W)
%%%		   LSA: a list of schema auxiliar relations
%%%		   W: the number of errors in LAR

verifySchemaRelations([],0).
verifySchemaRelations([(S,A)|LSA],NW):-
		(
		 (
		  isClass(S)
		 );
		 (
		  isObject(S)
		 )
		),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: schema relations cannot be defined in the ontology --> '),
		print_to_screen(S),
		print_to_screen('(/'),
		print_to_screen(A),
		print_to_screen(')'),
		print_to_screen('\n'),
		verifySchemaRelations(LSA,W),
		NW is W + 1.
verifySchemaRelations([_|LAR],W):-
		verifySchemaRelations(LAR,W).

%%% writePredicateAndConstantSymbols(in L)
%%%		   L: a list of pairs (S,A) where S is a symbol and A an arity
%%%		   Action: the pairs are prompted

writePredicateAndConstantSymbols([]).
writePredicateAndConstantSymbols([(S,A)|L]):-
		print_to_screen('\t'),
		print_to_screen(S),
		print_to_screen('\\'),
		print_to_screen(A),
		print_to_screen(' and '),
		print_to_screen(S),
		print_to_screen('\0'),
		print_to_screen('\n'),
		writePredicateAndConstantSymbols(L).

%%% writePredicateWithSeveralArities(in L)
%%%		   L: a list of 3-tuples (S,A1,A2) where S is a symbol and both A1 and A2 are arities
%%%		   Action: the 3-tuples are prompted

writePredicateWithSeveralArities([]).
writePredicateWithSeveralArities([(S,A1,A2)|L]):-
		print_to_screen('\t'),
		print_to_screen(S),
		print_to_screen('\\'),
		print_to_screen(A1),
		print_to_screen(' and '),
		print_to_screen(S),
		print_to_screen('\\'),
		print_to_screen(A2),
		print_to_screen('\n'),
		writePredicateWithSeveralArities(L).

%%% writeSchemaRepeatedAuxiliarRelations(in LSA)
%%%		   LSA: list of schema repeated auxiliar relations
%%%		   Action: the error message is prompted

writeSchemaRepeatedAuxiliarRelations([]).
writeSchemaRepeatedAuxiliarRelations([(S,A)|L]):-
		print_to_screen('\t'),
		print_to_screen(S),
		print_to_screen('/'),
		print_to_screen(A),
		print_to_screen('\n'),
		writeSchemaRepeatedAuxiliarRelations(L).

%%% verifyAxiomsListOfFiles(in LFC, out W)
%%%		   LFC: list of pairs (F,C) where
%%%		F is the name of the file
%%%		C is the number of axioms in F
%%%		   W: the number of errors in axioms
%%%		   Action: verify each axiom in the list of files

verifyAxiomsListOfFiles([],0).
verifyAxiomsListOfFiles([(F,C)|LFC],NW):-
		verifyAxiomsFile(F,0,C,W1),
		verifyAxiomsListOfFiles(LFC,W2),
		NW is W1 + W2.

%%% verifyAxiomsFile(in F, in I, in N, out W)
%%%		   F: file name
%%%		   I: index of the last treated axiom
%%%		   N: number of axioms
%%%		   W: the number of errors in axioms
%%%		   Action: verify each axiom in F

verifyAxiomsFile(_,N,N,0):-
		!.
verifyAxiomsFile(F,I,N,NW):-
		NI is I + 1,
		getAxiomKey(F,NI,K),
		!,
		M is NI mod 10,
		(
		 (
		  M = 1,
		  !,
		  print_to_screen('\n'),
		  print_to_screen('\t'),
		  print_to_screen('Current axiom (verifying) --> '),
		  print_to_screen(K),
		  print_to_screen(' ')
		 );
		 (
		  print_to_screen(M),
		  print_to_screen(' ')
		 )
		),
/*
(
 (
  K = merge195A81,
  !,
  trace
 );
 (
  true
 )
),
*/
		getCompleteAxiom(K,C,F,FL,LL,A),
		(
		 (
		  C >= 0,
		  C =< 2,
		  !,
		  verifyAxiom(A,F,FL,LL,[],W1,[],[]),
		  (
		   (
		    W1 = 0,
		    !
		   );
		   (
		    setAxiom(K,6,A)
		   )
		  )
		 );
		 (
		  C = 4,
		  !,
		  A ='$schema'(CL,AUXR,IAux,VP,FS),
		  verifySchema(CL,FS,AUXR,IAux,VP,F,FL,LL,W1),
		  (
		   (
		    W1 = 0,
		    !
		   );
		   (
		    setAxiom(K,6,A)
		   )
		  )
		 );
		 (
		  C = 3,				% Domain information
		  W1 = 0
		 );
		 (
		  C = 5,				% Axiom with conflictive functions or non-FO axiom
		  W1 = 0
		 )
		),
		verifyAxiomsFile(F,NI,N,W2),
		NW is W1 + W2.
verifyAxiomsFile(F,I,_):-
		!,
                NI is I + 1,
                getAxiomKey(F,NI,K),
                print_to_screen('\n'),
                print_to_screen('Error (verifying) at axiom --> '),
                print_to_screen(K),
                fail.

%%% verifyAxiom(in A, in F, in FL, in LL, in LV, out W, out LRVP, out LRT)
%%%		   A: an axiom
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the axiom A
%%%		   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs
%%%		   LRT: a list of pairs (RV,RVT) where
%%%		RV is a row variable that is used in a row operator
%%%		RVT is the tail row variable of the row operator where RV occurs
%%%		   Action: the following checks are performed
%%%		5. all the constants have been defined
%%%		6. predicates are predefined or defined according to one schema
%%%		7. all the variables are quantified (only closed formulas are allowed), but only once
%%%		8. row variables are placed in argument with row type
%%%		9. in each argument with row type either a row variable or multi-length list occurs

verifyAxiom(exists(LVE,F),FILE,FL,LL,LV,NW,NLRVP,NLRT):-
		disjoint_lists(LVE,LV),
		!,
		(
		 (
		  no_doubles(LVE),				    % Exists: correct
		  !,
		  append(LV,LVE,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,LRVP,LRT),
		  relateRowVariables(LVE,LRVP,LRT,NLRVP,NLRT),
		  NW = W
		 );
		 (
		  repeated_elements(LVE,RE),			    % ERROR: repeated variables in quantifier
		  remove_duplicates(RE,NRE),
		  writeRepeatedVariables(FILE,FL,LL,NRE),
		  remove_duplicates(LVE,NLVE),
		  append(LV,NLVE,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 1
		 )
		).
verifyAxiom(exists(LVE,F),FILE,FL,LL,LV,NW,[],[]):-
		(
		 (
		  no_doubles(LVE),				    % ERROR: variables already quantified
		  !,
		  shared_elements(LV,LVE,SE),
		  writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		  append(LV,LVE,LRV),
		  remove_duplicates(LRV,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 1
		 );
		 (
		  !,
		  repeated_elements(LVE,RE),			    % ERROR: repeated variables in quantifier and variables already quantified
		  remove_duplicates(RE,NRE),
		  writeRepeatedVariables(FILE,FL,LL,NRE),
		  remove_duplicates(LVE,NLVE),
		  shared_elements(LV,NLVE,SE),
		  writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		  append(LV,NLVE,LRV),
		  remove_duplicates(LRV,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 2
		 )
		).
verifyAxiom(forall(LVE,F),FILE,FL,LL,LV,NW,NLRVP,NLRT):-
		disjoint_lists(LVE,LV),
		!,
		(
		 (
		  no_doubles(LVE),				    % For all: correct
		  !,
		  append(LV,LVE,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,LRVP,LRT),
		  relateRowVariables(LVE,LRVP,LRT,NLRVP,NLRT),
		  NW = W
		 );
		 (
		  repeated_elements(LVE,RE),			    % ERROR: repeated variables in quantifier
		  remove_duplicates(RE,NRE),
		  writeRepeatedVariables(FILE,FL,LL,NRE),
		  remove_duplicates(LVE,NLVE),
		  append(LV,NLVE,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 1
		 )
		).
verifyAxiom(forall(LVE,F),FILE,FL,LL,LV,NW,[],[]):-
		(
		 (
		  no_doubles(LVE),				    % ERROR: variables already quantified
		  !,
		  shared_elements(LV,LVE,SE),
		  writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		  append(LV,LVE,LRV),
		  remove_duplicates(LRV,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 1
		 );
		 (
		  !,
		  repeated_elements(LVE,RE),			    % ERROR: repeated variables in quantifier and variables already quantified
		  remove_duplicates(RE,NRE),
		  writeRepeatedVariables(FILE,FL,LL,NRE),
		  remove_duplicates(LVE,NLVE),
		  shared_elements(LV,NLVE,SE),
		  writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		  append(LV,NLVE,LRV),
		  remove_duplicates(LRV,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 2
		 )
		).
verifyAxiom('exists*'(LVE,F),FILE,FL,LL,LV,NW,NLRVP,NLRT):-
		disjoint_lists(LVE,LV),
		!,
		(
		 (
		  no_doubles(LVE),				    % Exists *: correct
		  !,
		  append(LV,LVE,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,LRVP,LRT),
		  relateRowVariables(LVE,LRVP,LRT,NLRVP,NLRT),
		  NW = W
		 );
		 (
		  repeated_elements(LVE,RE),			    % ERROR: repeated variables in quantifier
		  remove_duplicates(RE,NRE),
		  writeRepeatedVariables(FILE,FL,LL,NRE),
		  remove_duplicates(LVE,NLVE),
		  append(LV,NLVE,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 1
		 )
		).
verifyAxiom('exists*'(LVE,F),FILE,FL,LL,LV,NW,[],[]):-
		(
		 (
		  no_doubles(LVE),				    % ERROR: variables already quantified
		  !,
		  shared_elements(LV,LVE,SE),
		  writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		  append(LV,LVE,LRV),
		  remove_duplicates(LRV,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 1
		 );
		 (
		  !,
		  repeated_elements(LVE,RE),			    % ERROR: repeated variables in quantifier and variables already quantified
		  remove_duplicates(RE,NRE),
		  writeRepeatedVariables(FILE,FL,LL,NRE),
		  remove_duplicates(LVE,NLVE),
		  shared_elements(LV,NLVE,SE),
		  writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		  append(LV,NLVE,LRV),
		  remove_duplicates(LRV,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 2
		 )
		).
verifyAxiom('forall*'(LVE,F),FILE,FL,LL,LV,NW,NLRVP,NLRT):-
		disjoint_lists(LVE,LV),
		!,
		(
		 (
		  no_doubles(LVE),				    % For all *: correct
		  !,
		  append(LV,LVE,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,LRVP,LRT),
		  relateRowVariables(LVE,LRVP,LRT,NLRVP,NLRT),
		  NW = W
		 );
		 (
		  repeated_elements(LVE,RE),			    % ERROR: repeated variables in quantifier
		  remove_duplicates(RE,NRE),
		  writeRepeatedVariables(FILE,FL,LL,NRE),
		  remove_duplicates(LVE,NLVE),
		  append(LV,NLVE,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 1
		 )
		).
verifyAxiom('forall*'(LVE,F),FILE,FL,LL,LV,NW,[],[]):-
		(
		 (
		  no_doubles(LVE),				    % ERROR: variables already quantified
		  !,
		  shared_elements(LV,LVE,SE),
		  writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		  append(LV,LVE,LRV),
		  remove_duplicates(LRV,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 1
		 );
		 (
		  !,
		  repeated_elements(LVE,RE),			    % ERROR: repeated variables in quantifier and variables already quantified
		  remove_duplicates(RE,NRE),
		  writeRepeatedVariables(FILE,FL,LL,NRE),
		  remove_duplicates(LVE,NLVE),
		  shared_elements(LV,NLVE,SE),
		  writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		  append(LV,NLVE,LRV),
		  remove_duplicates(LRV,NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,W,_,_),
		  NW is W + 2
		 )
		).
verifyAxiom(and(LF),FILE,FL,LL,LV,NW,LRVP,LRT):-			% And: correct
		!,
		verifyListOfAxioms(LF,FILE,FL,LL,LV,NW,LRVP,LRT).
verifyAxiom(or(LF),FILE,FL,LL,LV,NW,LRVP,LRT):-			 % Or: correct
		!,
		verifyListOfAxioms(LF,FILE,FL,LL,LV,NW,LRVP,LRT).
verifyAxiom('@and'(RV,H,S,F),FILE,FL,LL,LV,NW,LRVP,NLRT):-		 % @and: correct
		isRowVariable(RV),
		isSingleVariable(H),
		(
		 (
		  isAnonymousRowVariable(S),
		  member(RV,LV),
		  non_member(H,LV),
		  !,
		  append(LV,[H],NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,NW,LRVP,NLRT)
		 );
		 (
		  isRowVariable(S),
		  member(RV,LV),
		  non_member(H,LV),
		  non_member(S,LV),
		  !,
		  append(LV,[H,S],NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,NW,LRVP,LRT),
		  NLRT = [(RV,S)|LRT]
		 )
		).


verifyAxiom('@and'(L,H,S,F),FILE,FL,LL,LV,NW,LRVP,LRT):-		   % @and: correct
		isMultiLengthList(L,LE),
		isSingleVariable(H),
		(
		 (
		  isAnonymousRowVariable(S),
		  non_member(H,LV),
		  !,
		  append(LV,[H],NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,NW,LRVP,LRT)
		 );
		 (
		  isRowVariable(S),
		  non_member(H,LV),
		  !,
		  append(LV,[H],NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,NW,LRVP,LRT),
		  length(LE,N),
		  addTailRowVariableArities(S,LRVP,N)
		 )
		).
verifyAxiom('@and'(RV,H,S,F),FILE,FL,LL,LV,NW,[],[]):-			% ERROR
		!,
		writeIfNotRowVariable(FILE,FL,LL,RV,W1),		   % Row operator is not applied to a row variable
		writeIfNotSingleVariable(FILE,FL,LL,H,W2),		 % The head of the row variable is not a variable
		writeIfisRowVariable(FILE,FL,LL,S,W3),			% The tail of the row variable is not a variable
		writeIfNotQuantified(FILE,FL,LL,RV,LV,W4),		 % The row variable is not quantified
		writeIfQuantified(FILE,FL,LL,H,LV,W5),			% The head of the row variable is already quantified
		writeIfQuantified(FILE,FL,LL,S,LV,W6),			% The tail of the row variable is already quantified
		append(LV,[H,S],LRV),
		remove_duplicates(LRV,NLV),
		verifyAxiom(F,FILE,FL,LL,NLV,W7,_,_),
		NW is W1 + W2 + W3 + W4 + W5 + W6 + W7.
verifyAxiom('@or'(RV,H,S,F),FILE,FL,LL,LV,NW,LRVP,NLRT):-		  % @or: correct
		isRowVariable(RV),
		isSingleVariable(H),
		(
		 (
		  isAnonymousRowVariable(S),
		  member(RV,LV),
		  non_member(H,LV),
		  !,
		  append(LV,[H],NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,NW,LRVP,NLRT)
		 );
		 (
		  isRowVariable(S),
		  member(RV,LV),
		  non_member(H,LV),
		  non_member(S,LV),
		  !,
		  append(LV,[H,S],NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,NW,LRVP,LRT),
		  NLRT = [(RV,S)|LRT]
		 )
		).
verifyAxiom('@or'(L,H,S,F),FILE,FL,LL,LV,NW,LRVP,LRT):-		    % @or: correct
		isMultiLengthList(L,LE),
		isSingleVariable(H),
		(
		 (
		  isAnonymousRowVariable(S),
		  non_member(H,LV),
		  !,
		  append(LV,[H],NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,NW,LRVP,LRT)
		 );
		 (
		  isRowVariable(S),
		  non_member(H,LV),
		  !,
		  append(LV,[H],NLV),
		  verifyAxiom(F,FILE,FL,LL,NLV,NW,LRVP,LRT),
		  length(LE,N),
		  addTailRowVariableArities(S,LRVP,N)
		 )
		).
verifyAxiom('@or'(RV,H,S,F),FILE,FL,LL,LV,NW,[],[]):-			 % ERROR
		!,
		writeIfNotRowVariable(FILE,FL,LL,RV,W1),		   % Row operator is not applied to a row variable
		writeIfNotSingleVariable(FILE,FL,LL,H,W2),		 % The head of the row variable is not a variable
		writeIfisRowVariable(FILE,FL,LL,S,W3),			% The tail of the row variable is not a variable
		writeIfNotQuantified(FILE,FL,LL,RV,LV,W4),		 % The row variable is not quantified
		writeIfQuantified(FILE,FL,LL,H,LV,W5),			% The head of the row variable is already quantified
		writeIfQuantified(FILE,FL,LL,S,LV,W6),			% The tail of the row variable is already quantified
		append(LV,[H,S],LRV),
		remove_duplicates(LRV,NLV),
		verifyAxiom(F,FILE,FL,LL,NLV,W7,_,_),
		NW is W1 + W2 + W3 + W4 + W5 + W6 + W7.
verifyAxiom(implies(F1,F2),FILE,FL,LL,LV,NW,NLRVP,NLRT):-		  % Implies: correct
		!,
		verifyAxiom(F1,FILE,FL,LL,LV,W1,LRVP1,LRT1),
		verifyAxiom(F2,FILE,FL,LL,LV,W2,LRVP2,LRT2),
		NW is W1 + W2,
		append(LRVP1,LRVP2,NLRVP),
		append(LRT1,LRT2,NLRT).
verifyAxiom(equiv(F1,F2),FILE,FL,LL,LV,NW,NLRVP,NLRT):-		    % Equiv: correct
		!,
		verifyAxiom(F1,FILE,FL,LL,LV,W1,LRVP1,LRT1),
		verifyAxiom(F2,FILE,FL,LL,LV,W2,LRVP2,LRT2),
		NW is W1 + W2,
		append(LRVP1,LRVP2,NLRVP),
		append(LRT1,LRT2,NLRT).
verifyAxiom(not(F),FILE,FL,LL,LV,W,LRVP,LRT):-			  % Not: correct
		!,
		verifyAxiom(F,FILE,FL,LL,LV,W,LRVP,LRT).
verifyAxiom(equal(T1,T2),FILE,FL,LL,LV,NW,[],[]):-			    % Equal: correct
		!,
		verifyTerm(T1,FILE,FL,LL,LV,W1),
		verifyTerm(T2,FILE,FL,LL,LV,W2),
		NW is W1 + W2.
verifyAxiom(A,FILE,FL,LL,LV,W,LRVP,[]):-				   % Atom: correct
		verifyAtom(A,FILE,FL,LL,LV,W,LRVP).

%%% verifyListOfAxioms(in LA, in F, in FL, in LL, in LV, out W, out LRVP, out LRT)
%%%	   A: a list of axioms
%%%	   F: the file of the axiom
%%%	   FL: first line of the axiom
%%%	   LL: last line of the axiom
%%%	   LV: a list of quantified variables
%%%	   W: the number of errors in the axiom LA
%%%	   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs
%%%	   LRT: a list of pairs (RV,RVT) where
%%%		RV is a row variable that is used in a row operator
%%%		RVT is the tail row variable of the row operator where RV occurs

verifyListOfAxioms([],_,_,_,_,0,[],[]).
verifyListOfAxioms([A|LA],F,FL,LL,LV,NW,NLRVP,NLRT):-
		verifyAxiom(A,F,FL,LL,LV,W1,LRVP1,LRT1),
		verifyListOfAxioms(LA,F,FL,LL,LV,W2,LRVP2,LRT2),
		NW is W1 + W2,
		append(LRVP1,LRVP2,NLRVP),
		append(LRT1,LRT2,NLRT).

%%% verifyAtom(in A, in F, in FL, in LL, in LV, out W, out LRVP)
%%%		A: an atom
%%%		F: the file of the axiom
%%%		FL: first line of the axiom
%%%		LL: last line of the axiom
%%%		LV: a list of quantified variables
%%%		W: the number of errors in the atom A
%%%		LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs

verifyAtom('$partition'(C,L),F,FL,LL,LV,NW,LRVP):-			    % Partition
		!,
		verifyAtomPartition('$partition'(C,L),F,FL,LL,LV,NW,LRVP).
verifyAtom('$exhaustiveDecomposition'(C,L),F,FL,LL,LV,NW,LRVP):-		% Exhaustive decomposition
		!,
		verifyAtomExDecomposition('$exhaustiveDecomposition'(C,L),F,FL,LL,LV,NW,LRVP).
verifyAtom('$disjointDecomposition'(C,L),F,FL,LL,LV,NW,LRVP):-		  % Exhaustive decomposition
		!,
		verifyAtomDiDecomposition('$disjointDecomposition'(C,L),F,FL,LL,LV,NW,LRVP).
verifyAtom(A,F,FL,LL,LV,NW,NLRVP):-				   % Regular atom
		A =..[S|LT],
		length(LT,N),
		isPredicate(S,N),
		!,
		verifySubtermsAtom(S,0,N,LT,F,FL,LL,LV,NW,NLRVP).
verifyAtom(A,F,FL,LL,LV,NW,[]):-				% Error: undefined predicate
		A =..[S|LT],
		length(LT,N),
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The predicate "'),
		print_to_screen(S),
		print_to_screen('\\'),
		print_to_screen(N),
		print_to_screen('" is not defined.'),
		print_to_screen('\n'),
		verifyListOfSubterms(LT,F,FL,LL,LV,W),
		NW is W + 1.

%%% verifyAtomPartition(in A, in F, in FL, in LL, in LV, out W, out LRVP)
%%%		   A: a partition atom
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the atom A
%%%		   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs

verifyAtomPartition('$partition'(C,L),_,_,_,LV,0,[(L,('$partition',2))]):-	 % Correct
		isSingleVariable(C),
		member(C,LV),
		isRowVariable(L),
		member(L,LV),
		!.
verifyAtomPartition('$partition'(C,L),F,FL,LL,LV,NW,[]):-		  % Not correct
		(
		 (
		  isSingleVariable(C)
		 );
		 (
		  isRowVariable(L)
		 )
		),
		!,
		writeIfNotQuantified(F,FL,LL,C,LV,W1),
		writeIfNotSingleVariable(F,FL,LL,C,W2),
		writeIfNotQuantified(F,FL,LL,L,LV,W3),
		writeIfNotRowVariable(F,FL,LL,L,W4),
		NW is W1 + W2 + W3 + W4.
verifyAtomPartition('$partition'(C,L),F,FL,LL,_,NW,[]):-		   % Correct
		isClass(C),
		isMultiLengthList(L,ST),
		!,
		length(ST,N),
		assertRowArity('$partition',2,N),
		verifySubtermsAtomSubclass(C,ST,F,FL,LL,NW).
verifyAtomPartition('$partition'(C,L),F,FL,LL,_,1,[]):-		    % Not correct
		isClass(C),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(L),
		print_to_screen('" is not a multi-length list in '),
		print_to_screen('$partition'(C,L)),
		print_to_screen('.'),
		print_to_screen('\n').
verifyAtomPartition('$partition'(C,L),F,FL,LL,_,1,[]):-		    % Not correct
		isMultiLengthList(L,_),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(C),
		print_to_screen('" is not defined.'),
		print_to_screen('\n').
verifyAtomPartition('$partition'(C,L),F,FL,LL,_,1,[]):-		    % Not correct
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The atom '),
		print_to_screen('$partition'(C,L)),
		print_to_screen('" is not correct.'),
		print_to_screen('\n').

%%% verifyAtomExDecomposition(in A, in F, in FL, in LL, in LV, out W, out LRVP)
%%%		   A: an exhaustive decomposition atom
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the atom A
%%%		   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs

verifyAtomExDecomposition('$exhaustiveDecomposition'(C,L),_,_,_,LV,0,[(L,('$exhaustiveDecomposition',2))]):-
							% Correct
		isSingleVariable(C),
		member(C,LV),
		isRowVariable(L),
		member(L,LV),
		!.
verifyAtomExDecomposition('$exhaustiveDecomposition'(C,L),F,FL,LL,LV,NW,[]):-   % Not correct
		(
		 (
		  isSingleVariable(C)
		 );
		 (
		  isRowVariable(L)
		 )
		),
		!,
		writeIfNotQuantified(F,FL,LL,C,LV,W1),
		writeIfNotSingleVariable(F,FL,LL,C,W2),
		writeIfNotQuantified(F,FL,LL,L,LV,W3),
		writeIfNotRowVariable(F,FL,LL,L,W4),
		NW is W1 + W2 + W3 + W4.
verifyAtomExDecomposition('$exhaustiveDecomposition'(C,L),F,FL,LL,_,NW,[]):-    % Correct
		isClass(C),
		isMultiLengthList(L,ST),
		!,
		length(ST,N),
		assertRowArity('$exhaustiveDecomposition',2,N),
		verifySubtermsAtomSubclass(C,ST,F,FL,LL,NW).
verifyAtomExDecomposition('$exhaustiveDecomposition'(C,L),F,FL,LL,_,1,[]):-	% Not correct
		isClass(C),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(L),
		print_to_screen('" is not a multi-length list in '),
		print_to_screen('$exhaustiveDecomposition'(C,L)),
		print_to_screen('.'),
		print_to_screen('\n').
verifyAtomExDecomposition('$exhaustiveDecomposition'(C,L),F,FL,LL,_,1,[]):-	% Not correct
		isMultiLengthList(L,_),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(C),
		print_to_screen('" is not defined.'),
		print_to_screen('\n').
verifyAtomExDecomposition('$exhaustiveDecomposition'(C,L),F,FL,LL,_,1,[]):-	% Not correct
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The atom '),
		print_to_screen('$exhaustiveDecomposition'(C,L)),
		print_to_screen('" is not correct.'),
		print_to_screen('\n').

%%% verifyAtomDiDecomposition(in A, in F, in FL, in LL, in LV, out W, out LRVP)
%%%		   A: an disjoint decomposition atom
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the atom A
%%%		   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs

verifyAtomDiDecomposition('$disjointDecomposition'(C,L),_,_,_,LV,0,[(L,('$disjointDecomposition',2))]):-
							% Correct
		isSingleVariable(C),
		member(C,LV),
		isRowVariable(L),
		member(L,LV),
		!.
verifyAtomDiDecomposition('$disjointDecomposition'(C,L),F,FL,LL,LV,NW,[]):-	% Not correct
		(
		 (
		  isSingleVariable(C)
		 );
		 (
		  isRowVariable(L)
		 )
		),
		!,
		writeIfNotQuantified(F,FL,LL,C,LV,W1),
		writeIfNotSingleVariable(F,FL,LL,C,W2),
		writeIfNotQuantified(F,FL,LL,L,LV,W3),
		writeIfNotRowVariable(F,FL,LL,L,W4),
		NW is W1 + W2 + W3 + W4.
verifyAtomDiDecomposition('$disjointDecomposition'(C,L),F,FL,LL,_,NW,[]):-	 % Correct
		isClass(C),
		isMultiLengthList(L,ST),
		!,
		length(ST,N),
		assertRowArity('$disjointDecomposition',2,N),
		verifySubtermsAtomSubclass(C,ST,F,FL,LL,NW).
verifyAtomDiDecomposition('$disjointDecomposition'(C,L),F,FL,LL,_,1,[]):-	  % Not correct
		isClass(C),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(L),
		print_to_screen('" is not a multi-length list in '),
		print_to_screen('$disjointDecomposition'(C,L)),
		print_to_screen('.'),
		print_to_screen('\n').
verifyAtomDiDecomposition('$disjointDecomposition'(C,L),F,FL,LL,_,1,[]):-	  % Not correct
		isMultiLengthList(L,_),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(C),
		print_to_screen('" is not defined.'),
		print_to_screen('\n').
verifyAtomDiDecomposition('$disjointDecomposition'(C,L),F,FL,LL,_,1,[]):-	  % Not correct
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The atom '),
		print_to_screen('$disjointDecomposition'(C,L)),
		print_to_screen('" is not correct.'),
		print_to_screen('\n').

%%% verifySubtermsAtomSubclass(in C, in L, in F, in FL, in LL, out W)
%%%		   C: a constant that represents a $class of the ontology
%%%		   L: a list
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   W: the number of errors in the list of subterms L

verifySubtermsAtomSubclass(_,[],_,_,_,0).
verifySubtermsAtomSubclass(C,[SC|L],F,FL,LL,NW):-
		isClass(SC),
		isSubclass(SC,C),
		!,
		verifySubtermsAtomSubclass(C,L,F,FL,LL,NW).
verifySubtermsAtomSubclass(C,[SC|L],F,FL,LL,NW):-
		isClass(SC),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> "'),
		print_to_screen(SC),
		print_to_screen('" is not a $subclass of "'),
		print_to_screen(C),
		print_to_screen('".'),
		print_to_screen('\n'),
		verifySubtermsAtomSubclass(C,L,F,FL,LL,W),
		NW is W + 1.
verifySubtermsAtomSubclass(C,[SC|L],F,FL,LL,NW):-
		isRowVariable(SC),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The row variable '),
		print_to_screen(SC),
		print_to_screen(' is not allowed.'),
		print_to_screen('\n'),
		verifySubtermsAtomSubclass(C,L,F,FL,LL,W),
		NW is W + 1.
verifySubtermsAtomSubclass(C,[SC|L],F,FL,LL,NW):-
		isSingleVariable(SC),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The single variable '),
		printSingleVariable(SC),
		print_to_screen(' is not allowed.'),
		print_to_screen('\n'),
		verifySubtermsAtomSubclass(C,L,F,FL,LL,W),
		NW is W + 1.
verifySubtermsAtomSubclass(C,[SC|L],F,FL,LL,NW):-
		isMultiLengthList(SC,_),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The multi-length list '),
		print_to_screen(SC),
		print_to_screen(' is not allowed.'),
		print_to_screen('\n'),
		verifySubtermsAtomSubclass(C,L,F,FL,LL,W),
		NW is W + 1.
verifySubtermsAtomSubclass(C,[SC|L],F,FL,LL,NW):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> "'),
		print_to_screen(SC),
		print_to_screen('" is not defined.'),
		print_to_screen('\n'),
		verifySubtermsAtomSubclass(C,L,F,FL,LL,W),
		NW is W + 1.

%%% verifySubtermsDiDecomposition(in L, in F, in FL, in LL, out W)
%%%		   L: a list of non-variable terms
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   W: the number of errors in the list of subterms L

verifySubtermsDiDecomposition([],_,_,_,0).
verifySubtermsDiDecomposition([C|L],F,FL,LL,NW):-
		isClass(C),
		!,
		verifySubtermsDiDecomposition(L,F,FL,LL,NW).
verifySubtermsDiDecomposition([C|L],F,FL,LL,NW):-
		isSingleVariable(C),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The single variable '),
		printSingleVariable(C),
		print_to_screen(' is not allowed.'),
		print_to_screen('\n'),
		verifySubtermsDiDecomposition(L,F,FL,LL,W),
		NW is W + 1.
verifySubtermsDiDecomposition([C|L],F,FL,LL,NW):-
		isRowVariable(C),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The row variable '),
		printRowVariable(C),
		print_to_screen(' is not allowed.'),
		print_to_screen('\n'),
		verifySubtermsDiDecomposition(L,F,FL,LL,W),
		NW is W + 1.
verifySubtermsDiDecomposition([C|L],F,FL,LL,NW):-
		isMultiLengthList(C,_),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The multi-length list '),
		printMultiLengthList(C),
		print_to_screen(' is not allowed.'),
		print_to_screen('\n'),
		verifySubtermsDiDecomposition(L,F,FL,LL,W),
		NW is W + 1.
verifySubtermsDiDecomposition([C|L],F,FL,LL,NW):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> "'),
		print_to_screen(C),
		print_to_screen('" is not defined.'),
		print_to_screen('\n'),
		verifySubtermsDiDecomposition(L,F,FL,LL,W),
		NW is W + 1.

%%% verifySubtermsAtom(in P, in I, in A, in LT, in F, in FL, in LL, in LV, out W, out LRVP)
%%%		   P: a predicate symbol
%%%		   I: index of the last argument that we have treated
%%%		   A: arity of P
%%%		   LT: list of subterms
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the list of subterms LT
%%%		   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs

verifySubtermsAtom(_,N,N,[],_,_,_,_,0,[]).
verifySubtermsAtom(P,I,A,[ST|LT],F,FL,LL,LV,NW,NLRVP):-		    % Single variable: correct
		NI is I + 1,
		isSingleVariable(ST),
		(
		 (
		  getDomain(P,NI,_,0)
		 );
		 (
		  getDomainSubclass(P,NI,_,0)
		 )
		),
		!,
		verifySubtermsAtom(P,NI,A,LT,F,FL,LL,LV,NW,NLRVP).
verifySubtermsAtom(P,I,A,[ST|LT],F,FL,LL,LV,NW,NLRVP):-		    % Constant: correct
		NI is I + 1,
		isConstant(ST),
		(
		 (
		  getDomain(P,NI,T,0),
		  domainIsA(ST,T),
		  !
		 );
		 (
		  getDomainSubclass(P,NI,T,0),
		  domainClass(ST,T),
		  !
		 )
		),
		verifySubtermsAtom(P,NI,A,LT,F,FL,LL,LV,NW,NLRVP).
verifySubtermsAtom(P,I,A,[ST|LT],F,FL,LL,LV,NW,[]):-			  % Error: the type of the constant is not correct
		NI is I + 1,
		isConstant(ST),
		!,
		(
		 (
		  getDomain(P,NI,T,0),
		  C = '$instance'
		 );
		 (
		  getDomainSubclass(P,NI,T,0),
		  C = '$subclass'
		 )
		),
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> "'),
		print_to_screen(ST),
		print_to_screen('" is not '),
		print_to_screen(C),
		print_to_screen(' of "'),
		print_to_screen(T),
		print_to_screen('".'),
		print_to_screen('\n'),
		verifySubtermsAtom(P,NI,A,LT,F,FL,LL,LV,W,_),
		NW is W + 1.
verifySubtermsAtom(P,I,A,[ST|LT],F,FL,LL,LV,NW,NLRVP):-		    % Composed term: correct
		NI is I + 1,
		compound(ST),
		ST =..[FS|NST],
		(
		 (
		  getDomain(P,NI,T,0),
		  getRange(FS,STT,0),
		  domainClass(STT,T),
		  !
		 );
		 (
		  getDomainSubclass(P,NI,T,0),
		  getRangeSubclass(FS,STT,0),
		  domainClass(STT,T),
		  !
		 )
		),
		length(NST,NA),
		verifySubtermsTerm(FS,0,NA,NST,F,FL,LL,LV,W1),
		verifySubtermsAtom(P,NI,A,LT,F,FL,LL,LV,W2,NLRVP),
		NW is W1 + W2.
verifySubtermsAtom(P,I,A,[ST|LT],F,FL,LL,LV,NW,[]):-			  % Error: single variable or constant expected
		NI is I + 1,
		(
		 (
		  getDomain(P,NI,_,0)
		 );
		 (
		  getDomainSubclass(P,NI,_,0)
		 )
		),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The "'),
		print_to_screen(NI),
		print_to_screen('-th argument of "'),
		print_to_screen(P),
		print_to_screen('\\'),
		print_to_screen(A),
		print_to_screen('" is not correct. Single variable or term expected. "'),
		print_to_screen(ST),
		print_to_screen('" found.'),
		print_to_screen('\n'),
		verifySubtermsAtom(P,NI,A,LT,F,FL,LL,LV,W,_),
		NW is W + 1.
verifySubtermsAtom(P,I,A,[ST|LT],F,FL,LL,LV,NW,[(ST,(P,NI))|LRVP]):-		  % Row variable: correct
		NI is I + 1,
		isRowVariable(ST),
		(
		 (
		  getDomain(P,NI,_,1)
		 );
		 (
		  getDomainSubclass(P,NI,_,1)
		 )
		),
		!,
		verifySubtermsAtom(P,NI,A,LT,F,FL,LL,LV,NW,LRVP).
verifySubtermsAtom(P,I,A,[ST|LT],F,FL,LL,LV,NW,NLRVP):-		    % Multi-length list: correct
		NI is I + 1,
		isMultiLengthList(ST,LST),
		(
		 (
		  getDomain(P,NI,T,1)
		 );
		 (
		  getDomainSubclass(P,NI,T,1)
		 )
		),
		!,
		length(LST,N),
		assertRowArity(P,NI,N),
		verifySubtermsNonVariable(T,LST,F,FL,LL,W1),
		verifySubtermsAtom(P,NI,A,LT,F,FL,LL,LV,W2,NLRVP),
		NW is W1 + W2.
verifySubtermsAtom(P,I,A,[ST|LT],F,FL,LL,LV,NW,[]):-			  % Error: row variable or multi-length list expected
		NI is I + 1,
		(
		 (
		  getDomain(P,NI,_,1)
		 );
		 (
		  getDomainSubclass(P,NI,_,1)
		 )
		),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The "'),
		print_to_screen(NI),
		print_to_screen('-th argument of '),
		print_to_screen(P),
		print_to_screen('\\'),
		print_to_screen(A),
		print_to_screen(' is not correct. Row variable or multi-length list expected. "'),
		print_to_screen(ST),
		print_to_screen('" found.'),
		print_to_screen('\n'),
		verifySubtermsAtom(P,NI,A,LT,F,FL,LL,LV,W,_),
		NW is W + 1.

%%% verifySubtermsTerm(in FS, in I, in A, in LT, in F, in FL, in LL, in LV, out W)
%%%		   FS: a function symbol
%%%		   I: index of the last argument that we have treated
%%%		   A: arity of FS
%%%		   LT: list of subterms
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the list of subterms LT

verifySubtermsTerm(_,N,N,[],_,_,_,_,0).
verifySubtermsTerm(FS,I,A,[ST|LT],F,FL,LL,LV,NW):-		    % Single variable: correct
		NI is I + 1,
		isSingleVariable(ST),
		(
		 (
		  getDomain(FS,NI,_,0)
		 );
		 (
		  getDomainSubclass(FS,NI,_,0)
		 )
		),
		!,
		verifySubtermsTerm(FS,NI,A,LT,F,FL,LL,LV,NW).
verifySubtermsTerm(FS,I,A,[ST|LT],F,FL,LL,LV,NW):-		    % Constant: correct
		NI is I + 1,
		isConstant(ST),
		(
		 (
		  getDomain(FS,NI,T,0),
		  domainIsA(ST,T),
		  !
		 );
		 (
		  getDomainSubclass(FS,NI,T,0),
		  domainClass(ST,T),
		  !
		 )
		),
		verifySubtermsTerm(FS,NI,A,LT,F,FL,LL,LV,NW).
verifySubtermsTerm(FS,I,A,[ST|LT],F,FL,LL,LV,NW):-			  % Error: the type of the constant is not correct
		NI is I + 1,
		isConstant(ST),
		!,
		(
		 (
		  getDomain(FS,NI,T,0),
		  C = '$instance'
		 );
		 (
		  getDomainSubclass(FS,NI,T,0),
		  C = '$subclass'
		 )
		),
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> "'),
		print_to_screen(ST),
		print_to_screen('" is not '),
		print_to_screen(C),
		print_to_screen(' of "'),
		print_to_screen(T),
		print_to_screen('".'),
		print_to_screen('\n'),
		verifySubtermsTerm(FS,NI,A,LT,F,FL,LL,LV,W),
		NW is W + 1.
verifySubtermsTerm(FS,I,A,[ST|LT],F,FL,LL,LV,NW):-		    % Composed term: correct
		NI is I + 1,
		compound(ST),
		ST =..[NFS|NST],
		(
		 (
		  getDomain(FS,NI,T,0),
		  getRange(NFS,STT,0),
		  domainClass(STT,T),
		  !
		 );
		 (
		  getDomainSubclass(FS,NI,T,0),
		  getRangeSubclass(NFS,STT,0),
		  domainClass(STT,T),
		  !
		 )
		),
		length(NST,NA),
		verifySubtermsTerm(NFS,0,NA,NST,F,FL,LL,LV,W1),
		verifySubtermsTerm(FS,NI,A,LT,F,FL,LL,LV,W2),
		NW is W1 + W2.
verifySubtermsTerm(FS,I,A,[ST|LT],F,FL,LL,LV,NW):-			  % Error: single variable or constant expected
		NI is I + 1,
		(
		 (
		  getDomain(FS,NI,_,0)
		 );
		 (
		  getDomainSubclass(FS,NI,_,0)
		 )
		),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The "'),
		print_to_screen(NI),
		print_to_screen('-th argument of "'),
		print_to_screen(FS),
		print_to_screen('\\'),
		print_to_screen(A),
		print_to_screen('" is not correct. Single variable or term expected. "'),
		print_to_screen(ST),
		print_to_screen('" found.'),
		print_to_screen('\n'),
		verifySubtermsTerm(FS,NI,A,LT,F,FL,LL,LV,W),
		NW is W + 1.

%%% verifySubtermsNonVariable(in T, in LT, in F, in FL, in LL, out W)
%%%		   T: a type
%%%		   LT: a list of terms
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   W: the number of errors in the list of subterms LT

verifySubtermsNonVariable(_,[],_,_,_,0).
verifySubtermsNonVariable(T,[H|LT],F,FL,LL,NW):-
		domainIsA(H,T),
		!,
		verifySubtermsNonVariable(T,LT,F,FL,LL,NW).
verifySubtermsNonVariable(T,[H|LT],F,FL,LL,NW):-
		isSingleVariable(H),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The single variable "'),
		printSingleVariable(H),
		print_to_screen('" cannot occur in the multi-length list.'),
		print_to_screen('\n'),
		verifySubtermsNonVariable(T,LT,F,FL,LL,W),
		NW is W + 1.
verifySubtermsNonVariable(T,[H|LT],F,FL,LL,NW):-
		isRowVariable(H),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The row variable "'),
		printRowVariable(H),
		print_to_screen('" cannot occur in the multi-length list.'),
		print_to_screen('\n'),
		verifySubtermsNonVariable(T,LT,F,FL,LL,W),
		NW is W + 1.
verifySubtermsNonVariable(T,[H|LT],F,FL,LL,NW):-
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> "'),
		print_to_screen(H),
		print_to_screen('" is not an $instance of "'),
		print_to_screen(T),
		print_to_screen('".'),
		print_to_screen('\n'),
		verifySubtermsNonVariable(T,LT,F,FL,LL,W),
		NW is W + 1.

%%% verifyTerm(in T, in F, in FL, in LL, in LV, out W)
%%%		   T: a term
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the term T

verifyTerm(V,_,_,_,LV,0):-			    % Single variable: correct
		isSingleVariable(V),
		member(V,LV),
		!.
verifyTerm(C,_,_,_,_,0):-				% Constant: correct
		isConstant(C),
		!.
verifyTerm(T,F,FL,LL,LV,W):-				% Composed term: correct
		compound(T),
		!,
		T =..[_|ST],
		verifySubterms(ST,F,FL,LL,LV,W).
verifyTerm(V,F,FL,LL,_,1):-			   % Error: single variable, but not quantified
		isSingleVariable(V),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The single variable "'),
		printSingleVariable(V),
		print_to_screen('" is not quantified.'),
		print_to_screen('\n').
verifyTerm(RV,F,FL,LL,_,1):-			  % Error: row variables are not allowed
		isRowVariable(RV),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The row variable "'),
		print_to_screen(RV),
		print_to_screen('" is not allowed.'),
		print_to_screen('\n').
verifyTerm(L,F,FL,LL,_,1):-			   % Error: multi-length lists are not allowed
		isMultiLengthList(L,_),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The multi-length list "'),
		print_to_screen(L),
		print_to_screen('" is not allowed.'),
		print_to_screen('\n').
verifyTerm(T,F,FL,LL,_,1):-			   % Error: undefined
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(T),
		print_to_screen('" is undefined.'),
		print_to_screen('\n').

%%% verifySubterms(in ST, in F, in FL, in LL, in LV, out W)
%%%		   ST: a list of subterms
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the list of subterms ST

verifySubterms([],_,_,_,_,0):-
		!.
verifySubterms([T|ST],F,FL,LL,LV,NW):-
		verifyTerm(T,F,FL,LL,LV,W1),
		verifySubterms(ST,F,FL,LL,LV,W2),
		NW is W1 + W2.

%%% verifyListOfSubterms(in LST, in F, in FL, in LL, in LV, out W)
%%%		   LST: a list of subterms
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the list of subterms LST
%%%		   We know that there is at least one error in the axiom

verifyListOfSubterms([],_,_,_,_,0).
verifyListOfSubterms([ST|LST],F,FL,LL,LV,NW):-		% Defined constant: correct
		isConstant(ST),
		!,
		verifyListOfSubterms(LST,F,FL,LL,LV,NW).
verifyListOfSubterms([ST|LST],F,FL,LL,LV,NW):-		% Single variable: correct
		isSingleVariable(ST),
		member(ST,LV),
		!,
		verifyListOfSubterms(LST,F,FL,LL,LV,NW).
verifyListOfSubterms([ST|LST],F,FL,LL,LV,NW):-		% Error: single variable, but not quantified
		isSingleVariable(ST),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The single variable "'),
		printSingleVariable(ST),
		print_to_screen('" is not quantified.'),
		print_to_screen('\n'),
		verifyListOfSubterms(LST,F,FL,LL,LV,W),
		NW is W + 1.
verifyListOfSubterms([ST|LST],F,FL,LL,LV,NW):-		% Row variable: correct
		isRowVariable(ST),
		member(ST,LV),
		!,
		verifyListOfSubterms(LST,F,FL,LL,LV,NW).
verifyListOfSubterms([ST|LST],F,FL,LL,LV,NW):-		% Error: row variable, but not quantified
		isRowVariable(ST),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The row variable "'),
		printRowVariable(ST),
		print_to_screen('" is not quantified.'),
		print_to_screen('\n'),
		verifyListOfSubterms(LST,F,FL,LL,LV,W),
		NW is W + 1.
verifyListOfSubterms([ST|LST],F,FL,LL,LV,NW):-		% Multi-length list: correct
		isMultiLengthList(ST,NST),
		!,
		verifyListOfSubterms(NST,F,FL,LL,LV,W1),
		verifyListOfSubterms(LST,F,FL,LL,LV,W2),
		NW is W1 + W2.
verifyListOfSubterms([ST|LST],F,FL,LL,LV,NW):-		% Error: undefined
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> "'),
		print_to_screen(ST),
		print_to_screen('" is undefined.'),
		print_to_screen('\n'),
		verifyListOfSubterms(LST,F,FL,LL,LV,W),
		NW is W + 1.

%%% addTailRowVariableArities(in RV, in LRVP, in A)
%%%		   RV: a row variable
%%%		   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs
%%%		   A: an arity
%%%		   Action: we assert "rowArity/3" axioms for each argument where RV occurs

addTailRowVariableArities(_,[],_).
addTailRowVariableArities(RV,[(RV,(P,I))|LRVP],N):-
		!,
		NN is N - 1,
		addRowVariableArities(P,I,NN),
		addTailRowVariableArities(RV,LRVP,N).
addTailRowVariableArities(RV,[_|LRVP],N):-
		addTailRowVariableArities(RV,LRVP,N).

%%% addRowVariableArities(in P, in I, in N)
%%%		   P: a predicate
%%%		   I: index of an argument of P
%%%		   A: arity of the I-th argument of P due to a row operator

addRowVariableArities(_,_,0):-
		!.
addRowVariableArities(P,I,N):-
		assertRowArity(P,I,N),
		NN is N - 1,
		addRowVariableArities(P,I,NN).

%%% relateRowVariables(in LV, in LRVP, in LRT, out NLRVP, out NLRT)
%%%		   LV: a list of variables
%%%		   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs
%%%		   LRT: a list of pairs (RV,RVT) where
%%%		RV is a row variable that is used in a row operator
%%%		RVT is the tail row variable of the row operator where RV occurs
%%%		   NLRVP: the list that is obtained from LRVP by removing the variables in LV
%%%		   NLRT: the list that is obtained from LRT by removing the variables in LV
%%%		   Action: the arguments where the same row variable occurs are related via "relatedArity/4"

relateRowVariables([],LRVP,LRT,LRVP,LRT).
relateRowVariables([RV|LV],LRVP,LRT,NLRVP,NLRT):-
		isRowVariable(RV),
		!,
		relateRowVariable(RV,LRVP,LRVP1,LRVP2),
		relateTailRowVariable(RV,LRVP1,LRVP2,LRT,LRVP3,LRT1),
		relateRowVariables(LV,LRVP3,LRT1,NLRVP,NLRT).
relateRowVariables([_|LV],LRVP,LRT,NLRVP,NLRT):-
		relateRowVariables(LV,LRVP,LRT,NLRVP,NLRT).

%%% relateRowVariable(in RV, in LRVP, out NLRVP1, out NLRVP2)
%%%		   RV: a row variable
%%%		   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs
%%%		   NLRVP1: the list that is obtained from LRVP by exclusively keeping the row variable RV
%%%		   NLRVP2: the list that is obtained from LRVP by removing the row variable RV
%%%		   Action: the arguments where the same row variable occurs are related via "relatedArity/4"

relateRowVariable(_,[],[],[]).
relateRowVariable(RV,[(RV,(P,I))|LRVP],[(RV,(P,I))|NLRVP1],NLRVP2):-
		!,
		relateRowVariableArgument(RV,P,I,LRVP),
		relateRowVariable(RV,LRVP,NLRVP1,NLRVP2).
relateRowVariable(RV,[H|LRVP],NLRVP1,[H|NLRVP2]):-
		relateRowVariable(RV,LRVP,NLRVP1,NLRVP2).

%%% relateRowVariableArgument(in RV, in P, in I, in LRVP)
%%%		   RV: a row variable
%%%		   P: a predicate where RV occurs
%%%		   I: the index of the argument of P where RV occurs
%%%		   LRVP: a list of pairs (RV,(P,I)) where
%%%		RV is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs
%%%		   Action: the arguments where the same row variable occurs are related via "relatedArity/4"

relateRowVariableArgument(_,_,_,[]).
relateRowVariableArgument(RV,P1,I1,[(RV,(P2,I2))|LRVP]):-
		(
		 (
		  P1 \= P2
		 );
		 (
		  I1 \= I2
		 )
		),
		!,
		assertRelatedArity(P1,I1,P2,I2),
		relateRowVariableArgument(RV,P1,I1,LRVP).
relateRowVariableArgument(RV,P,I,[_|LRVP]):-
		relateRowVariableArgument(RV,P,I,LRVP).

%%% relateTailRowVariable(in RV, in LRVP1, in LRVP2, in LRT, out NLRVP, out NLRT)
%%%		   RV: a row variable
%%%		   LRVP1,LRVP2: lists of pairs (RV',(P,I)) where
%%%		RV' is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs
%%%		RV' is RV in LRVP1
%%%		RV' is not RV in LRVP2
%%%		   LRT: a list of pairs (RV,RVT) where
%%%		RV is a row variable that is used in a row operator
%%%		RVT is the tail row variable of the row operator where RV occurs
%%%		   NLRVP: the list that is obtained from LRVP2 by removing the tail row variables of RV
%%%		   NLRT: the list that is obtained from LRT by removing the row variable RV
%%%		   Action: the arguments where the row variable and the tail row variable of a row operator occur are related via "relatedTailArity/4"

relateTailRowVariable(_,_,LRV,[],LRV,[]).
relateTailRowVariable(RV,LRVP1,LRVP2,[(RV,TRV)|LRT],NLRVP,NLRT):-
		!,
		relateTailRowVariableWithHead(TRV,LRVP1,LRVP2,NLRVP2),
		relateTailRowVariable(RV,LRVP1,NLRVP2,LRT,NLRVP,NLRT).
relateTailRowVariable(RV,LRVP1,LRVP2,[H|LRT],NLRVP,[H|NLRT]):-
		relateTailRowVariable(RV,LRVP1,LRVP2,LRT,NLRVP,NLRT).

%%% relateTailRowVariableWithHead(in RV, in LRVP1, in LRVP2, out NLRVP)
%%%		   RV: a row variable that occurs in the tail of a row operator
%%%		   LRVP1,LRVP2: lists of pairs (RV',(P,I)) where
%%%		RV' is a row variable
%%%		P is a predicate where RV occurs
%%%		I is the index of the argument where RV occurs
%%%		RV' is the row variable in the row operator of RV
%%%		LRVP1 and LRVP2 are disjoint
%%%		   NLRVP: the list that is obtained from LRVP2 by removing the tail row variables of RV
%%%		   Action: the arguments where the row variable and the tail row variable of a row operator occur are related via "relatedTailArity/4"

relateTailRowVariableWithHead(_,_,[],[]).
relateTailRowVariableWithHead(RV,LRVP1,[(RV,(P,I))|LRVP2],NLRVP2):-
		!,
		relateTailRowVariableWithListOfPredicates(LRVP1,P,I),
		relateTailRowVariableWithHead(RV,LRVP1,LRVP2,NLRVP2).
relateTailRowVariableWithHead(RV,LRVP1,[H|LRVP2],[H|NLRVP2]):-
		relateTailRowVariableWithHead(RV,LRVP1,LRVP2,NLRVP2).

%%% relateTailRowVariableWithListOfPredicates(in LRVP, in P, in I)
%%%		   LRVP: a list of pairs (RV,(P',I')) where
%%%		RV is a row variable
%%%		P' is a predicate where RV occurs
%%%		I' is the index of the argument where RV occurs
%%%		   P: a predicate
%%%		   I: index of an argument of P
%%%		   Action: the arguments I' and I of predicates P' and P are related via "relatedTailArity/4" for each element in LRVP

relateTailRowVariableWithListOfPredicates([],_,_).
relateTailRowVariableWithListOfPredicates([(_,(NP,NI))|LRVP],P,I):-
		assertRelatedTailArity(NP,NI,P,I),
		relateTailRowVariableWithListOfPredicates(LRVP,P,I).

%%% verifySchema(in C, in F, in R, in A, in VP, in FILE, in FL, in LL, in LV, out W)
%%%		   C: a class
%%%		   F: a formula
%%%		   R: the auxiliar relation
%%%		   A: arity of the predicates defined using this schema
%%%		   VP: a variable predicate
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors in the axiom A
%%%		   Action: the following checks are performed
%%%		5. all the constants have been defined
%%%		6. predicates are predefined or defined according to one schema
%%%		7. all the variables are quantified (only closed formulas are allowed), but only once
%%%		10. both the auxiliar relation and the variable predicate occur in the formula 

verifySchema(C,F,R,A,VP,FILE,FL,LL,NW):-
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,[],W1,W2,W3),
		(
		 (
		  W2 > 0,
		  W3 > 0,
		  !,
		  W4 = 0
		 );
		 (
		  W2 > 0,
		  W3 = 0,
		  !,
		  W4 = 1,		  % FALTAN LOS MENSAJES INDICANDO EL ERROR!!!
		  print_to_screen('\n\t\t'),
		  print_to_screen('Error: file '),
		  print_to_screen(FILE),
		  print_to_screen('.kif, lines '),
		  print_to_screen(FL),
		  print_to_screen('-'),
		  print_to_screen(LL),
		  print_to_screen(' --> the variable predicate "'),
		  printVariablePredicate(VP),
		  print_to_screen('" does not occur in the formula.'),
		  print_to_screen('\n')
		 );
		 (
		  W2 = 0,
		  W3 > 0,
		  !,
		  W4 = 1,		  % FALTAN LOS MENSAJES INDICANDO EL ERROR!!!
		  print_to_screen('\n\t\t'),
		  print_to_screen('Error: file '),
		  print_to_screen(FILE),
		  print_to_screen('.kif, lines '),
		  print_to_screen(FL),
		  print_to_screen('-'),
		  print_to_screen(LL),
		  print_to_screen(' --> the auxiliary relation "'),
		  print_to_screen(R),
		  NA is A + 1,
		  print_to_screen('/'),
		  print_to_screen(NA),
		  print_to_screen('" does not occur in the formula.'),
		  print_to_screen('\n')
		 );
		 (
		  W2 = 0,
		  W3 = 0,
		  W4 = 2,		  % FALTAN LOS MENSAJES INDICANDO EL ERROR!!!
		  print_to_screen('\n\t\t'),
		  print_to_screen('Error: file '),
		  print_to_screen(FILE),
		  print_to_screen('.kif, lines '),
		  print_to_screen(FL),
		  print_to_screen('-'),
		  print_to_screen(LL),
		  print_to_screen(' --> the variable predicate "'),
		  printVariablePredicate(VP),
		  print_to_screen('" does not occur in the formula.'),
		  print_to_screen('\n\t\t'),
		  print_to_screen('Error: file '),
		  print_to_screen(FILE),
		  print_to_screen('.kif, lines '),
		  print_to_screen(FL),
		  print_to_screen('-'),
		  print_to_screen(LL),
		  print_to_screen(' --> the auxiliary relation "'),
		  print_to_screen(R),
		  NA is A + 1,
		  print_to_screen('/'),
		  print_to_screen(NA),
		  print_to_screen('" does not occur in the formula.'),
		  print_to_screen('\n')
		 )
		),
		NW is W1 + W4.

%%% verifyFormulaSchema(in C, in F, in R, in A, in VP, in FILE, in FL, in LL, in LV, out W1, out W2, out W3)
%%%		   C: a class
%%%		   F: a formula
%%%		   R: the auxiliar relation
%%%		   A: arity of the predicates defined using this schema
%%%		   VP: a variable predicate
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W1: the number of errors in the axiom A
%%%		   W2: 
%%%			   0	  if the auxiliary relation R does NOT occur in F
%%%			   1	  if the auxiliary relation R DOES occur in F
%%%		   W3: 
%%%			   0	  if the variable predicate VP does NOT occur in F
%%%			   1	  if the variable predicate VP DOES occur in F

verifyFormulaSchema(C,exists(LVE,F),R,A,VP,FILE,FL,LL,LV,W1,W2,W3):-		    % Exists: correct
		disjoint_lists(LVE,LV),
		no_doubles(LVE),
		!,
		append(LV,LVE,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3).
verifyFormulaSchema(C,exists(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		   % ERROR: repeated variables in quantifier
		disjoint_lists(LVE,LV),
		!,
		repeated_elements(LVE,RE),
		remove_duplicates(RE,NRE),
		writeRepeatedVariables(FILE,FL,LL,NRE),
		remove_duplicates(LVE,NLVE),
		append(LV,NLVE,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 1.
verifyFormulaSchema(C,exists(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		   % ERROR: variables already quantified
		no_doubles(LVE),
		!,
		shared_elements(LV,LVE,SE),
		writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		append(LV,LVE,LRV),
		remove_duplicates(LRV,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 1.
verifyFormulaSchema(C,exists(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		   % ERROR: repeated variables in quantifier and variables already quantified
		!,
		repeated_elements(LVE,RE),
		remove_duplicates(RE,NRE),
		writeRepeatedVariables(FILE,FL,LL,NRE),
		remove_duplicates(LVE,NLVE),
		shared_elements(LV,NLVE,SE),
		writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		append(LV,NLVE,LRV),
		remove_duplicates(LRV,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 2.
verifyFormulaSchema(C,forall(LVE,F),R,A,VP,FILE,FL,LL,LV,W1,W2,W3):-		    % For all: correct
		disjoint_lists(LVE,LV),
		no_doubles(LVE),
		!,
		append(LV,LVE,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3).
verifyFormulaSchema(C,forall(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		   % ERROR: repeated variables in quantifier
		disjoint_lists(LVE,LV),
		!,
		repeated_elements(LVE,RE),
		remove_duplicates(RE,NRE),
		writeRepeatedVariables(FILE,FL,LL,NRE),
		remove_duplicates(LVE,NLVE),
		append(LV,NLVE,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 1.
verifyFormulaSchema(C,forall(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		   % ERROR: variables already quantified
		no_doubles(LVE),
		!,
		shared_elements(LV,LVE,SE),
		writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		append(LV,LVE,LRV),
		remove_duplicates(LRV,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 1.
verifyFormulaSchema(C,forall(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		   % ERROR: repeated variables in quantifier and variables already quantified
		!,
		repeated_elements(LVE,RE),
		remove_duplicates(RE,NRE),
		writeRepeatedVariables(FILE,FL,LL,NRE),
		remove_duplicates(LVE,NLVE),
		shared_elements(LV,NLVE,SE),
		writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		append(LV,NLVE,LRV),
		remove_duplicates(LRV,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 2.
verifyFormulaSchema(C,'exists*'(LVE,F),R,A,VP,FILE,FL,LL,LV,W1,W2,W3):-		   % Exists *: correct
		disjoint_lists(LVE,LV),
		no_doubles(LVE),
		!,
		append(LV,LVE,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3).
verifyFormulaSchema(C,'exists*'(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		  % ERROR: repeated variables in quantifier
		disjoint_lists(LVE,LV),
		!,
		repeated_elements(LVE,RE),
		remove_duplicates(RE,NRE),
		writeRepeatedVariables(FILE,FL,LL,NRE),
		remove_duplicates(LVE,NLVE),
		append(LV,NLVE,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 1.
verifyFormulaSchema(C,'exists*'(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		  % ERROR: variables already quantified
		no_doubles(LVE),
		!,
		shared_elements(LV,LVE,SE),
		writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		append(LV,LVE,LRV),
		remove_duplicates(LRV,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 1.
verifyFormulaSchema(C,'exists*'(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		  % ERROR: repeated variables in quantifier and variables already quantified
		!,
		repeated_elements(LVE,RE),
		remove_duplicates(RE,NRE),
		writeRepeatedVariables(FILE,FL,LL,NRE),
		remove_duplicates(LVE,NLVE),
		shared_elements(LV,NLVE,SE),
		writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		append(LV,NLVE,LRV),
		remove_duplicates(LRV,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 2.
verifyFormulaSchema(C,'forall*'(LVE,F),R,A,VP,FILE,FL,LL,LV,W1,W2,W3):-		   % For all *: correct
		disjoint_lists(LVE,LV),
		no_doubles(LVE),
		!,
		append(LV,LVE,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3).
verifyFormulaSchema(C,'forall*'(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		  % ERROR: repeated variables in quantifier
		disjoint_lists(LVE,LV),
		!,
		repeated_elements(LVE,RE),
		remove_duplicates(RE,NRE),
		writeRepeatedVariables(FILE,FL,LL,NRE),
		remove_duplicates(LVE,NLVE),
		append(LV,NLVE,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 1.
verifyFormulaSchema(C,'forall*'(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		  % ERROR: variables already quantified
		no_doubles(LVE),
		!,
		shared_elements(LV,LVE,SE),
		writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		append(LV,LVE,LRV),
		remove_duplicates(LRV,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 1.
verifyFormulaSchema(C,'forall*'(LVE,F),R,A,VP,FILE,FL,LL,LV,NW1,W2,W3):-		  % ERROR: repeated variables in quantifier and variables already quantified
		!,
		repeated_elements(LVE,RE),
		remove_duplicates(RE,NRE),
		writeRepeatedVariables(FILE,FL,LL,NRE),
		remove_duplicates(LVE,NLVE),
		shared_elements(LV,NLVE,SE),
		writeAlreadyQuantifiedVariables(FILE,FL,LL,SE),
		append(LV,NLVE,LRV),
		remove_duplicates(LRV,NLV),
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,NLV,W1,W2,W3),
		NW1 is W1 + 2.
verifyFormulaSchema(C,and(LF),R,A,VP,FILE,FL,LL,LV,W1,W2,W3):-			% And: correct
		!,
		verifyListOfFormulasSchema(C,LF,R,A,VP,FILE,FL,LL,LV,W1,W2,W3).
verifyFormulaSchema(C,or(LF),R,A,VP,FILE,FL,LL,LV,W1,W2,W3):-			 % Or: correct
		!,
		verifyListOfFormulasSchema(C,LF,R,A,VP,FILE,FL,LL,LV,W1,W2,W3).
verifyFormulaSchema(C,implies(F1,F2),R,A,VP,FILE,FL,LL,LV,NW1,NW2,NW3):-		% Implies: correct
		!,
		verifyFormulaSchema(C,F1,R,A,VP,FILE,FL,LL,LV,W11,W12,W13),
		verifyFormulaSchema(C,F2,R,A,VP,FILE,FL,LL,LV,W21,W22,W23),
		NW1 is W11 + W21,
		NW2 is W12 + W22,
		NW3 is W13 + W23.
verifyFormulaSchema(C,equiv(F1,F2),R,A,VP,FILE,FL,LL,LV,NW1,NW2,NW3):-		  % Equiv: correct
		!,
		verifyFormulaSchema(C,F1,R,A,VP,FILE,FL,LL,LV,W11,W12,W13),
		verifyFormulaSchema(C,F2,R,A,VP,FILE,FL,LL,LV,W21,W22,W23),
		NW1 is W11 + W21,
		NW2 is W12 + W22,
		NW3 is W13 + W23.		
verifyFormulaSchema(C,not(F),R,A,VP,FILE,FL,LL,LV,W1,W2,W3):-			 % Not: correct
		!,
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,LV,W1,W2,W3).
verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,LV,W1,W2,W3):-			% Atom: correct
		verifyAtomSchema(C,F,R,A,VP,FILE,FL,LL,LV,W1,W2,W3).

%%% verifyListOfFormulasSchema(in C, in LF, in R, in A, in VP, in FILE, in FL, in LL, in LV, out W1, out W2, out W3)
%%%		   C: a class
%%%		   LF: a list of formulas
%%%		   R: the auxiliar relation
%%%		   A: arity of the predicates defined using this schema
%%%		   VP: a variable predicate
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W1: the number of errors in the axiom A
%%%		   W2: 
%%%			   0	  if the auxiliary relation R does NOT occur in LF
%%%			   1	  if the auxiliary relation R DOES occur in LF
%%%		   W3: 
%%%			   0	  if the variable predicate VP does NOT occur in LF
%%%			   1	  if the variable predicate VP DOES occur in LF
%%%		   Action: the following checks are performed
%%%		5. all the constants have been defined
%%%		6. predicates are predefined or defined according to one schema
%%%		7. all the variables are quantified (only closed formulas are allowed), but only once
%%%		10. both the auxiliar relation and the variable predicate occur in the formula 

verifyListOfFormulasSchema(_,[],_,_,_,_,_,_,_,0,0,0).
verifyListOfFormulasSchema(C,[F|LF],R,A,VP,FILE,FL,LL,LV,NW1,NW2,NW3):-
		verifyFormulaSchema(C,F,R,A,VP,FILE,FL,LL,LV,W11,W12,W13),
		verifyListOfFormulasSchema(C,LF,R,A,VP,FILE,FL,LL,LV,W21,W22,W23),
		NW1 is W11 + W21,
		NW2 is W12 + W22,
		NW3 is W13 + W23.

%%% verifyAtomSchema(in C, in T, in R, in A, in VP, in FILE, in FL, in LL, in LV, out W1, out W2, out W3)
%%%		   C: in class
%%%		   T: an atom
%%%		   R: the auxiliar relation
%%%		   A: arity of the predicates defined using this schema
%%%		   VP: a variable predicate
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W1: the number of errors in the axiom A
%%%		   W2: 
%%%			   0	  if the auxiliary relation R does NOT occur in F
%%%			   1	  if the auxiliary relation R DOES occur in F
%%%		   W3: 
%%%			   0	  if the variable predicate VP does NOT occur in F
%%%			   1	  if the variable predicate VP DOES occur in F

verifyAtomSchema(C,T,R,A,VP,FILE,FL,LL,LV,W1,W2,0):-
		T =..[R|LST],
		length(LST,N),
		N is A + 1,				   % Auxiliar relation
		!,
		verifySubtermsAtomSchema(C,R,0,N,LST,VP,FILE,FL,LL,LV,W1,W3),
		(
		 (
		  W3 > 0,
		  !,
		  W2 = 1
		 );
		 (
		  W2 = 0
		 )
		).
verifyAtomSchema(C,T,_,A,VP,FILE,FL,LL,LV,NW1,0,1):-
		T =..[VP|LST],
		length(LST,A),				% Variable predicate
		!,
		verifySubtermsAtomSchema(C,C,0,A,LST,VP,FILE,FL,LL,LV,W1,W2),
		(
		 (
		  W2 = 0,
		  !,
		  NW1 = W1
		 );
		 (
		  NW1 is W1 + 1,
		  print_to_screen('\n\t\t'),
		  print_to_screen('Error: file '),
		  print_to_screen(FILE),
		  print_to_screen('.kif, lines '),
		  print_to_screen(FL),
		  print_to_screen('-'),
		  print_to_screen(LL),
		  print_to_screen(' --> the variable predicate "'),
		  printVariablePredicate(VP),
		  print_to_screen('" cannot be used in the current atom as a term.'),
		  print_to_screen('\n')
		 )
		).
verifyAtomSchema(C,T,_,_,VP,FILE,FL,LL,LV,NW1,0,0):-
		T =..[S|LST],
		length(LST,N),				
		isPredicate(S,N),					% Defined predicate: correct
		!,
		verifySubtermsAtomSchema(C,S,0,N,LST,VP,FILE,FL,LL,LV,W1,W2),
		(
		 (
		  W2 = 0,
		  !,
		  NW1 = W1
		 );
		 (
		  NW1 is W1 + 1,
		  print_to_screen('\n\t\t'),
		  print_to_screen('Error: file '),
		  print_to_screen(FILE),
		  print_to_screen('.kif, lines '),
		  print_to_screen(FL),
		  print_to_screen('-'),
		  print_to_screen(LL),
		  print_to_screen(' --> the variable predicate "'),
		  printVariablePredicate(VP),
		  print_to_screen('" cannot be used in the current atom as a term.'),
		  print_to_screen('\n')
		 )
		).
verifyAtomSchema(_,T,_,_,_,FILE,FL,LL,_,1,0,0):-
		T =..[S|LST],
		length(LST,N),
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(FILE),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> the predicate "'),
		print_to_screen(S),
		print_to_screen('\\'),
		print_to_screen(N),
		print_to_screen('" is undefined.'),
		print_to_screen('\n').

%%% verifySubtermsAtomSchema(in C, in P, in I, in A, in LST, in VP, in F, in FL, in LL, in LV, out W1, out W2)
%%%		   C: a class
%%%		   P: a predicate symbol
%%%		   I: index of the last argument that we have treated
%%%		   A: arity of P
%%%		   LST: a list of subterms
%%%		   VP: a variable predicate
%%%		   F: the file of the axiom
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: a list of quantified variables
%%%		   W1: the number of errors in the list of subterms LST
%%%		   W2: 
%%%			   0	  if the variable predicate VP does NOT occur in LST
%%%			   1	  if the variable predicate VP DOES occur in LST

verifySubtermsAtomSchema(_,_,_,_,[],_,_,_,_,_,0,0).
verifySubtermsAtomSchema(C,P,0,A,[VP|LST],VP,F,FL,LL,LV,W1,1):-	    % Variable predicate: correct
		getDomain(P,1,T,0),
		(
		 (
		  C = T,
		  !
		 );
		 (
		  isSubclass(C,T)
		 )
		),
		!,
		verifySubtermsAtomSchema(C,P,1,A,LST,VP,F,FL,LL,LV,W1,_).
verifySubtermsAtomSchema(C,P,I,A,[ST|LST],VP,F,FL,LL,LV,W1,W2):-		% Defined constant: correct
		NI is I + 1,
		getDomain(P,NI,T,0),
		isConstant(ST),
		domainIsA(ST,T),
		!,
		verifySubtermsAtomSchema(C,P,NI,A,LST,VP,F,FL,LL,LV,W1,W2).
verifySubtermsAtomSchema(C,P,I,A,[ST|LST],VP,F,FL,LL,LV,W1,W2):-		% Single variable: correct
		NI is I + 1,
		getDomain(P,NI,_,0),
		isSingleVariable(ST),
		member(ST,LV),
		!,
		verifySubtermsAtomSchema(C,P,NI,A,LST,VP,F,FL,LL,LV,W1,W2).
verifySubtermsAtomSchema(C,P,I,A,[ST|LST],VP,F,FL,LL,LV,NW1,W2):-		% Error: single variable, but not quantified
		isSingleVariable(ST),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The single variable "'),
		printSingleVariable(ST),
		print_to_screen('" is not quantified.'),
		print_to_screen('\n'),
		NI is I + 1,
		verifySubtermsAtomSchema(C,P,NI,A,LST,VP,F,FL,LL,LV,W1,W2),
		NW1 is W1 + 1.
verifySubtermsAtomSchema(C,P,I,A,[ST|LST],VP,F,FL,LL,LV,NW1,W2):-		% Error: row variable
		isRowVariable(ST),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The row variable "'),
		printRowVariable(ST),
		print_to_screen('" is not allowed.'),
		print_to_screen('\n'),
		NI is I + 1,
		verifySubtermsAtomSchema(C,P,NI,A,LST,VP,F,FL,LL,LV,W1,W2),
		NW1 is W1 + 1.
verifySubtermsAtomSchema(C,P,I,A,[ST|LST],VP,F,FL,LL,LV,NW1,W2):-		% Error: multi-length list
		isMultiLengthList(ST,_),
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The multi-length list "'),
		printMultiLengthList(ST),
		print_to_screen('" is not allowed.'),
		print_to_screen('\n'),
		NI is I + 1,
		verifySubtermsAtomSchema(C,P,NI,A,LST,VP,F,FL,LL,LV,W1,W2),
		NW1 is W1 + 1.
verifySubtermsAtomSchema(C,P,I,A,[ST|LST],VP,F,FL,LL,LV,NW1,W2):-		% Error: undefined or incorrect
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> "'),
		print_to_screen(ST),
		print_to_screen('" is undefined or incorrect.'),
		print_to_screen('\n'),
		NI is I + 1,
		verifySubtermsAtomSchema(C,P,NI,A,LST,VP,F,FL,LL,LV,W1,W2),
		NW1 is W1 + 1.

%%% writeRepeatedVariables(in F, in FL, in LL, in LV)
%%%		   F: a file
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: the list of repeated variables
%%%%		  Action: the error message is prompted

writeRepeatedVariables(F,FL,LL,LV):-
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The following variables occur repeated in the same quantifier: '),
		writeListOfRepeatedVariables(LV),
		print_to_screen('\n').

%%% writeListOfRepeatedVariables(in LV)
%%%		   LV: a list of repeated variables
%%%		   Action: the list of variables is prompted
%%% Deprecated: sustituir por un predicado que comience con print, al estilo de printSingleVariable o printRowVariable

writeListOfRepeatedVariables([]):-
		!,
		print_to_screen('You should not read this message. Please, contact to the maintaner of the application. (Code: repeated variables in a quantifier)').
writeListOfRepeatedVariables(['#var'(V)]):-
		!,
		print_to_screen(V),
		print_to_screen('.').
writeListOfRepeatedVariables(['#rowVar'(V)]):-
		!,
		print_to_screen(V),
		print_to_screen('.').
writeListOfRepeatedVariables(['#var'(V)|LV]):-
		print_to_screen(V),
		print_to_screen(', '),
		writeListOfRepeatedVariables(LV).
writeListOfRepeatedVariables(['#rowVar'(V)|LV]):-
		print_to_screen(V),
		print_to_screen(', '),
		writeListOfRepeatedVariables(LV).

%%% writeAlreadyQuantifiedVariables(in F, in FL, in LL, in LV)
%%%		   F: a file
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   LV: the list of already quantified variables
%%%		   Action: the error message is prompted
%%% Deprecated: sustituir por un predicado que comience con print, al estilo de printSingleVariable o printRowVariable

writeAlreadyQuantifiedVariables(F,FL,LL,LV):-
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The following variables occur repeated in the same quantifier: '),
		writeListOfAlreadyQuantifiedVariables(LV),
		print_to_screen('\n').

%%% writeListOfAlreadyQuantifiedVariables(in LV)
%%%		   LV: a list of already quantified variables
%%%		   Action: the list of variables is prompted
%%% Deprecated: sustituir por un predicado que comience con print, al estilo de printSingleVariable o printRowVariable

writeListOfAlreadyQuantifiedVariables([]):-
		!,
		print_to_screen('You should not read this message. Please, contact to the maintaner of the application. (Code: repeated variables in a quantifier)'),
		print_to_screen('\n').
writeListOfAlreadyQuantifiedVariables(['#var'(V)]):-
		!,
		print_to_screen(V),
		print_to_screen('.').
writeListOfAlreadyQuantifiedVariables(['#rowVar'(V)]):-
		!,
		print_to_screen(V),
		print_to_screen('.').
writeListOfAlreadyQuantifiedVariables(['#var'(V)|LV]):-
		print_to_screen(V),
		print_to_screen(', '),
		writeListOfAlreadyQuantifiedVariables(LV).
writeListOfAlreadyQuantifiedVariables(['#rowVar'(V)|LV]):-
		print_to_screen(V),
		print_to_screen(', '),
		writeListOfAlreadyQuantifiedVariables(LV).

%%% writeIfNotRowVariable(in F, in FL, in LL, in T, out W)
%%%		   F: a file
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   T: a term
%%%		   W: the number of errors (0 or 1)
%%%		   Action: an error is prompted if T is not a row variable

writeIfNotRowVariable(_,_,_,'#rowVar'(_),0):-
		!.
writeIfNotRowVariable(F,FL,LL,'#var'(V),1):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(V),
		print_to_screen( 'is not a row variable.'),
		print_to_screen('\n').
writeIfNotRowVariable(_,_,_,'#row'(_),1):-
		!,
		print_to_screen('You should not read this message. Please, contact to the maintaner of the application. (Code: check if not row variable)'),
		print_to_screen('\n').
writeIfNotRowVariable(F,FL,LL,V,1):-
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(V),
		print_to_screen( 'is not a row variable.'),
		print_to_screen('\n').

%%% writeIfNotSingleVariable(in F, in FL, in LL, in T, out W)
%%%		   F: a file
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   T: a term
%%%		   W: the number of errors (0 or 1)
%%%		   Action: an error is prompted if T is not a variable

writeIfNotSingleVariable(_,_,_,'#var'(_),0):-
		!.
writeIfNotSingleVariable(F,FL,LL,'#rowVar'(V),1):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(V),
		print_to_screen( 'is not a single variable.'),
		print_to_screen('\n').
writeIfNotSingleVariable(_,_,_,'#row'(_),1):-
		!,
		print_to_screen('You should not read this message. Please, contact to the maintaner of the application. (Code: check if not variable)'),
		print_to_screen('\n').
writeIfNotSingleVariable(F,FL,LL,V,1):-
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(V),
		print_to_screen( 'is not a single variable.'),
		print_to_screen('\n').

%%% writeIfNotQuantified(in F, in FL, in LL, in V, in LV out W)
%%%		   F: a file
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   T: a term
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors (0 or 1)
%%%		   Action: an error is prompted if T is not a quantified variable

writeIfNotQuantified(_,_,_,V,LV,0):-
		member(V,LV),
		!.
writeIfNotQuantified(_,_,_,'#row'(_),_,1):-
		!,
		print_to_screen('You should not read this message. Please, contact to the maintaner of the application. (Code: check if not quantified)'),
		print_to_screen('\n').
writeIfNotQuantified(F,FL,LL,'#var'(V),_,1):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The variable '),
		print_to_screen(V),
		print_to_screen( 'is not quantified.'),
		print_to_screen('\n').
writeIfNotQuantified(F,FL,LL,'#rowVar'(V),_,1):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The row variable '),
		print_to_screen(V),
		print_to_screen( 'is not quantified.'),
		print_to_screen('\n').
writeIfNotQuantified(F,FL,LL,V,_,1):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(V),
		print_to_screen( 'is not quantified.'),
		print_to_screen('\n').

%%% writeIfQuantified(in F, in FL, in LL, in V, in LV out W)
%%%		   F: a file
%%%		   FL: first line of the axiom
%%%		   LL: last line of the axiom
%%%		   T: a term
%%%		   LV: a list of quantified variables
%%%		   W: the number of errors (0 or 1)
%%%		   Action: an error is prompted if T is already quantified

writeIfQuantified(_,_,_,V,LV,0):-
		non_member(V,LV),
		!.
writeIfQuantified(_,_,_,'#row'(_),_,1):-
		!,
		print_to_screen('You should not read this message. Please, contact to the maintaner of the application. (Code: check if quantified)'),
		print_to_screen('\n').
writeIfQuantified(F,FL,LL,'#var'(V),_,1):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The variable '),
		print_to_screen(V),
		print_to_screen( 'is already quantified.'),
		print_to_screen('\n').
writeIfQuantified(F,FL,LL,'#rowVar'(V),_,1):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> The row variable '),
		print_to_screen(V),
		print_to_screen( 'is already quantified.'),
		print_to_screen('\n').
writeIfQuantified(F,FL,LL,V,_,1):-
		!,
		print_to_screen('\n\t\t'),
		print_to_screen('Error: file '),
		print_to_screen(F),
		print_to_screen('.kif, lines '),
		print_to_screen(FL),
		print_to_screen('-'),
		print_to_screen(LL),
		print_to_screen(' --> '),
		print_to_screen(V),
		print_to_screen( 'is already quantified.'),
		print_to_screen('\n').

%%% printSingleVariable(in V)
%%%		   V: a single variable
%%%		   Action: the single variable is prompted

printSingleVariable('#var'(V)):-
		print_to_screen('?'),
		print_to_screen(V).

%%% printRowVariable(in V)
%%%		   V: a row variable
%%%		   Action: the row variable is prompted

printRowVariable('#rowVar'(V)):-
		print_to_screen('@'),
		print_to_screen(V).

%%% printMultiLengthList(in L)
%%%		   L: a multi-length list
%%%		   Action: the multi-length list is prompted

printMultiLengthList('#row'(L)):-
		print_to_screen('@('),
		printList(L),
		print_to_screen(')').

%%% printList(in L)
%%%		   L: a list
%%%		   Action: the list is prompted

printList([]).
printList([E]):-
		!,
		print_to_screen(E).
printList([E|L]):-
		print_to_screen(E),
		print_to_screen(','),
		printList(L).

%%% printVariablePredicate(in V)
%%%		   V: a variable predicate
%%%		   Action: the variable predicate is prompted

printVariablePredicate(V):-
		print_to_screen(V).
