%%% writeAxioms
%%%		Action: write axioms in the output files

writeAxioms:-
		findall((F,L),numberOfLines(F,L),LFL),
		catch(make_directory('Axioms'),_,true),
		writeAxiomsListOfFiles(LFL).

%%% writeAxiomsListOfFiles(in LFL)
%%%		LFL: a list of pairs (F,L) where
%%%			F is the file name
%%%			L is the number of lines
%%%		Action: write axioms in the output files

writeAxiomsListOfFiles([]).
writeAxiomsListOfFiles([(F,L)|LFL]):-
		getAutomaticProver(AS),
		getOutputLanguage(OL),
		atom_concat(F,'.kif',NF1),
		catch((open(NF1,read,S1), C = 0),_,C = 1),
%		atom_concat(AS,'.',PAS),
%		atom_concat(PAS,OL,SOL),
		atom_concat('Axioms/',F,F2),
		atom_concat(F2,'.adimen.',F3),
%		atom_concat(F3,SOL,NF2),
		atom_concat(F3,OL,NF2),
		(
		 (
		  AS = 'vanhelsing',
		  !,
		  NAS = 'eprover'
		 );
		 (
		  NAS = AS
		 )
		),
		open(NF2,write,S2),
		!,
		(
		 (
		  C = 0,
		  !,
		  writeAxiomsFile(F,S1,S2,OL,NAS,1,L,RA,UC),
		  close(S1)
		 );
		 (
		  C = 1,
		  numberOfAxioms(F,NL),
		  writeNewCreatedAxiomsFile(F,S2,OL,NAS,1,NL,RA,UC)
		 )
		),
		close(S2),
		print_to_screen('\t'),
		print_to_screen('File '),
		print_to_screen(NF2),
		print_to_screen(' completed (total axioms --> '),
		print_to_screen(RA),
		print_to_screen(', unit clauses --> '),
		print_to_screen(UC),
		print_to_screen(').'),
		print_to_screen('\n'),
		writeAxiomsListOfFiles(LFL).

%%% getOutputLanguage(out L)
%%%		L: output language ('tstp' by default)

getOutputLanguage(F):-
		output_language_flag(F),
		!.
getOutputLanguage(tstp).														% Default

%%% getAutomaticProver(out AP)
%%%		AP: automatic prover ('vampire' by default)

getAutomaticProver(AP):-
		solver_flag(AP),
		!.
getAutomaticProver(vampire).													% Default

%%% writeAxiomsFile(in F, in S1, in S2, in OL, in AP, in CL, in TL, out NRA, out NUC)
%%%		F: a file name
%%%		S1: input stream
%%%		S2: output stream
%%%		OL: output language
%%%		AP: the automatic prover to be used
%%%		CL: current line of the file
%%%		TL: total number of lines in the file
%%%		NRA: number of real axioms that have been written to the file
%%%		NUC: number of unit clauses that have been written to the file (axioms consisting of a single literal)
%%%		Action: write axioms in the output file

writeAxiomsFile(_,_,_,_,_,CL,TL,0,0):-
		CL > TL,
		!.
writeAxiomsFile(F,S1,S2,OL,AP,CL,TL,NRA,NUC):-
		axiom(C,F,CL,LL,K,A),
		NL is LL - CL + 1,
		writeAxiomWithText(S1,S2,OL,AP,C,A,K,NL,RA1,UC1),
		NCL is LL + 1,
		writeAxiomsFile(F,S1,S2,OL,AP,NCL,TL,RA2,UC2),
		NRA is RA1 + RA2,
		NUC is UC1 + UC2.

%%% writeNewCreatedAxiomsFile(in F, in S, in OL, in AP, in CL, in TL, out NRA, out NUC)
%%%		F: a file name
%%%		S: output stream
%%%		OL: output language
%%%		AP: the automatic prover to be used
%%%		CL: current line of the file
%%%		TL: total number of lines in the file
%%%		NRA: number of real axioms that have been written to the file
%%%		NUC: number of unit clauses that have been written to the file (axioms consisting of a single literal)
%%%		Action: write automatically sorted axioms in the output file

writeNewCreatedAxiomsFile(_,_,_,_,CL,TL,0,0):-
		CL > TL,
		!.
writeNewCreatedAxiomsFile(F,S,OL,AP,CL,TL,NRA,NUC):-
		axiom(C,F,CL,LL,K,A),
		writeAxiomWithoutText(S,OL,AP,C,A,K,RA1,UC1),
		NCL is LL + 1,
		writeNewCreatedAxiomsFile(F,S,OL,AP,NCL,TL,RA2,UC2),
		NRA is RA1 + RA2,
		NUC is UC1 + UC2.

%%% writeAxiomWithText(in S1, in S2, in OL, in C, in T, in K, in NL, out NRA, out NUC)
%%%		S1: input stream
%%%		S2: output stream
%%%		OL: output language
%%%		C: axiom internal code
%%%			-4 -> Comment
%%%			-3 -> Documentation
%%%			-2 -> Empty line
%%%			-1 -> Comment
%%%			 0 -> FOF axiom without row variables
%%%			 1 -> Non-FOF axiom (formulas as terms)
%%%			 2 -> Non-FOF axiom (variables as predicates)
%%%			 3 -> FOF axiom with row variables
%%%			 4 -> FOF axiom with float numbers
%%%			 5 -> FOF axiom with large integer numbers
%%%		A: axiom
%%%		K: axiom key
%%%			comments and empty lines have the key "text"
%%%			documentations have the key "documentation"
%%%		NL: number of lines of the axiom
%%%		NRA: number of real axioms that have been written to the file
%%%		NUC: number of unit clauses that have been written to the file (axioms consisting of a single literal)
%%%		Action: the axiom is written in the output stream

% % Empty line -> All formats % %

writeAxiomWithText(S1,S2,_,_,-2,_,text,NL,0,0):-
		!,
		print_to_file(S2,'\n'),
		readLinesAxiom(S1,NL).

% % TSTP format % %

writeAxiomWithText(S1,S2,tstp,_,-4,_,'$comment',NL,0,0):-
		printTextAxiom(S1,S2,1,NL),
		!.
writeAxiomWithText(S1,S2,tstp,_,-3,_,'$documentation',NL,0,0):-
		printTextAxiom(S1,S2,1,NL),
		!.
writeAxiomWithText(S1,S2,tstp,_,-1,_,text,NL,0,0):-
		printTextAxiom(S1,S2,1,NL),
		!.
writeAxiomWithText(S1,S2,tstp,AP,0,F,K,NL,1,UC):-
		print_to_file(S2,'fof( '),
		print_to_file(S2,K),
		print_to_file(S2,', axiom,'),
		print_to_file(S2,'\n'),
		writeFormula(S2,0,tstp,AP,1,F,UC),
		!,
		print_to_file(S2,').'),
		print_to_file(S2,'\n'),
		print_to_file(S2,'\n'),
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,tstp,_,3,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,tstp,_,4,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,tstp,_,5,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,tstp,_,6,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).

% % TPTP format % %

writeAxiomWithText(S1,S2,tptp,_,-4,_,'$comment',NL,0,0):-
		printTextAxiom(S1,S2,1,NL),
		!.
writeAxiomWithText(S1,S2,tptp,_,-3,_,'$documentation',NL,0,0):-
		printTextAxiom(S1,S2,1,NL),
		!.
writeAxiomWithText(S1,S2,tptp,_,-1,_,text,NL,0,0):-
		printTextAxiom(S1,S2,1,NL),
		!.
writeAxiomWithText(S1,S2,tptp,AP,0,F,K,NL,1,UC):-
		!,
		print_to_file(S2,'input_formula( '),
		print_to_file(S2,K),
		print_to_file(S2,', axiom,'),
		print_to_file(S2,'\n'),
		writeFormula(S2,0,tptp,AP,1,F,UC),
		!,
		print_to_file(S2,').'),
		print_to_file(S2,'\n'),
		print_to_file(S2,'\n'),
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,tptp,_,3,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,tptp,_,4,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,tptp,_,5,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,tptp,_,6,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).

% % mace4 format % %

writeAxiomWithText(S1,S2,mace4,_,-4,_,'$comment',NL,0,0):-
		printTextAxiom(S1,S2,1,NL),
		!.
writeAxiomWithText(S1,S2,mace4,_,-3,_,'$documentation',NL,0,0):-
		printTextAxiom(S1,S2,1,NL),
		!.
writeAxiomWithText(S1,S2,mace4,_,-1,_,text,NL,0,0):-
		printTextAxiom(S1,S2,1,NL),
		!.
writeAxiomWithText(S1,S2,mace4,AP,0,F,_,NL,1,UC):-
		!,
		print_to_file(S2,'( '),
		print_to_file(S2,'\n'),
		writeFormula(S2,0,mace4,AP,1,F,UC),
		!,
		print_to_file(S2,').'),
		print_to_file(S2,'\n'),
		print_to_file(S2,'\n'),
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,mace4,_,3,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,mace4,_,4,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,mace4,_,5,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).
writeAxiomWithText(S1,S2,mace4,_,6,F,_,NL,0,0):-
		writeFormula(S2,1,kif,_,1,F,_),
		!,
		readLinesAxiom(S1,NL).

% % KIF format % %

writeAxiomWithText(S1,S2,kif,_,-4,_,'$comment',NL,0,0):-
		printTextAxiom(S1,S2,0,NL),
		!.
writeAxiomWithText(S1,S2,kif,_,-3,_,'$documentation',NL,0,0):-
		printTextAxiom(S1,S2,0,NL),
		!.
writeAxiomWithText(S1,S2,kif,_,-1,_,text,NL,0,0):-
		printTextAxiom(S1,S2,0,NL),
		!.
writeAxiomWithText(S1,S2,kif,_,N,F,_,NL,0,0):-
		(
		 (
		  N = 0
		 );
		 (
		  N = 3
		 );
		 (
		  N = 4
		 )
		),
		writeFormula(S2,0,kif,_,0,F,_),
		!,
		readLinesAxiom(S1,NL).

% % Error % %

writeAxiomWithText(_,_,L,AP,N,_,K,_,_,_):-
		print_to_screen('Error writting the axiom '),
		print_to_screen(K),
		print_to_screen('\n'),
		print_to_screen('Language --> '),
		print_to_screen(L),
		print_to_screen('\n'),
		print_to_screen('Automatic prover --> '),
		print_to_screen(AP),
		print_to_screen('\n'),
		print_to_screen('Axiom type --> '),
		print_to_screen(N),
		print_to_screen('\n'),
		fail.

%%% writeAxiomWithoutText(in S, in OL, in C, in T, in K, out NRA, out NUC)
%%%		S: output stream
%%%		OL: output language
%%%		C: axiom internal code
%%%			-3 -> Documentation
%%%			-2 -> Empty line
%%%			-1 -> Comment
%%%			 0 -> FOF axiom without row variables
%%%			 1 -> Non-FOF axiom (formulas as terms)
%%%			 2 -> Non-FOF axiom (variables as predicates)
%%%			 3 -> FOF axiom with row variables
%%%			 4 -> FOF axiom with float numbers
%%%			 5 -> FOF axiom with large integer numbers
%%%		A: axiom
%%%		K: axiom key
%%%			comments and empty lines have the key "text"
%%%			documentations have the key "documentation"
%%%		NRA: number of real axioms that have been written to the file
%%%		NUC: number of unit clauses that have been written to the file (axioms consisting of a single literal)
%%%		Action: the axiom is written in the output stream

% % TSTP format % %

writeAxiomWithoutText(S,tstp,AP,0,F,K,1,UC):-
		print_to_file(S,'fof( '),
		print_to_file(S,K),
		print_to_file(S,', axiom,'),
		print_to_file(S,'\n'),
		writeFormula(S,0,tstp,AP,1,F,UC),
		!,
		print_to_file(S,').'),
		print_to_file(S,'\n'),
		print_to_file(S,'\n').

% % TPTP format % %

writeAxiomWithoutText(S,tptp,AP,0,F,K,1,UC):-
		print_to_file(S,'input_formula( '),
		print_to_file(S,K),
		print_to_file(S,', axiom,'),
		print_to_file(S,'\n'),
		writeFormula(S,0,tptp,AP,1,F,UC),
		!,
		print_to_file(S,').'),
		print_to_file(S,'\n'),
		print_to_file(S,'\n').

% % mace4 format % %

writeAxiomWithoutText(S,mace4,AP,0,F,_,1,UC):-
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeFormula(S,0,mace4,AP,1,F,UC),
		!,
		print_to_file(S,').'),
		print_to_file(S,'\n'),
		print_to_file(S,'\n').

% % KIF format % %

writeAxiomWithoutText(S,kif,_,0,F,_,0,0):-
		writeFormula(S,0,kif,_,0,F,_),
		!.

% % Error % %

writeAxiomWithoutText(_,L,AP,N,_,K,_,_):-
		print_to_screen('Error writting the axiom '),
		print_to_screen(K),
		print_to_screen('\n'),
		print_to_screen('Language --> '),
		print_to_screen(L),
		print_to_screen('\n'),
		print_to_screen('Automatic prover --> '),
		print_to_screen(AP),
		print_to_screen('\n'),
		print_to_screen('Axiom type --> '),
		print_to_screen(N),
		print_to_screen('\n'),
		fail.

%%% writeFormula(in S, in C, in OL, in AP, in T, in F, out NUC)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		F: a formula
%%%		NUC: number of unit clauses that have been written to the file (axioms consisting of a single literal)
%%%		Action: the formula is written in the output stream

% % TSTP format % %

writeFormula(S,C,tstp,AP,T,exists(LV,F),0):-
print_to_screen('Warning! Multi-sorted existentially quantified variable\n'),
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'?'),
		print_to_file(S,'['),
		writeListOfVariables(S,tstp,LV),
		print_to_file(S,']'),
		print_to_file(S,': '),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tstp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tstp,AP,T,forall(LV,F),0):-
print_to_screen('Warning! Multi-sorted universally quantified variable\n'),
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'!'),
		print_to_file(S,'['),
		writeListOfVariables(S,tstp,LV),
		print_to_file(S,']'),
		print_to_file(S,': '),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tstp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tstp,AP,T,'exists*'(LV,F),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'?'),
		print_to_file(S,'['),
		writeListOfVariables(S,tstp,LV),
		print_to_file(S,']'),
		print_to_file(S,': '),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tstp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tstp,AP,T,'forall*'(LV,F),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'!'),
		print_to_file(S,'['),
		writeListOfVariables(S,tstp,LV),
		print_to_file(S,']'),
		print_to_file(S,': '),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tstp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tstp,AP,T,and(LF),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeListOfFormulasAnd(S,C,tstp,AP,T,LF),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tstp,AP,T,or(LF),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeListOfFormulasOr(S,C,tstp,AP,T,LF),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tstp,AP,T,implies(F1,F2),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,tstp,AP,NT,F1,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'=>'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeFormula(S,C,tstp,AP,NT,F2,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tstp,AP,T,equiv(F1,F2),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,tstp,AP,NT,F1,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'<=>'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeFormula(S,C,tstp,AP,NT,F2,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tstp,AP,T,not(F),UC):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'~ '),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,tstp,AP,NT,F,UC),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tstp,AP,T,A,UC):-
		printCommentMark(S,C),
		printTabs(S,T),
		writeAtom(S,C,tstp,AP,T,A),
		(
		 (
		  C = 1,
		  !,
		  UC = 0
		 );
		 (
		  C = 0,
		  UC = 1
		 )
		).

% % TPTP format % %

writeFormula(S,C,tptp,AP,T,exists(LV,F),0):-
print_to_screen('Warning! Multi-sorted existentially quantified variable\n'),
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'?'),
		print_to_file(S,'['),
		writeListOfVariables(S,tptp,LV),
		print_to_file(S,']'),
		print_to_file(S,': '),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tptp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tptp,AP,T,forall(LV,F),0):-
print_to_screen('Warning! Multi-sorted universally quantified variable\n'),
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'!'),
		print_to_file(S,'['),
		writeListOfVariables(S,tptp,LV),
		print_to_file(S,']'),
		print_to_file(S,': '),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tptp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tptp,AP,T,'exists*'(LV,F),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'?'),
		print_to_file(S,'['),
		writeListOfVariables(S,tptp,LV),
		print_to_file(S,']'),
		print_to_file(S,': '),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tptp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tptp,AP,T,'forall*'(LV,F),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'!'),
		print_to_file(S,'['),
		writeListOfVariables(S,tptp,LV),
		print_to_file(S,']'),
		print_to_file(S,': '),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tptp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tptp,AP,T,and(LF),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeListOfFormulasAnd(S,C,tptp,AP,T,LF),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tptp,AP,T,or(LF),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeListOfFormulasOr(S,C,tptp,AP,T,LF),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tptp,AP,T,implies(F1,F2),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,tptp,AP,NT,F1,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'=>'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeFormula(S,C,tptp,AP,NT,F2,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tptp,AP,T,equiv(F1,F2),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,tptp,AP,NT,F1,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'<=>'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeFormula(S,C,tptp,AP,NT,F2,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tptp,AP,T,not(F),UC):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'~ '),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,tptp,AP,NT,F,UC),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,tptp,AP,T,A,UC):-
		printCommentMark(S,C),
		printTabs(S,T),
		writeAtom(S,C,tptp,AP,T,A),
		(
		 (
		  C = 1,
		  !,
		  UC = 0
		 );
		 (
		  C = 0,
		  UC = 1
		 )
		).

% % mace4 format % %

writeFormula(S,C,mace4,AP,T,exists(LV,F),0):-
print_to_screen('Warning! Multi-sorted existentially quantified variable\n'),
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'( '),
		writeExistentialQuantifiers(S,mace4,LV),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,mace4,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,mace4,AP,T,forall(LV,F),0):-
print_to_screen('Warning! Multi-sorted universally quantified variable\n'),
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'( '),
		writeUniversalQuantifiers(S,mace4,LV),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,mace4,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,mace4,AP,T,'exists*'(LV,F),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'( '),
		writeExistentialQuantifiers(S,mace4,LV),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,mace4,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,mace4,AP,T,'forall*'(LV,F),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'( '),
		writeUniversalQuantifiers(S,mace4,LV),
		print_to_file(S,'\n'),
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,mace4,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,mace4,AP,T,and(LF),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeListOfFormulasAnd(S,C,mace4,AP,T,LF),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,mace4,AP,T,or(LF),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeListOfFormulasOr(S,C,mace4,AP,T,LF),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,mace4,AP,T,implies(F1,F2),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,mace4,AP,NT,F1,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'->'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeFormula(S,C,mace4,AP,NT,F2,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,mace4,AP,T,equiv(F1,F2),0):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,mace4,AP,NT,F1,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'<->'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		writeFormula(S,C,mace4,AP,NT,F2,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,mace4,AP,T,not(F),UC):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'- '),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,mace4,AP,NT,F,UC),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,mace4,AP,T,A,UC):-
		printCommentMark(S,C),
		printTabs(S,T),
		writeAtom(S,C,mace4,AP,T,A),
		(
		 (
		  C = 1,
		  !,
		  UC = 0
		 );
		 (
		  C = 0,
		  UC = 1
		 )
		).

% % KIF format % %

writeFormula(S,C,kif,_,T,exists(LV,F),_):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'exists '),
		print_to_file(S,'('),
		writeListOfVariables(S,kif,LV),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,kif,_,NT,F,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,kif,_,T,forall(LV,F),_):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'forall '),
		print_to_file(S,'('),
		writeListOfVariables(S,kif,LV),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,kif,_,NT,F,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,kif,_,T,'exists*'(LV,F),_):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'exists* '),
		print_to_file(S,'('),
		writeListOfVariables(S,kif,LV),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,kif,_,NT,F,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,kif,_,T,'forall*'(LV,F),_):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'forall* '),
		print_to_file(S,'('),
		writeListOfVariables(S,kif,LV),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,kif,_,NT,F,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,kif,_,T,and(LF),_):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'and '),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeListOfFormulas(S,C,kif,NT,LF),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,kif,_,T,or(LF),_):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'or '),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeListOfFormulas(S,C,kif,NT,LF),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,kif,_,T,implies(F1,F2),_):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'=> '),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,kif,_,NT,F1,_),
		writeFormula(S,C,kif,_,NT,F2,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,kif,_,T,equiv(F1,F2),_):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'<=> '),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,kif,_,NT,F1,_),
		writeFormula(S,C,kif,_,NT,F2,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,kif,_,T,not(F),_):-
		!,
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,'('),
		print_to_file(S,'not '),
		print_to_file(S,'\n'),
		NT is T + 1,
		writeFormula(S,C,kif,_,NT,F,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeFormula(S,C,kif,_,T,A,_):-
		printCommentMark(S,C),
		printTabs(S,T),
		writeAtom(S,C,kif,_,T,A).

%%% writeListOfFormulasAnd(in S, in C, in OL, in AP, in LT, in LF)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language --> tstp or tptp
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		LF: a list of formulas in conjunction
%%%		Action: the formula is written in the output stream

% % TSTP format % %

writeListOfFormulasAnd(_,_,tstp,_,_,[]).
writeListOfFormulasAnd(S,C,tstp,AP,T,[F]):-
		!,
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tstp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeListOfFormulasAnd(S,C,tstp,AP,T,[F|LF]):-
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tstp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,' &'),
		print_to_file(S,'\n'),
		writeListOfFormulasAnd(S,C,tstp,AP,T,LF).

% % TPTP format % %

writeListOfFormulasAnd(_,_,tptp,_,_,[]).
writeListOfFormulasAnd(S,C,tptp,AP,T,[F]):-
		!,
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tptp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeListOfFormulasAnd(S,C,tptp,AP,T,[F|LF]):-
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,tptp,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,' &'),
		print_to_file(S,'\n'),
		writeListOfFormulasAnd(S,C,tptp,AP,T,LF).

% % mace4 format % %

writeListOfFormulasAnd(_,_,mace4,_,_,[]).
writeListOfFormulasAnd(S,C,mace4,AP,T,[F]):-
		!,
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,mace4,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n').
writeListOfFormulasAnd(S,C,mace4,AP,T,[F|LF]):-
		NT1 is T + 1,
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,'('),
		print_to_file(S,'\n'),
		NT2 is NT1 + 1,
		writeFormula(S,C,mace4,AP,NT2,F,_),
		printCommentMark(S,C),
		printTabs(S,NT1),
		print_to_file(S,')'),
		print_to_file(S,'\n'),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,' &'),
		print_to_file(S,'\n'),
		writeListOfFormulasAnd(S,C,mace4,AP,T,LF).

%%% writeListOfFormulasOr(in S, in C, in OL, in AP, in LT, in F)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language --> tstp or tptp
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		LF: a list of formulas in disjunction
%%%		Action: the formula is written in the output stream

% % TSTP format % %

writeListOfFormulasOr(_,_,tstp,_,_,[]).
writeListOfFormulasOr(S,C,tstp,AP,T,[F]):-
		!,
		NT is T + 1,
		writeFormula(S,C,tstp,AP,NT,F,_).
writeListOfFormulasOr(S,C,tstp,AP,T,[F|LF]):-
		NT is T + 1,
		writeFormula(S,C,tstp,AP,NT,F,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,' |'),
		print_to_file(S,'\n'),
		writeListOfFormulasOr(S,C,tstp,AP,T,LF).

% % TPTP format % %

writeListOfFormulasOr(_,_,tptp,_,_,[]).
writeListOfFormulasOr(S,C,tptp,AP,T,[F]):-
		!,
		NT is T + 1,
		writeFormula(S,C,tptp,AP,NT,F,_).
writeListOfFormulasOr(S,C,tptp,AP,T,[F|LF]):-
		NT is T + 1,
		writeFormula(S,C,tptp,AP,NT,F,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,' |'),
		print_to_file(S,'\n'),
		writeListOfFormulasOr(S,C,tptp,AP,T,LF).

% % mace4 format % %

writeListOfFormulasOr(_,_,mace4,_,_,[]).
writeListOfFormulasOr(S,C,mace4,AP,T,[F]):-
		!,
		NT is T + 1,
		writeFormula(S,C,mace4,AP,NT,F,_).
writeListOfFormulasOr(S,C,mace4,AP,T,[F|LF]):-
		NT is T + 1,
		writeFormula(S,C,mace4,AP,NT,F,_),
		printCommentMark(S,C),
		printTabs(S,T),
		print_to_file(S,' |'),
		print_to_file(S,'\n'),
		writeListOfFormulasOr(S,C,mace4,AP,T,LF).

%%% writeListOfFormulas(in S, in C, in OL, in LT, in LF)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language --> kif
%%%		T: number of tabs
%%%		LF: a list of formulas
%%%		Action: the formula is written in the output stream

% % KIF format % %

writeListOfFormulas(_,_,kif,_,[]).
writeListOfFormulas(S,C,kif,T,[F|LF]):-
		writeFormula(S,C,kif,_,T,F,_),
		writeListOfFormulas(S,C,kif,T,LF).

%%% writeAtom(in S, in C, in OL, in AP, in T, in A)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		A: an atom
%%%		Action: the atom is written in the output stream

% % Any format % %

writeAtom(S,_,_,_,_,false):-
		!,
		print_to_file(S,'false'),
		print_to_file(S,'\n').
writeAtom(S,_,_,_,_,true):-
		!,
		print_to_file(S,'true'),
		print_to_file(S,'\n').

% % TSTP format % %

writeAtom(S,C,tstp,AP,T,equal(T1,T2)):-
		!,
		print_to_file(S,'( '),
		writeTerm(S,C,tstp,AP,T,T1),
		print_to_file(S,' = '),
		NT is T + 1,
		writeTerm(S,C,tstp,AP,NT,T2),
		print_to_file(S,' )'),
		print_to_file(S,'\n').
writeAtom(S,C,tstp,AP,T,A):- % Corregir codigo, tener en cuenta el prover
		!,
		A =.. [P|ST],
		writePredicate(S,C,tstp,AP,T,P),
		print_to_file(S,'('),
		NT is T + 1,
		writeAtomSubterms(S,C,tstp,AP,NT,ST),
		print_to_file(S,')'),
		print_to_file(S,'\n').

% % TPTP format % %

writeAtom(S,C,tptp,AP,T,equal(T1,T2)):-
		!,
		print_to_file(S,'equal( '),
		writeTerm(S,C,tptp,AP,T,T1),
		print_to_file(S,', '),
		NT is T + 1,
		writeTerm(S,C,tptp,AP,NT,T2),
		print_to_file(S,' )'),
		print_to_file(S,'\n').
writeAtom(S,C,tptp,AP,T,A):-
		!,
		A =.. [P|ST],
		writePredicate(S,C,tptp,AP,T,P),
		print_to_file(S,'('),
		NT is T + 1,
		writeAtomSubterms(S,C,tptp,AP,NT,ST),
		print_to_file(S,')'),
		print_to_file(S,'\n').

% % mace4 format % %

writeAtom(S,C,mace4,AP,T,equal(T1,T2)):-
		!,
		print_to_file(S,'( '),
		writeTerm(S,C,mace4,AP,T,T1),
		print_to_file(S,' = '),
		NT is T + 1,
		writeTerm(S,C,mace4,AP,NT,T2),
		print_to_file(S,' )'),
		print_to_file(S,'\n').
writeAtom(S,C,mace4,AP,T,A):- % Corregir codigo, tener en cuenta el prover
		!,
		A =.. [P|ST],
		writePredicate(S,C,mace4,AP,T,P),
		print_to_file(S,'('),
		NT is T + 1,
		writeAtomSubterms(S,C,mace4,AP,NT,ST),
		print_to_file(S,')'),
		print_to_file(S,'\n').

% % KIF format % %

writeAtom(S,C,kif,_,T,A):-
		A =..[P|ST],
		print_to_file(S,'('),
		print_to_file(S,P),
		NT is T + 1,
		writeAtomSubterms(S,C,kif,_,NT,ST),
		print_to_file(S,')'),
		print_to_file(S,'\n').

%%% writePredicate(in S, in C, in OL, in AP, in T, in P)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		P: a predicate
%%%		Action: the predicate is written in the output stream

% % Vampire 8.1 format % %


writePredicate(S,_,_,vampire,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('p__d__',N,NP),
		print_to_file(S,NP).
writePredicate(S,_,_,vampire,_,P):-
		atom_concat('p__',P,NP),
		print_to_file(S,NP).

% % E Prover format % %

writePredicate(S,_,_,eprover,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('p__d__',N,NP),
		print_to_file(S,NP).
writePredicate(S,_,_,eprover,_,P):-
		atom_concat('p__',P,NP),
		print_to_file(S,NP).

% % Vampire 7 format % %

writePredicate(S,_,_,vampire7,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('p__d__',N,NP),
		print_to_file(S,NP).
writePredicate(S,_,_,vampire7,_,P):-
		atom_concat('p__',P,NP),
		print_to_file(S,NP).

% % tptpSUMO format % %

writePredicate(S,_,_,tptpSUMO,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('s__',N,NP),
		print_to_file(S,NP).
writePredicate(S,_,_,tptpSUMO,_,P):-
		atom_concat('s__',P,NP),
		print_to_file(S,NP).

% % mace4 format % %

writePredicate(S,_,_,mace4,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('dp__',N,NP),
		print_to_file(S,NP).
writePredicate(S,_,_,mace4,_,P):-
		atom_concat('p__',P,NP),
		print_to_file(S,NP).

%%% writeAtomSubterms(in S, in C, in OL, in AP, in T, in ST)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		ST: a list of subterms
%%%		Action: the subterms of the atom are written in the output stream

% % TSTP format % %

writeAtomSubterms(_,_,tstp,_,_,[]):-
		!.
writeAtomSubterms(S,C,tstp,AP,T,[H]):-
		!,
		writeTerm(S,C,tstp,AP,T,H).
writeAtomSubterms(S,C,tstp,AP,T,[H|ST]):-
		writeTerm(S,C,tstp,AP,T,H),
		print_to_file(S,','),
		NT is T + 1,
		writeAtomSubterms(S,C,tstp,AP,NT,ST).

% % TPTP format % %

writeAtomSubterms(_,_,tptp,_,_,[]):-
		!.
writeAtomSubterms(S,C,tptp,AP,T,[H]):-
		!,
		writeTerm(S,C,tptp,AP,T,H).
writeAtomSubterms(S,C,tptp,AP,T,[H|ST]):-
		writeTerm(S,C,tptp,AP,T,H),
		print_to_file(S,','),
		NT is T + 1,
		writeAtomSubterms(S,C,tptp,AP,NT,ST).

% % mace4 format % %

writeAtomSubterms(_,_,mace4,_,_,[]):-
		!.
writeAtomSubterms(S,C,mace4,AP,T,[H]):-
		!,
		writeTerm(S,C,mace4,AP,T,H).
writeAtomSubterms(S,C,mace4,AP,T,[H|ST]):-
		writeTerm(S,C,mace4,AP,T,H),
		print_to_file(S,','),
		NT is T + 1,
		writeAtomSubterms(S,C,mace4,AP,NT,ST).

% % KIF format % %

writeAtomSubterms(_,_,kif,_,_,[]):-
		!.
writeAtomSubterms(S,C,kif,_,T,[H|ST]):-
		print_to_file(S,' '),
		writeTerm(S,C,kif,_,T,H),
		NT is T + 1,
		writeAtomSubterms(S,C,kif,_,NT,ST).

%%% writeTerm(in S, in C, in OL, in AP, in T, in TE)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		TE: a term
%%%		Action: the term is written in the output stream

% % TSTP format % %

writeTerm(S,_,tstp,_,_,'#var'(V)):-
		!,
		writeVariable(S,tstp,V).
writeTerm(S,C,tstp,AP,T,A):-
		compound(A),
		!,
		A =..[F|ST],
		length(ST,N),
		writeFunction(S,C,tstp,AP,T,F,N),
		print_to_file(S,'('),
		writeSubterms(S,C,tstp,AP,T,ST,','),
		print_to_file(S,')').
writeTerm(S,C,tstp,AP,T,A):-
		writeConstant(S,C,tstp,AP,T,A).

% % TPTP format % %

writeTerm(S,_,tptp,_,_,'#var'(V)):-
		!,
		writeVariable(S,tptp,V).
writeTerm(S,C,tptp,AP,T,A):-
		compound(A),
		!,
		A =..[F|ST],
		length(ST,N),
		writeFunction(S,C,tptp,AP,T,F,N),
		print_to_file(S,'('),
		writeSubterms(S,C,tptp,AP,T,ST,','),
		print_to_file(S,')').
writeTerm(S,C,tptp,AP,T,A):-
		writeConstant(S,C,tptp,AP,T,A).

% % mace4 format % %

writeTerm(S,_,mace4,_,_,'#var'(V)):-
		!,
		writeVariable(S,mace4,V).
writeTerm(S,C,mace4,AP,T,A):-
		compound(A),
		!,
		A =..[F|ST],
		length(ST,N),
		writeFunction(S,C,mace4,AP,T,F,N),
		print_to_file(S,'('),
		writeSubterms(S,C,mace4,AP,T,ST,','),
		print_to_file(S,')').
writeTerm(S,C,mace4,AP,T,A):-
		writeConstant(S,C,mace4,AP,T,A).

% % KIF format % %

writeTerm(S,_,kif,_,_,'#var'(V)):-
		!,
		writeVariable(S,kif,V).
writeTerm(S,_,kif,_,_,'#rowVar'(V)):-
		!,
		writeRowVariable(S,kif,V).
writeTerm(S,_,kif,_,_,'#row'(L)):-
		!,
		writeMultiLengthList(S,kif,L).
writeTerm(S,_,kif,_,_,'#pred'(P)):-
		!,
		writeVariablePredicate(S,kif,P).
writeTerm(S,C,kif,AP,T,A):-
		compound(A),
		!,
		A =..[F|ST],
		length(ST,N),
		print_to_file(S,'('),
		writeFunction(S,C,kif,AP,T,F,N),
		print_to_file(S,' '),
		writeSubterms(S,C,kif,AP,T,ST,' '),
		print_to_file(S,')').
writeTerm(S,C,kif,AP,T,A):-
		writeConstant(S,C,kif,AP,T,A).

%%% writeConstant(in S, in C, in OL, in AP, in T, in CO)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		CO: a constant
%%%		Action: the constant is written in the output stream

% % KIF format % %

writeConstant(S,_,kif,_,_,P):-
		!,
		print_to_file(S,P).

% % Vampire 8.1 format % %

writeConstant(S,_,_,vampire,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('c__d__',N,NP),
		print_to_file(S,NP).
writeConstant(S,_,_,vampire,_,P):-
		atom_concat('c__',P,NP),
		print_to_file(S,NP).

% % E Prover format % %

writeConstant(S,_,_,eprover,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('c__d__',N,NP),
		print_to_file(S,NP).
writeConstant(S,_,_,eprover,_,P):-
		atom_concat('c__',P,NP),
		print_to_file(S,NP).

% % Vampire 7 format % %

writeConstant(S,_,_,vampire7,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('c__d__',N,NP),
		print_to_file(S,NP).
writeConstant(S,_,_,vampire7,_,P):-
		atom_concat('c__',P,NP),
		print_to_file(S,NP).

% % tptpSUMO format % %

writeConstant(S,_,_,tptpSUMO,_,P):-
		isPredicate(P,2),
		!,
		atom_concat('s__',P,P1),
		atom_concat(P1,'__m',NP),
		print_to_file(S,NP).
writeConstant(S,_,_,tptpSUMO,_,P):-
		number(P),
		!,
		print_to_file(S,P).		
writeConstant(S,_,_,tptpSUMO,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('s__',N,NP),
		print_to_file(S,NP).
writeConstant(S,_,_,tptpSUMO,_,P):-
		atom_concat('s__',P,NP),
		print_to_file(S,NP).

% % mace4 format % %

writeConstant(S,_,_,mace4,_,P):-
		atom_codes(P,[H|LC]),
		char_code('$',H),
		!,
		atom_codes(N,LC),
		atom_concat('dc__',N,NP),
		print_to_file(S,NP).
writeConstant(S,_,_,mace4,_,P):-
		atom_concat('c__',P,NP),
		print_to_file(S,NP).

%%% writeFunction(in S, in C, in OL, in AP, in T, in F, in A)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		F: a function symbol
%%%		A: an arity
%%%		Action: the function is written in the output stream

% % KIF format % %

writeFunction(S,_,kif,_,_,F,_):-
		!,
		print_to_file(S,F).

% % Vampire 8.1 format % %

writeFunction(S,_,_,vampire,_,F,A):-
		atom_concat('f__',F,NF),
		number_codes(A,LC),
		atom_codes(N,LC),
		atom_concat(NF,N,C),
		print_to_file(S,C).

% % E Prover format % %

writeFunction(S,_,_,eprover,_,F,A):-
		atom_concat('f__',F,NF),
		number_codes(A,LC),
		atom_codes(N,LC),
		atom_concat(NF,N,C),
		print_to_file(S,C).

% % Vampire 7 format % %

writeFunction(S,_,_,vampire7,_,F,A):-
		atom_concat('f__',F,NF),
		number_codes(A,LC),
		atom_codes(N,LC),
		atom_concat(NF,N,C),
		print_to_file(S,C).

% % tptpSUMO format % %

writeFunction(S,_,_,tptpSUMO,_,F,A):-
		atom_concat('s__',F,NF),
		number_codes(A,LC),
		atom_codes(N,LC),
		atom_concat(NF,N,C),
		print_to_file(S,C).

% % mace4 format % %

writeFunction(S,_,_,mace4,_,F,A):-
		atom_concat('f__',F,NF),
		number_codes(A,LC),
		atom_codes(N,LC),
		atom_concat(NF,N,C),
		print_to_file(S,C).

%%% writeSubterms(in S, in C, in OL, in AP, in T, in ST, in D)
%%%		S: output stream
%%%		C: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		OL: output language
%%%		AP: automatic prover
%%%		T: number of tabs
%%%		ST: a list of subterms
%%%		D: the separator of subterms
%%%		Action: the function is written in the output stream

writeSubterms(S,C,OL,AP,T,[ST],_):-
		!,
		writeTerm(S,C,OL,AP,T,ST).
writeSubterms(S,C,OL,AP,T,[ST|LST],D):-
		writeTerm(S,C,OL,AP,T,ST),
		print_to_file(S,D),
		writeSubterms(S,C,OL,AP,T,LST,D).

%%% writeListOfVariables(in S, in OL, in LV)
%%%		S: output stream
%%%		OL: output language
%%%		LV: a list of variables
%%%		Action: the variable is written in the output stream

% % TSTP format % %

writeListOfVariables(_,_,[]).
writeListOfVariables(S,tstp,['#var'(V)]):-
		!,
		writeVariable(S,tstp,V).
writeListOfVariables(S,tstp,['#var'(V)|LV]):-
		writeVariable(S,tstp,V),
		print_to_file(S,','),
		writeListOfVariables(S,tstp,LV).

% % TPTP format % %

writeListOfVariables(_,_,[]).
writeListOfVariables(S,tptp,['#var'(V)]):-
		!,
		writeVariable(S,tptp,V).
writeListOfVariables(S,tptp,['#var'(V)|LV]):-
		writeVariable(S,tptp,V),
		print_to_file(S,','),
		writeListOfVariables(S,tptp,LV).

% % KIF format % %

writeListOfVariables(_,_,[]).
writeListOfVariables(S,kif,['#var'(V)]):-
		!,
		writeVariable(S,kif,V).
writeListOfVariables(S,kif,['#rowVar'(V)]):-
		!,
		writeRowVariable(S,kif,V).
writeListOfVariables(S,kif,['#var'(V)|LV]):-
		!,
		writeVariable(S,kif,V),
		print_to_file(S,' '),
		writeListOfVariables(S,kif,LV).
writeListOfVariables(S,kif,['#rowVar'(V)|LV]):-
		writeRowVariable(S,kif,V),
		print_to_file(S,' '),
		writeListOfVariables(S,kif,LV).

%%% writeExistentialQuantifiers(in S, in OL, in LV)
%%%		S: output stream
%%%		OL: output language
%%%		LV: a list of variables
%%%		Action: existential quantifiers are written in the output stream

% % mace4 format % %

writeExistentialQuantifiers(_,_,[]).
writeExistentialQuantifiers(S,mace4,['#var'(V)|LV]):-
		print_to_file(S,'exists '),
		writeVariable(S,mace4,V),
		print_to_file(S,' '),
		writeExistentialQuantifiers(S,mace4,LV).

%%% writeUniversalQuantifiers(in S, in OL, in LV)
%%%		S: output stream
%%%		OL: output language
%%%		LV: a list of variables
%%%		Action: universal quantifiers are written in the output stream

% % mace4 format % %

writeUniversalQuantifiers(_,_,[]).
writeUniversalQuantifiers(S,mace4,['#var'(V)|LV]):-
		print_to_file(S,'all '),
		writeVariable(S,mace4,V),
		print_to_file(S,' '),
		writeUniversalQuantifiers(S,mace4,LV).

%%% writeVariable(in S, in OL, in V)
%%%		S: output stream
%%%		OL: output language
%%%		V: a variable
%%%		Action: the variable is written in the output stream

% % TSTP format % %

writeVariable(S,tstp,V):-
		print_to_file(S,V).

% % TPTP format % %

writeVariable(S,tptp,V):-
		print_to_file(S,V).

% % mace4 format % %

writeVariable(S,mace4,V):-
		print_to_file(S,V).

% % KIF format % %

writeVariable(S,kif,V):-
		print_to_file(S,'?'),
		print_to_file(S,V).

%%% writeRowVariable(in S, in OL, in V)
%%%		S: output stream
%%%		OL: output language
%%%		V: a row variable
%%%		Action: the row variable is written in the output stream

% % KIF format % %

writeRowVariable(S,kif,V):-
		print_to_file(S,'@'),
		print_to_file(S,V).

%%% writeMultiLengthList(in S, in OL, in L)
%%%		S: output stream
%%%		OL: output language
%%%		L: a multi-length list
%%%		Action: the multi-length list is written in the output stream

% % KIF format % %

writeMultiLengthList(S,kif,[E|L]):-
		print_to_file(S,'@('),
		writeListOfSubterms(S,kif,[E|L]),
		print_to_file(S,')').

writeMultiLengthList(S,kif,T):-													% It is not a multi-length list. In fact, it is a row type
		print_to_file(S,'@'),
		print_to_file(S,T).


%%% writeListOfSubterms(in S, in OL, in L)
%%%		S: output stream
%%%		OL: output language
%%%		L: a list of terms
%%%		Action: the list of terms is written in the output stream

writeListOfSubterms(_,kif,[]).
writeListOfSubterms(S,kif,[E]):-
		!,
		writeTerm(S,_,kif,_,_,E).
writeListOfSubterms(S,kif,[E|L]):-
		!,
		writeTerm(S,_,kif,_,_,E),
		print_to_file(S,','),
		writeListOfSubterms(S,kif,L).

%%% writeVariablePredicate(in S, in OL, in V)
%%%		S: output stream
%%%		OL: output language
%%%		V: a variable predicate
%%%		Action: the variable predicate is written in the output stream

% % KIF format % %

writeVariablePredicate(S,kif,V):-
		print_to_file(S,'!'),
		print_to_file(S,V).

%%% printTabs(in S, in T)
%%%		S: output stream
%%%		T: a number of tabs
%%%		Action: T tabs are written in the output stream

printTabs(_,0):-
		!.
printTabs(S,T):-
		print_to_file(S,'\t'),
		NT is T - 1,
		printTabs(S,NT).

%%% printCommentMark(in S, in F)
%%%		S: output stream
%%%		F: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		Action: the comment mark is written in output stream

printCommentMark(_,0).
printCommentMark(S,1):-
		print_to_file(S,'%'),
		print_to_file(S,' ').

%%% printTextAxiom(in S1, in S2, in C, in NL)
%%%		S1: input stream
%%%		S2: output stream
%%%		F: a flag that indicates
%%%			0	no comment mark
%%%			1	comment mark
%%%		NL: number of lines

printTextAxiom(_,_,_,0):-
		!.
printTextAxiom(S1,S2,C,NL):-
		read_next_line(S1,LC),
		atom_codes(L,LC),
		printCommentMark(S2,C),
		print_to_file(S2,L),
		print_to_file(S2,'\n'),
		NNL is NL - 1,
		printTextAxiom(S1,S2,C,NNL).

%%% readLinesAxiom(in S, in NL)
%%%		S: input stream
%%%		NL: number of lines

readLinesAxiom(_,0):-
		!.
readLinesAxiom(S,NL):-
		read_next_line(S,_),
		NNL is NL - 1,
		readLinesAxiom(S,NNL).

%%% Input-output predicates %%%

%%% print_text_axiom(in T)
%%%		T: the text of an axiom

print_text_axiom([]).
print_text_axiom([LC|T]):-
		atom_codes(L,LC),
		print_to_screen(L),
		print_to_screen('\n'),
		print_text_axiom(T).

%%% print_to_file(in S, in T)
%%%		S: output strem
%%%		T: term
%%%		Action: the term is written in S

print_to_file(S,T):-
		write_term(S,T,[]).

%%% print_to_screen(in T)
%%%		T: term
%%%		Action: the term is prompted

print_to_screen(T):-
		current_output(S),
		print_to_file(S,T).
