%%% writeGoals
%%%		Action: write goals in the output files

writeGoals:-
		findall((F,N),numberOfGoals(F,N),LFN),
		catch(make_directory('Goals'),_,true),
		writeGoalsListOfFiles(LFN),
		catch(make_directory('Runs'),_,true),
		print_to_screen('\tExecution scripts ...\n'),
		writeExecutionScripts.

%%% writeGoalsListOfFiles(in LFN)
%%%		LFN: a list of pairs (F,N) where
%%%			F is the file name
%%%			N is the number of goals
%%%		Action: write goals in the output files

writeGoalsListOfFiles([]).
writeGoalsListOfFiles([(F,N)|LFN]):-
		getAutomaticProver(AP),
		getOutputLanguage(OL),
		writeGoalsFile(F,OL,AP,1,N,RG),
		print_to_screen('\t'),
		print_to_screen('File '),
		print_to_screen(F),
		print_to_screen(' completed (total goals --> '),
		print_to_screen(RG),
		print_to_screen(').'),
		print_to_screen('\n'),
		writeGoalsListOfFiles(LFN).

%%% writeGoalsFile(in F, in OL, in AP, in CG, in NG, out NRG)
%%%		F: a file name
%%%		OL: output language
%%%		AP: the automatic prover to be used
%%%		IG: index of goals
%%%		NG: total number of goals in the file
%%%		NRG: number of real goals that have been written to the file
%%%		Action: write automatically sorted goals in the output file

writeGoalsFile(_,_,_,IG,NG,0):-
		IG > NG,
		!.
writeGoalsFile(F,OL,AP,IG,NG,NRG):-
		getGoalKey(F,IG,K),
		getGoal(K,G),
		atom_concat('Goals/',K,K1),
		atom_concat(K1,'.adimen.',K2),
		atom_concat(K2,OL,NK),
		(
		 (
		  AP = 'vanhelsing',
		  !,
		  NAP = 'eprover'
		 );
		 (
		  NAP = AP
		 )
		),
		open(NK,write,S),
		writeGoal(S,OL,NAP,G,K,RG1),
		close(S),
		NIG is IG + 1,
		writeGoalsFile(F,OL,AP,NIG,NG,RG2),
		NRG is RG1 + RG2.

%%% writeGoal(in S, in OL, in AP, in G, in K, out NRA)
%%%		S: output stream
%%%		OL: output language
%%%		AP: the automatic prover to be used
%%%		G: goal
%%%		K: goal key
%%%		NRA: number of real goals that have been written to the file
%%%		Action: the goal is written in the output stream

% % TSTP format % %

writeGoal(S,tstp,AP,G,K,1):-
		print_to_file(S,'fof( '),
		print_to_file(S,K),
		print_to_file(S,', conjecture,'),
		print_to_file(S,'\n'),
		writeFormula(S,0,tstp,AP,1,G,_),
		!,
		print_to_file(S,').'),
		print_to_file(S,'\n'),
		print_to_file(S,'\n').

% % TPTP format % %

writeGoal(S,tptp,AP,G,K,1):-
		print_to_file(S,'input_formula( '),
		print_to_file(S,K),
		print_to_file(S,', conjecture,'),
		print_to_file(S,'\n'),
		writeFormula(S,0,tptp,AP,1,G,_),
		!,
		print_to_file(S,').'),
		print_to_file(S,'\n'),
		print_to_file(S,'\n').

% % KIF format % %

writeGoal(S,kif,_,G,_,0):-
		writeFormula(S,0,kif,_,0,G,_),
		!.

% % Error % %

writeGoal(_,L,AP,_,K,_):-
		print_to_screen('Error writting the goal '),
		print_to_screen(K),
		print_to_screen('\n'),
		print_to_screen('Language --> '),
		print_to_screen(L),
		print_to_screen('\n'),
		print_to_screen('Automatic prover --> '),
		print_to_screen(AP),
		print_to_screen('\n'),
		fail.

%%% Input-output predicates %%%

%%% print_text_goal(in T)
%%%		T: the text of an goal

print_text_goal([]).
print_text_goal([LC|T]):-
		atom_codes(L,LC),
		print_to_screen(L),
		print_to_screen('\n'),
		print_text_goal(T).


%%% writeExecutionScripts
%%%		Action:	scripts for the execution of tests are sorted

writeExecutionScripts:-
		open('Runs/initTests.sh',write,S),
		writeSourceCommand(S),
		close(S),
		writeFastTestExecutionScripts('goal',1,600),
		writeFastTestExecutionScripts('multipleMapping',1,600),
		writeFastTestExecutionScripts('equalEvent',1,600),
		writeFastTestExecutionScripts('subclassEvent',1,600),
		writeFastTestExecutionScripts('commonSubclassEvent',1,600),
		writeFastTestExecutionScripts('antonymPattern1',1,600),
		writeFastTestExecutionScripts('antonymPattern2',1,600),
		writeFastTestExecutionScripts('antonymPattern3',1,600),
		writeFastTestExecutionScripts('agentRelation',1,600),
		writeFastTestExecutionScripts('instrumentRelation',1,600),
		writeFastTestExecutionScripts('resultRelation',1,600),
		writeFastTestExecutionScripts('negatedMultipleMapping',1,600),
		writeFastTestExecutionScripts('negatedEqualEvent',1,600),
		writeFastTestExecutionScripts('negatedSubclassEvent',1,600),
		writeFastTestExecutionScripts('negatedCommonSubclassEvent',1,600),
		writeFastTestExecutionScripts('negatedAntonymPattern1',1,600),
		writeFastTestExecutionScripts('negatedAntonymPattern2',1,600),
		writeFastTestExecutionScripts('negatedAntonymPattern3',1,600),
		writeFastTestExecutionScripts('negatedAgentRelation',1,600),
		writeFastTestExecutionScripts('negatedInstrumentRelation',1,600),
		writeFastTestExecutionScripts('negatedResultRelation',1,600),
		writeFastTestExecutionScripts('whiteBoxFalsityTest',1,600),
		writeFastTestExecutionScripts('whiteBoxTruthTest',1,600).

%%% writeFastTestExecutionScripts(in K, in P, in T)
%%%		K: test root key
%%%		P: number of parallel tests
%%%		T: execution-time limit per test
%%%		Action: scripts for the rapid execution of tests are sorted

writeFastTestExecutionScripts(K,P,T):-
		getAutomaticProver(AP),
		(
		 (
		  numberOfGoals(K,N),
		  !,
		  NP is ceiling(N/P),
		  number_chars(T,LC),
		  atom_chars(CT,LC),
		  writeFastAutomaticTestExecutionScripts(K,1,1,NP,CT,AP)
		 );
		 (
		  true
		 )
		).

%%% writeFastAutomaticTestExecutionScripts(in K, in P, in IT, in NT, in T, in AP)
%%%		K: test root key
%%%		P: script number
%%%		IT: number of initial test
%%%		NT: number of tests per script
%%%		T: execution-time limit per test
%%%		AP: the automatic prover to be used
%%%		Action: scripts for the rapid execution of tests are sorted

writeFastAutomaticTestExecutionScripts(K,_,IT,_,_,_):-
		numberOfGoals(K,N),
		IT > N,
		!.

% Vampire (from v8.1)

writeFastAutomaticTestExecutionScripts(K,P,IT,NT,T,vampire):-
		numberOfGoals(K,N),
		IT =< N,
		LT1 is IT + NT-1,
		(
		 (
		  LT1 < N,
		  !,
		  LT = LT1
		 );
		 (
		  LT = N
		 )
		),
		(
		 (
		  VV = '2.6'
		 );
		 (
		  VV = '3.0'
		 );
		 (
		  VV = '4.0'
		 );
		 (
		  VV = '4.1'
		 )
		),
		SF1 = 'cat Goals/',
		SF2 = '.adimen.tstp Axioms/adimen.sumo.tstp > Runs/',
		atom_concat('.adimen.sumo.vampire',VV,SF311),
		atom_concat(SF311,'.tstp\nvampire',SF31),
		atom_concat(SF31,VV,SF32),
		atom_concat(SF32,' --proof tptp --output_axiom_names on --mode casc -m 2048 -t ',SF3301),
		atom_concat(SF3301,T,SF3302),
		atom_concat(SF3302,' Runs/',SF330),
		atom_concat('.adimen.sumo.vampire',VV,SF41),
		atom_concat(SF41,'.tstp > Runs/output.',SF4),
		SF531 = '.adimen.sumo.vampire',
		atom_concat(SF531,VV,SF532),
		atom_concat(SF532,'.txt\nrm Runs/',SF530),
		atom_concat('.adimen.sumo.vampire',VV,SF61),
		atom_concat(SF61,'.tstp\n\n',SF6),
		findall(C,(between(IT,LT,I),getGoalKey(K,I,G),composeCommand16(G,SF1,SF2,SF330,SF4,SF530,SF6,C)),LC30),
		FN1 = 'Runs/runFast',
		atom_concat(FN1,K,FN2),
		atom_concat(FN2,'Tests',FN3),
		number_chars(IT,LC1),
		atom_chars(CIT,LC1),
		number_chars(LT,LC2),
		atom_chars(CLT,LC2),
		atom_concat(FN3,CIT,FN4),
		atom_concat(FN4,'-',FN5),
		atom_concat(FN5,CLT,FN6),
		atom_concat('Vampire',VV,VVV),
		atom_concat(VVV,'.sh',VVVSH),
		atom_concat(FN6,VVVSH,FN30),
		open(FN30,write,S30),
		writeListOfCommands(S30,LC30),
		writeAlertCommand(S30,VVV,'FastTests'),
		close(S30),
		NP is P+1,
		NIT is LT+1,
		VV = '4.1',
		writeFastAutomaticTestExecutionScripts(K,NP,NIT,NT,T,vampire).

% E prover (v2.0)

writeFastAutomaticTestExecutionScripts(K,P,IT,NT,T,eprover):-
		numberOfGoals(K,N),
		IT =< N,
		LT1 is IT + NT-1,
		(
		 (
		  LT1 < N,
		  !,
		  LT = LT1
		 );
		 (
		  LT = N
		 )
		),
		SF1 = 'cat Goals/',
		SF2 = '.adimen.tstp Axioms/adimen.sumo.tstp > Runs/',
		SF31 = '.adimen.sumo.eprover2.0.tstp\neprover2.0 --auto --memory-limit=2048 --cpu-limit=',
		atom_concat(SF31,T,SF32),
		atom_concat(SF32,' --print-pid --tstp-in --resources-info --proof-object -s Runs/',SF3),
		SF4 = '.adimen.sumo.eprover2.0.tstp > Runs/output.',
		SF5 = '.adimen.sumo.eprover2.0.txt\nrm Runs/',
		SF6 = '.adimen.sumo.eprover2.0.tstp\n\n',
		findall(C,(between(IT,LT,I),getGoalKey(K,I,G),composeCommand16(G,SF1,SF2,SF3,SF4,SF5,SF6,C)),LC),
		FN1 = 'Runs/runFast',
		atom_concat(FN1,K,FN2),
		atom_concat(FN2,'Tests',FN3),
		number_chars(IT,LC1),
		atom_chars(CIT,LC1),
		number_chars(LT,LC2),
		atom_chars(CLT,LC2),
		atom_concat(FN3,CIT,FN4),
		atom_concat(FN4,'-',FN5),
		atom_concat(FN5,CLT,FN6),
		atom_concat(FN6,'EProver2.0.sh',FN),
		open(FN,write,S),
		writeListOfCommands(S,LC),
		writeAlertCommand(S,'EProver2.0','FastTests'),
		close(S),
		NP is P+1,
		NIT is LT+1,
		writeFastAutomaticTestExecutionScripts(K,NP,NIT,NT,T,eprover).

% tptpSUMO

writeFastAutomaticTestExecutionScripts(K,P,IT,NT,T,tptpSUMO):-
		numberOfGoals(K,N),
		IT =< N,
		LT1 is IT + NT-1,
		(
		 (
		  LT1 < N,
		  !,
		  LT = LT1
		 );
		 (
		  LT = N
		 )
		),
		VV = '3.0',
		SF1 = 'cat Goals/',
		SF2 = '.adimen.tptpSUMO.tstp Axioms/CSR003+1-v5.3.0.ax > Runs/',
		SF31 = '.tptp.sumo.vampire.tstp\nvampire',
		atom_concat(SF31,VV,SF32),
		atom_concat(SF32,' --proof tptp --output_axiom_names on --mode casc -t ',SF3301),
		atom_concat(SF3301,T,SF3302),
		atom_concat(SF3302,' Runs/',SF330),
		SF4 = '.tptp.sumo.vampire.tstp > Runs/output.',
		SF531 = '.tptp.sumo.vampire',
		atom_concat(SF531,VV,SF532),
		atom_concat(SF532,'.txt\nrm Runs/',SF530),
		SF6 = '.tptp.sumo.vampire.tstp\n\n',
		findall(C,(between(IT,LT,I),getGoalKey(K,I,G),composeCommand16(G,SF1,SF2,SF330,SF4,SF530,SF6,C)),LC30),
		FN1 = 'Runs/runFast',
		atom_concat(FN1,K,FN2),
		atom_concat(FN2,'Tests',FN3),
		number_chars(IT,LC1),
		atom_chars(CIT,LC1),
		number_chars(LT,LC2),
		atom_chars(CLT,LC2),
		atom_concat(FN3,CIT,FN4),
		atom_concat(FN4,'-',FN5),
		atom_concat(FN5,CLT,FN6),
		VVV = 'tptpSUMO',
		atom_concat(VVV,'.sh',VVVSH),
		atom_concat(FN6,VVVSH,FN30),
		open(FN30,write,S30),
		writeListOfCommands(S30,LC30),
		writeAlertCommand(S30,VVV,'FastTests'),
		close(S30),
		NP is P+1,
		NIT is LT+1,
		writeFastAutomaticTestExecutionScripts(K,NP,NIT,NT,T,tptpSUMO).

%%% writeListOfCommands(in S, in LC)
%%%		S: output stream
%%%		LC: list of commands
%%%		Action: the list of commands is written in S

writeListOfCommands(_,[]).
writeListOfCommands(S,[C|LC]):-
		print_to_file(S,C),
		writeListOfCommands(S,LC).

%%% writeSourceCommand(in S)
%%%		S: output stream
%%%		Action: the command to obtain the source file is written in S

writeSourceCommand(S):-
		findall(F,(numberOfAxioms(F,_),F \= 'automaticCheck',F \= 'whiteBox',F \= 'orWhiteBox'),LF),
		print_to_file(S,'cat'),
		writeSourceCommandListOfFiles(S,LF),
		print_to_file(S,' > '),
		getOutputLanguage(OL),
		atom_concat('Axioms/adimen.sumo.',OL,TF),
		print_to_file(S,TF),
		print_to_file(S,'\n\n').

%%% writeSourceCommandListOfFiles(in S, in LF)
%%%		S: output stream
%%%		LF: list of files
%%%		Action: the command to obtain the source file is written in S

writeSourceCommandListOfFiles(_,[]).
writeSourceCommandListOfFiles(S,[F|LF]):-
		getOutputLanguage(OL),
		atom_concat('Axioms/',F,F1),
		atom_concat(F1,'.adimen.',F2),
		atom_concat(F2,OL,NF),
		print_to_file(S,' '),
		print_to_file(S,NF),
		writeSourceCommandListOfFiles(S,LF).

%%% writeAlertCommand(in S, in AP, in T)
%%%		S: output stream
%%%		AP: the automatic prover used
%%%		T: the kind of test performed
%%%		Action: the command to obtain the source file is written in S

writeAlertCommand(S,AP,T):-
		print_to_file(S,'php Runs/'),
		print_to_file(S,AP),
		print_to_file(S,'.'),
		print_to_file(S,T),
		print_to_file(S,'.mail.php'),
		print_to_file(S,'\n').

%%% composeCommand16(in G, in SF1, in SF2, in SF3, in SF4, in SF5, in SF6, out C)
%%%		G: goal name
%%%		SF1, SF2, SF3, SF4, SF5, SF6: command portions
%%%		C: command

composeCommand16(G,SF1,SF2,SF3,SF4,SF5,SF6,C):-
		atom_concat(SF1,G,SFG1),
		atom_concat(SF2,G,SFG2),
		atom_concat(SF3,G,SFG3),
		atom_concat(SF4,G,SFG4),
		atom_concat(G,SF6,SFG6),
		atom_concat(SFG1,SFG2,SC1),
		atom_concat(SC1,SFG3,SC2),
		atom_concat(SC2,SFG4,SC3),
		atom_concat(SC3,SF5,SC4),
		atom_concat(SC4,SFG6,C).
